/********************************************************************
   TORNADO-PX/DDC4 DCM Demo Utility. rev.1A
   FIFO test using ADC-1 input data stream.
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo contains test for FIFO acquisition procedure and
	performs the following operations:
	  - program FIFO PAF/PAE flags
	  - configure FIFO to acquire ADC-1 data using FIFO PAF
	    termination flag at the sampling rate 65MHz, which is
	    defined by the PFG-1. The recommended period of external
	    analog input signal at ADC-1 input must be 1MHz.
	  - read FIFO data and check for correct number of words
	    written and correct functionality of FIFO PAF/EF/PAE flags
	  - FIFO data are transferred into DPRAM and can be veiwed
	    via Code Composer signal viewer in time domain (only
	    first 0x7800 samples of FIFO data are tranfreed into DPRAM).
	  - This demo must be loaded into DSP on-chip RAM and requires
	    OVLY bit of C54x PMST reg to be set to '1' when loading program
		via emulator.



   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          	- SA-mode
	  		- no BOOT mode
***********************************************************************/

#include	"math.h"
#include	"ddc4_dsp.h"
#include	"ddc4_dx.h"



void init_cpu()
 {
	TPXDDC4_init_cpu();
 }

void init_sio()
{
		// init McBSP-1 port
	TPXDDC4_init_mcbsp1();

}


	// this function gnerates the delay abot 500uS,
	// which is required to lock frequency by PFG
void pfg_setup_delay()
 { unsigned i;

	for(i=0;i<20000;i++);
 }



	// set PFG-1 to 65MHz = (10MHz*(N=39)*(V=1)/(M=3)*(R=2)*(X=1))
	// (N=39, M=3, V=0 (1), X=0 (1), R=1 (2))
#define PFG1_N_DATA       39
#define PFG1_M_DATA       3
#define PFG1_V_DATA       0
#define PFG1_X_DATA       0
#define PFG1_R_DATA       1



	// FIFO PAE/PAF flags offset
#define	_FIFO_PAE_OFFSET	0x04000L
#define	_FIFO_PAF_OFFSET	0x08000L


#define	_DPRAM_DATA_LEN		0x7800L

unsigned rd;

void main()
{
  	unsigned long i;
  	int *mptr, fifo_data;
  	int n;


	init_cpu();
	init_sio();

  	// configure PFG-1 and wait until
	// output frequency will be locked
	TPXDDC4_write_pfg(TPXDDC4_PFG1_SEL,PFG1_N_DATA,PFG1_M_DATA,PFG1_V_DATA,PFG1_X_DATA,PFG1_R_DATA);
	pfg_setup_delay();


	// configure ADC-1/2 CLK selectors.
	// ADC-1 CLK is set to PFG-1 clock
	// ADC-2 CLK is set PFG-1
    n=0;
    TPXDDC4_set_adc_clk_sel(  	TPXDDC4_ADC1_CLK_SEL_PFG1
							| 	TPXDDC4_ADC2_CLK_SEL_PFG1
							| 	TPXDDC4_PFG1_AUX_FDIV_OFF
							| 	TPXDDC4_PFG2_AUX_FDIV_OFF
			       		   );

	if((rd=TPXDDC4_adc_clk_sel())!= (  		TPXDDC4_ADC1_CLK_SEL_PFG1
					 					| 	TPXDDC4_ADC2_CLK_SEL_PFG1
					 					| 	TPXDDC4_PFG1_AUX_FDIV_OFF
					 					| 	TPXDDC4_PFG2_AUX_FDIV_OFF
			       	     			)
	  )
    	goto error;



	//--- perform FIFO fill-in with the ADC-1 data
	//    with termination on active PAF condition

	// configure PDC/FIFO Input Selectors
	// PDC-1/2/3/4 are configured to ADC-2 output
	// FIFO is configured to ADC-1 output
    n=1;
    TPXDDC4_set_pdc_fifo_isel(  	TPXDDC4_PDC12_ISEL_ADC2
				  				| 	TPXDDC4_PDC34_ISEL_ADC2
				  				| 	TPXDDC4_FIFO_ISEL_ADC1
				 			);

    if((rd=TPXDDC4_pdc_fifo_isel())!= (  	TPXDDC4_PDC12_ISEL_ADC2
				  	   					| 	TPXDDC4_PDC34_ISEL_ADC2
				  	   					| 	TPXDDC4_FIFO_ISEL_ADC1
				          			  )
      )
      	goto error;

	// now reset FIFO and program FIFO PAF/PAE flags
    n=2;
	TPXDDC4_SET_FIFO_RESET;
	rd=TPXDDC4_set_fifo_flag_offset(_FIFO_PAE_OFFSET,_FIFO_PAF_OFFSET);

	if(   	rd!=0
        || 	TPXDDC4_fifo_ef()  ==0
	   	|| 	TPXDDC4_fifo_pae() ==0
	   	|| 	TPXDDC4_fifo_ff()  !=0
	   	|| 	TPXDDC4_fifo_paf() !=0
      )
      	goto error;

	// set FIFO DAQ mode with PAF termination flag
	// and initialize DAQ procedure
    TPXDDC4_SET_FIFO_MODE_DAQ;
	TPXDDC4_SET_FIFO_DAQTF_SEL_PAF;

	while(1)
	{
		TPXDDC4_SET_FIFO_RESET;         // perform FIFO reset and rest of all DAQ control
        TPXDDC4_SET_FIFO_DAQ_RUN;

        n=3;
		if(   	TPXDDC4_fifo_mode()      != TPXDDC4_FIFO_MODE_DAQ
	   		|| 	TPXDDC4_fifo_daq_run()   != TPXDDC4_FIFO_DAQ_RUN
	   		|| 	TPXDDC4_fifo_daqtf_sel() != TPXDDC4_FIFO_DAQTF_SEL_PAF
	   		|| 	TPXDDC4_fifo_daq_end()   != 0
	  	  )
		  	goto error;


		// wait until DAQ terminates
		while(TPXDDC4_fifo_daq_end()==0);

		// now read FIFO data, store it in DPRAM,
		// and verify number of written FIFO samples.
        n=4;
		mptr = (int *) TPXDDC4_DPRAM_SADDR;

		for(i=0;i<(_FIFO_PAF_OFFSET); i++)
	  	{

			// check for FIFO PAF/EF/PAE flags function perfectly
	    	if(   	TPXDDC4_fifo_ef()!=0
               	|| (   	i>=(_FIFO_PAF_OFFSET-_FIFO_PAE_OFFSET+1L)
	       	   		&& 	TPXDDC4_fifo_pae()==0
                   )
				// !!!! PAF flag is cleared after the 2nd read cycle
	       		|| (i>2 && TPXDDC4_fifo_paf()!=0)
	       		|| TPXDDC4_fifo_ff()!=0
	      	  )
               	goto error;

			// read and move FIFO data to DPRAM with
			// 16-bit sign extension.
			// (only 1st _DPRAM_DATA_LEN FIFO samples
			//  are moved to DPRAM, since DPRAM has
			//  smaller capacity than FIFO).
			// DPRAM data can veiwed via Code Composer STudio
			// graphic data viewer in time domain.
	    	fifo_data = TPXDDC4_fifo();
            if(i<_DPRAM_DATA_LEN) *mptr++ = ((fifo_data <<4) >>4);
	  	}

		// ensure that EF/PAE are now both set to 'ON'
		// and PAF/FF is set to OFF
		n=5;
		if(   TPXDDC4_fifo_ef()==0
           || TPXDDC4_fifo_pae()==0
           || TPXDDC4_fifo_paf()!=0
           || TPXDDC4_fifo_ff()!=0
          )
		  	goto error;

		// get here on successful execution
		
		n=7;	// <- set the breakpint here anc run the animation mode of the CCS
				// Open the SIGNAL GRAPHICS (TIME DOMAIN) window in CCS at 0x8000 address 
				// in order to view the acquired signal.
	}

error:
	// trap here on any error
    while(1);
}

