/********************************************************************
   TPRNADO-PX/DDC4 DCM Demo Utility.
   PDC/FIFO input selector demo. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo dynamically switches between different sources for
	PDC-1/2, PDC-3/4 and FIFO as the following:
	  - ADC-1
	  - ADC-2
	Selection is performed dynamically with approx 1sec interval.
	PFG-1/2 are configured for 3.1MHz and 15MHz correspondingly.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - SA-mode
	  - no BOOT mode
***********************************************************************/

#include	"math.h"
#include	"c54x_def.h"
#include	"ddc4_dsp.h"
#include	"ddc4_dx.h"



void init_cpu()
 {
	TPXDDC4_init_cpu();
 }

void init_sio()
{
		// init McBSP-1 port
	TPXDDC4_init_mcbsp1();

}

	// set PFG-1 = (100/4*8)=3.1MHz
	// (N=30, M=3, V=0 (1), X=2, R=3)
#define PFG1_N_DATA       30
#define PFG1_M_DATA       3
#define PFG1_V_DATA       0
#define PFG1_X_DATA       2
#define PFG1_R_DATA       3

	// set PFG-2 = (60/1*4)=15MHz
	// (N=24, M=4, V=0 (1), X=0, R=2)
#define PFG2_N_DATA       24
#define PFG2_M_DATA       4
#define PFG2_V_DATA       0
#define PFG2_X_DATA       0
#define PFG2_R_DATA       2




#define	_CLEAR_TM_EVENT		*C54X_IFR=8
#define	_WAIT_TM_EVENT		while(!(*C54X_IFR & 8))

// generates pause approx 1sec	
pause()
{ int i;

		//--- set 0.5kHz timer output frequency
	*C54X_TM_TCR = (TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410) ? 0x0019:0x001F;
	*C54X_TM_PRD = 20000;
	*C54X_TM_TCR = (TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410) ? 0x0009:0x000F;	// start timer

        	//--- clear TIMER interrupt request
   _CLEAR_TM_EVENT;


   for(i=0;i<1000;i++)
     {
	  _WAIT_TM_EVENT;
	  _CLEAR_TM_EVENT;
     }

}


unsigned rd,n;

void main()
{

	init_cpu();
	init_sio();

	  	// configure PFG-1
	TPXDDC4_write_pfg(TPXDDC4_PFG1_SEL,
			  PFG1_N_DATA,
                          PFG1_M_DATA,
                          PFG1_V_DATA,
  	  	    	  PFG1_X_DATA,
                          PFG1_R_DATA);

		// configure PFG-2
	TPXDDC4_write_pfg(TPXDDC4_PFG2_SEL,
			  PFG2_N_DATA,
			  PFG2_M_DATA,
			  PFG2_V_DATA,
  	  	    	  PFG2_X_DATA,
			  PFG2_R_DATA );


		// configure ADC-1/2 CLK selectors.
		// ADC-1 CLK is set to PFG-1 clock
		// ADC-2 CLK is set PFG-2.
        n=0;
        TPXDDC4_set_adc_clk_sel(  TPXDDC4_ADC1_CLK_SEL_PFG1
				| TPXDDC4_ADC2_CLK_SEL_PFG2
				| TPXDDC4_PFG1_AUX_FDIV_OFF
				| TPXDDC4_PFG2_AUX_FDIV_OFF
			       );

	if((rd=TPXDDC4_adc_clk_sel())!= (  TPXDDC4_ADC1_CLK_SEL_PFG1
					 | TPXDDC4_ADC2_CLK_SEL_PFG2
					 | TPXDDC4_PFG1_AUX_FDIV_OFF
					 | TPXDDC4_PFG2_AUX_FDIV_OFF
			       	     )
	    ) goto error;


        n=1;
        TPXDDC4_set_pdc_fifo_isel(0);	// clear PDC/FIFO Input Selectors
        if((rd=TPXDDC4_pdc_fifo_isel())!=0) goto error;

	while (1)
	 {

		//--- perform step-by-step config for PFG-1/2 input

          n=2;
	  TPXDDC4_set_pdc12_isel(TPXDDC4_PDC_FIFO_ISEL_ADC1);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;
          pause();	



          n=3;
	  TPXDDC4_set_pdc12_isel(TPXDDC4_PDC_FIFO_ISEL_ADC2);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC2
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;
          pause();	




          n=4;
	  TPXDDC4_set_pdc12_isel(TPXDDC4_PDC_FIFO_ISEL_ADC1);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;




		//--- perform step-by-step config for PFG-3/4 input

          n=5;
	  TPXDDC4_set_pdc34_isel(TPXDDC4_PDC_FIFO_ISEL_ADC1);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;
          pause();	



          n=6;
	  TPXDDC4_set_pdc34_isel(TPXDDC4_PDC_FIFO_ISEL_ADC2);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC2
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;
          pause();	




          n=7;
	  TPXDDC4_set_pdc34_isel(TPXDDC4_PDC_FIFO_ISEL_ADC1);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;




		//--- perform step-by-step config for FIFO input

          n=8;
	  TPXDDC4_set_fifo_isel(TPXDDC4_PDC_FIFO_ISEL_ADC1);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;
          pause();	



          n=9;
	  TPXDDC4_set_fifo_isel(TPXDDC4_PDC_FIFO_ISEL_ADC2);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC2
	  				  )
          ) goto error;
          pause();	




          n=10;
	  TPXDDC4_set_fifo_isel(TPXDDC4_PDC_FIFO_ISEL_ADC1);
	  if((rd=TPXDDC4_pdc_fifo_isel())!= (  TPXDDC4_PDC12_ISEL_ADC1
	    				   | TPXDDC4_PDC34_ISEL_ADC1
					   | TPXDDC4_FIFO_ISEL_ADC1
	  				  )
          ) goto error;

          pause();	

	 }


    error:              // trap here on any error
    	while(1);	

}

