/********************************************************************
   TORNADO-PX/DDC4 DCM Demo Utility.
   FLASH test utility. rev.1B
   (C) MicroLAB Systems, 2000-2003


   Description:
   ------------
	This demo tests on-board FLASH memory.
	FLASH memory capacity tested is set upon the DDC4 board revision ID and
	is 128Kx8 for DDC4 rev.1A and 512Kx8 for DDC$ rev.1B
	If error occurs, then DIO-0/1 are set to 1/0.
	In case of success DIO-0/1 are set to 0/1.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - SA-mode
	  - no BOOT mode
***********************************************************************/

#include	"math.h"
#include	"c54x_def.h"
#include	"ddc4_dsp.h"	// include general DDC4 C54x defs
#include	"ddc4_dx.h"	// include general DDC4 extension functions



void set_dio_output(d0,d1)
{
	TPXDDC4_set_gpio0_data(d0 & 0x1);
	TPXDDC4_set_gpio1_data(d1 & 0x1);
}


void init_cpu()
{
	TPXDDC4_init_cpu();
}

#define		FLASH_TEST_CYCLES_128K	11
#define		FLASH_TEST_CYCLES_512K	17

unsigned	FLASH_TEST_CYCLES;
#define 	FLASH_CYCLE_LEN			0x100L

unsigned long FLASH_SADDR[FLASH_TEST_CYCLES_512K] 	=
		{ 0x00000L, 0x01000L,	// page #0
          0x02000L, 0x04000L,	// page #0
          0x08000L, 0x08800L,	// page #1
          0x10000L, 0x10400L,	// page #2/#1
          0x18000L, 0x18200L,	// page #3/#1
          0x18100L,				// page #3/#1
          0x20000L,				// page #2
          0x30000L,				// page #3
          0x40000L,				// page #4
          0x50000L,				// page #5
          0x60000L,				// page #6
          0x70000L				// page #7
	    };

unsigned FLASH_SDATA[FLASH_TEST_CYCLES_512K]	=
		{ 0x00, 0xe9,
		  0x3f, 0x55,
		  0x25, 0xe3,
		  0x8c, 0xa1,
		  0x66, 0x99,
		  0xbf,
		  0xfa,
		  0x15,
		  0x79,
		  0x59,
		  0xc1,
		  0x9f
        };



void main()
{  int i,j;
   unsigned d;
   unsigned long a;

	init_cpu();
	TPXDDC4_set_gpio_dir( TPXDDC4_GPIO0_DIR_OUTPUT | TPXDDC4_GPIO1_DIR_OUTPUT);

   	// set DIO-0/1 first to 0/0, then to '1/1',
	// and then to '0/0' again in order to synchronize start
	set_dio_output(0,0);
	set_dio_output(1,1);
	set_dio_output(0,0);

	// set number of test cycles upon the FLASH capacity
	FLASH_TEST_CYCLES = ((TPXDDC4_REV_ID_IS_REV1B)? FLASH_TEST_CYCLES_512K:FLASH_TEST_CYCLES_128K);

	// erase the chip with verification
	if(TPXDDC4_flash_erase_chip(TPXDDC4_ON)!=0) goto error;

	// now write shifted RAMPS to all predefied FLASH areas
    for(i=0;i<FLASH_TEST_CYCLES;i++)
    {
		for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
            j<FLASH_CYCLE_LEN;
            a++,j++,d++)
        {
		 	if(TPXDDC4_flash_write(a,d)!=0)
			 	goto error;
	    }
	}

	// now verify written data
    for(i=0;i<FLASH_TEST_CYCLES;i++)
    {
    	for(a=FLASH_SADDR[i], j=0, d=FLASH_SDATA[i];
        	j<FLASH_CYCLE_LEN;
            a++,j++,d++)
       	{
		 	if(TPXDDC4_flash_read(a)!=(d & TPXDDC4_FLASH_DATA_BITMASK))
			 	goto error;
	    }
	}

terminate_ok:
	// TRAP here in case programming & verification is OK
	// set DIO-0/1 to '0/1'
   	set_dio_output(0,1);
   	while(1);

error:
	// TRAP here in case of any error
	// set DIO-0/1 to '1/0'
   	set_dio_output(1,0);
   	while(1);


}

