/********************************************************************
   TORNADO-PX/DDC4 Coprocessor Demo Utility.
   FLASH bootloader test: DPRAM test kernel with communication via UART ch #1. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo continuously tests DPRAM and displays test results via 
	UART channel #1.

	External 9600 baud ASYNC terminal with RS232C interface or 
	PC COM port running in terminal mode at 9600 bauds shall be 
	used with this demo.


   DSP Bootmode
   ------------
	This demo can be either loaded into DSP on-chip RAM via JTAG emulator
	or can be burned into on-board FLASH memory for further 8-bit FLASH boot.

	When loading directly to DSP on-chip memory, the DSP OVLY bit must be 
	set to '1'.

	In case this demo is considered to be as the boot code, then its code 
	must be programmed (burned-in) into on-board FLASH memory. Refer to
	FL_BURN.C utility for details how to program on-board FLASH memory.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	baud rate: 		9600
	word length:	8 bits
	stop bits: 		2

    on-board configuration switch:
      - SA-mode
	  - no BOOT mode

***********************************************************************/

#include	<math.h>
#include	"c54x_def.h"
#include	"ddc4_dsp.h"
#include	"ddc4_dx.h"


char welcome_message[]	= "TORNADO-PX/DDC4 Stand-alone FLASH-bootloader/DPRAM/UART#1 test\n\n>>";
char dsp_c5410_message[]= "DSP type: C5410 100MHz";
char dsp_c5416_message[]= "DSP type: C5416 160MHz";
char ok_message[]		= " OK ";
char err_message[]		= " ERROR ";


uart_transmit_char(c)
int c;
{
  while(!(TPXDDC4_UART1_LSTAT_RG & TPXDDC4_UART_LSTAT_THEMPY));
  TPXDDC4_UART1_TDATA_RG = c;
}


unsigned uart_receive_char()
{   unsigned d;

  do
    { d = TPXDDC4_UART1_LSTAT_RG;
    }
     while(!(d & TPXDDC4_UART_LSTAT_DR));

  return(TPXDDC4_UART1_RDATA_RG & 0xff);
}


int uart_receiver_status()
{
  return(TPXDDC4_UART1_LSTAT_RG & TPXDDC4_UART_LSTAT_DR);
}


void init_uart()
 {
		// configure UART for 9600/8/N/2

	   TPXDDC4_UART1_LCNTR_RG =   TPXDDC4_UART_LCNTR_DLAB
				    | TPXDDC4_UART_LCNTR_WLEN8
		     		    | TPXDDC4_UART_LCNTR_STOP2 ;

	   TPXDDC4_UART1_DIVL_RG  = TPXDDC4_UART_DIVL_9600;
	   TPXDDC4_UART1_DIVH_RG  = TPXDDC4_UART_DIVH_9600;

	   TPXDDC4_UART1_LCNTR_RG =   TPXDDC4_UART_LCNTR_WLEN8
		                    | TPXDDC4_UART_LCNTR_STOP2 ;

	   TPXDDC4_UART1_MCNTR_RG = 0;
	   TPXDDC4_UART1_IE_RG    = TPXDDC4_UART_IE_DR;
 }


void uart_transmit_mess(s)
char *s;
{
	  while((*s &0xff)!=0) uart_transmit_char(*(s++));
}



// generates quasi-random next memory data
unsigned next_cell(i)
{
  return( (((~i ^ 0x93a5 -1) <<8)  & 0xff00)
	 |((i+1) & 0xff)  
	);
}


#define ML_OK		0
#define ML_MEM_ERR	-1


// memory test procedure
int mem_test(init_value, sa, ea)
unsigned init_value, *sa, *ea;

{ int r;
  unsigned *ptr,i;

	r=ML_MEM_ERR;

	// fill DPRAM
      i=init_value;	
  	  for(ptr=sa; ptr<=ea; ptr++)
	  { *ptr =i;
	    i=next_cell(i);
	  } ;

	// verify written DPRAM data
      i=init_value;	
      for(ptr=sa; ptr<=ea; ptr++)
	  { if ((*ptr)!=i) return (r);
	    i=next_cell(i);
	  }

      // test is OK
      return(ML_OK);
}







void main()
{
	unsigned iv, *start, *end;

	// init DSP
	TPXDDC4_init_cpu();
	
	// disable FLASH and enable DPRAM for run0time test
	TPXDDC4_set_xdm_sel(TPXDDC4_XDM_SEL_DPRAM);

	// initialize UART channel #1 and transmit welcome message
	init_uart();
    uart_transmit_mess(welcome_message);

	// ident DSP type and display the corresponding message	
	if(TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410)
		uart_transmit_mess(dsp_c5410_message);
	else
		uart_transmit_mess(dsp_c5416_message);

	// now prepare for running continuous DPRAM test with
	// results display via UART#1 serial interface
	start	= (unsigned *) 0x8000;
	end		= (unsigned *) 0xff7f;
	iv  	= 0x3e91;

	// continuous memory test loop
    while (1)
	 {
		switch(mem_test(iv,start,end))
		  {
		    case ML_OK:
		        uart_transmit_mess(ok_message);
				break;

		    default:
		        uart_transmit_mess(err_message);
				goto error;
		  }


		iv =(  ((((iv+1)^0x935f)>>8) & 0x00ff)
			 | ((((iv-1)^0xa75a)<<8) & 0xff00)  
			);
	 }


	// trap here on memory test error
  error: 
	while(1);

}

