/********************************************************************
   TORNADO-PX/DDC4 DCM Demo Utility. rev.1A
   Host-to-DSP communication test for T-PX/DDC4 and TORNADO-3x/E3x/P3x.
   (C) MicroLAB Systems, 2000


   file:
   -----
   	Host TORNADO-3x/E3x/P3x driver.

   Description:
   ------------
	This demo performs communication test between DSP of host
	TORNADO-3x/E3x/P3x mainboard and DSP of T-PX/DDC4 DCM:
	 - test for communication via DPRAM
	 - test for communication via DPSEM
	 - test for DSP-to-Host Interrupt communication
	 - test for communication via HPI

   Host TORNADO platform:
   ----------------------
	This demo utilize host TORNADO resources and runs in HOST mode.
	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board.

	This demo utilize T-PX/DDC4 on-board DSP resources. MIRAGE
	emulator is required to upload code to the DSP of T-PX/DDC4.

	TI CCS tools are required to recompile this demo.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - HOST-mode
	  - DCM is installed onto TORNADO-3x/E3x/P3x mainboard
***********************************************************************/


#include <c3x_def.h>
#include <t3x_dsp.h>
#include <ddc4_h3x.h>


init_cpu()
{
   *C3X_PBUS_GC=T3X_PBUS_GC_DATA;
}



unsigned START_SADDR = 0x0L,
         LAST_SADDR  = 0x7ffL;

unsigned sa;
unsigned *D,data,data_start, data_rd;


#define	MCD_ADDR		(0x7f00L)	// MCD address in DPRAM

#define	M_ID			0x110
#define	PRG_ID			0x111

#define	MCMND_NOP		0
#define	MCMND_SRAM_COMM		1
#define	MCMND_IRQ_COMM		2
#define	MCMND_DPSEM_COMM	3
#define MCMND_HPI_HINT_COMM	5

#define	MSTAT_BUSY		2
#define	MSTAT_ATTN		1
#define	MSTAT_RDY		0

#define	ERR_OK			0
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1


// this Host-to-DSP communication descriptor is
// used to transmit command to DSP
struct MCD_DD
 { int  	m_id,
		prg_id,
		cmnd,
		status,
		error;

   unsigned     up[4];
 };

struct MCD_DD *MCD;




//---- these functions return stutus of the IRQ flags for XIRQ-0/1/2
int _xirq0_data()
{
   asm(" ldi if,r0");
   asm(" and 1,r0");
}

int _xirq1_data()
{
   asm(" ldi if,r0");
   asm(" and 2,r0");
}

int _xirq2_data()
{
   asm(" ldi if,r0");
   asm(" and 4,r0");
}



int n=0;	// is used for tracking error conditions


main()
{ unsigned d,i;


   init_cpu();

	// set T-PX/DDC4 DSP to the RESET state
   n=10;
   HTPXDDC4_SET_MRESET;
   if(HTPXDDC4_MRESET_DATA!=HTPXDDC4_MRESET_RESET) goto error;

	// set no-boot and enable DPRAM/HHPI error control
   n=11;
   HTPXDDC4_SET_CNTR2_RG(  HTPXDDC4_BMODE_NONE
   			 | HTPXDDC4_DPRAM_ERR_EN_ON
			 | HTPXDDC4_HPI_ERR_EN_ON
                        );
   HTPXDDC4_CLR_DPRAM_ERR;
   HTPXDDC4_CLR_HPI_ERR;
   HTPXDDC4_HPI_INIT;

	// read M->H IRQ and clear interrupt request
   d=HTPXDDC4_DPRAM_MH_RQ_DATA;
   HTPXDDC4_SET_DPRAM_MH_RQ(0);

	// now verify that PIOX I/F data
	// is initialized correctly
   n=12;
   if(   HTPXDDC4_CNTR2_RG_DATA!= (  HTPXDDC4_BMODE_NONE
   			 	   | HTPXDDC4_DPRAM_ERR_EN_ON
			 	   | HTPXDDC4_HPI_ERR_EN_ON
                                  )
      || HTPXDDC4_DPRAM_ERR_DATA!=0
      || HTPXDDC4_HPI_ERR_DATA!=0
      || HTPXDDC4_HINT_IRQ_DATA!=0
      || HTPXDDC4_DPRAM_IRQ_DATA!=0
     ) goto error;



	// disable all host IRQs
   n=13;
   HTPXDDC4_SET_IE_RG(0);	
   if(HTPXDDC4_IE_RG_DATA!=0) goto error;


	// disable host IRQ selector
   n=14;
   HTPXDDC4_SET_HIRQ_SEL_RG(0);	
   if(HTPXDDC4_HIRQ_SEL_RG_DATA!=0) goto error;


	// clear XIRQs and verify
   n=15;
   C3X_XINT_CLEAR(0);
   C3X_XINT_CLEAR(1);
   C3X_XINT_CLEAR(2);

   if(   _xirq0_data()!=0
      || _xirq1_data()!=0
      || _xirq2_data()!=0
     ) goto error;


	// clear DPSEMs
   for(i=0;i<8;i++) HTPXDDC4_SET_DPSEM(n,1);



	// assign MCD pointer
   MCD = (struct MCD_DD *) (HTPXDDC4_DPRAM_SADDR + MCD_ADDR);


   	//------- testing DSP initialization ...
   MCD->m_id	= 0xffff;
   MCD->prg_id	= 0xffff;
   MCD->cmnd	= 0xffff;
   MCD->status	= 0xffff;
   MCD->error	= 0x7fff;
   MCD->up[0]	= 0xffff;
   MCD->up[1]	= 0xffff;

   HTPXDDC4_SET_MGO;

   n=16;
	// wait for DSP will init
   while ((MCD->status & HTPXDDC4_DPRAM_DATA_BITMASK)!=MSTAT_RDY)
     {
      	if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;
     }

   n=17;
	// validate MCD returned
   if(	 (MCD->m_id   & HTPXDDC4_DPRAM_DATA_BITMASK) !=M_ID      	
      || (MCD->prg_id & HTPXDDC4_DPRAM_DATA_BITMASK) !=PRG_ID 	
      || (MCD->cmnd   & HTPXDDC4_DPRAM_DATA_BITMASK) !=MCMND_NOP
      || (MCD->status & HTPXDDC4_DPRAM_DATA_BITMASK) !=MSTAT_RDY
      || (MCD->error  & HTPXDDC4_DPRAM_DATA_BITMASK) !=ERR_OK
      || (MCD->up[0]  & HTPXDDC4_DPRAM_DATA_BITMASK) !=0x5555
      || (MCD->up[1]  & HTPXDDC4_DPRAM_DATA_BITMASK) !=0xaaaa
     )
	     goto error;



	//------- testing communication via DPRAM ...
   n=20;

   MCD->cmnd=MCMND_SRAM_COMM;
   MCD->error=0x7fed;
   MCD->up[0]=0xffff;
   MCD->up[1]=0xffff;
   MCD->up[2]=0xffff;
   MCD->up[3]=0xffff;
   MCD->status=MSTAT_ATTN;

   n=21;
	// wait for DSP will init
   while ((MCD->status & HTPXDDC4_DPRAM_DATA_BITMASK)!=MSTAT_RDY)
     {
      	if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;
     }

   n=22;
   if(   (MCD->error & HTPXDDC4_DPRAM_DATA_BITMASK)!=ERR_OK
      || (MCD->up[0] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0xfedc
      || (MCD->up[1] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0xba98
      || (MCD->up[2] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x7654
      || (MCD->up[3] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x3210
     )
	       goto error;



   	//------ testing HPI/HINT ...
   n=30;
   if(HTPXDDC4_HINT_DATA!=0)	goto error;


		// enable XIRQ-1 linked to HINT
   n=31;	
   HTPXDDC4_SET_IE_RG(HTPXDDC4_HINT_IE_ON);
   HTPXDDC4_SET_HIRQ_EN_ON;
   HTPXDDC4_SET_HIRQ_SEL(HTPXDDC4_HIRQ1_SEL);

   if(   _xirq1_data()!=0
      || HTPXDDC4_HINT_IE_DATA==0
      || HTPXDDC4_HINT_DATA!=0
      || HTPXDDC4_HIRQ_EN_DATA==0
      || HTPXDDC4_HIRQ_SEL_DATA!=HTPXDDC4_HIRQ1_SEL
     ) goto error;


   MCD->cmnd=MCMND_HPI_HINT_COMM;
   MCD->error=0x7fed;
   MCD->status=MSTAT_ATTN;

   n=32;
   while ((MCD->status & HTPXDDC4_DPRAM_DATA_BITMASK)!=MSTAT_RDY)
     {
      	if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;
     }

   n=33;
   if((MCD->error & HTPXDDC4_DPRAM_DATA_BITMASK)!=ERR_OK) goto error;

   n=34;
   if(HTPXDDC4_HINT_DATA==0)  goto error;

   n=35;
   if(   _xirq1_data()==0
      || _xirq0_data()!=0
      || _xirq2_data()!=0
     )	
       goto  error;
   C3X_XINT_CLEAR(1);


   n=36;
   HTPXDDC4_CLR_HPIC_HINT;
   if(HTPXDDC4_HPIC_HINT_DATA!=0) goto error;



	//---- testing communication via HM_RQ/MH_RQ ...

	// enable XIRQ0 linked to DPRAM IRQ
   HTPXDDC4_SET_IE_RG(HTPXDDC4_DPRAM_IE_ON);
   HTPXDDC4_HIRQ_EN_DATA;


		//>>> first test communication via XIRQ0
   n=40;
   HTPXDDC4_SET_HIRQ_SEL(HTPXDDC4_HIRQ0_SEL);

   if(   _xirq0_data()!=0
      || HTPXDDC4_DPRAM_IE_DATA==0
      || HTPXDDC4_DPRAM_IRQ_DATA!=0
      || (HTPXDDC4_DPRAM_MH_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=0
      || HTPXDDC4_HIRQ_EN_DATA==0
      || HTPXDDC4_HIRQ_SEL_DATA!=HTPXDDC4_HIRQ0_SEL
     ) goto error;


   MCD->cmnd=MCMND_IRQ_COMM;
   MCD->error=0x7fed;
   MCD->up[0]=0xffff;
   MCD->up[1]=0xffff;
   MCD->up[2]=0xffff;
   MCD->up[3]=0xffff;

   n=41;
   if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;

	// release H->M IRQ
   HTPXDDC4_SET_DPRAM_HM_RQ(0x3974);


	// wait for returned IRQ
   while (_xirq0_data()==0) ;


   C3X_XINT_CLEAR(0);

   n=42;
   if(   (MCD->error & HTPXDDC4_DPRAM_DATA_BITMASK)!=ERR_OK
      || (MCD->up[0] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x0123
      || (MCD->up[1] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x4567
      || (MCD->up[2] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x89ab
      || (MCD->up[3] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0xcdef
      || (HTPXDDC4_DPRAM_MH_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=(~0x3974 & HTPXDDC4_DPRAM_DATA_BITMASK)
      || (HTPXDDC4_DPRAM_HM_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=0
      || HTPXDDC4_DPRAM_IRQ_DATA!=0
      || _xirq0_data()!=0
      || _xirq1_data()!=0
      || _xirq2_data()!=0
     )
	goto  error;

   n=43;
   HTPXDDC4_SET_DPRAM_MH_RQ(0);
   if(   HTPXDDC4_DPRAM_ERR_DATA!=0
      || (HTPXDDC4_DPRAM_MH_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=0
      ) goto error;



		//>>> now test communication via XIRQ2
   n=44;
   HTPXDDC4_SET_HIRQ_SEL(HTPXDDC4_HIRQ2_SEL);

   if(   _xirq2_data()!=0
      || HTPXDDC4_DPRAM_IE_DATA==0
      || HTPXDDC4_DPRAM_IRQ_DATA!=0
      || (HTPXDDC4_DPRAM_HM_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=0
      || (HTPXDDC4_DPRAM_MH_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=0
      || HTPXDDC4_HIRQ_EN_DATA==0
      || HTPXDDC4_HIRQ_SEL_DATA!=HTPXDDC4_HIRQ2_SEL
     ) goto error;


   MCD->cmnd=MCMND_IRQ_COMM;
   MCD->error=0xffff;
   MCD->up[0]=0xffff;
   MCD->up[1]=0xffff;
   MCD->up[2]=0xffff;
   MCD->up[3]=0xffff;

   n=45;
   if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;

	// release H->M IRQ
   HTPXDDC4_SET_DPRAM_HM_RQ(0xef01);


	// wait for returned IRQ
   while (_xirq2_data()==0) ;


   C3X_XINT_CLEAR(2);

   n=46;
   if(   (MCD->error & HTPXDDC4_DPRAM_DATA_BITMASK)!=ERR_OK
      || (MCD->up[0] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x0123
      || (MCD->up[1] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x4567
      || (MCD->up[2] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x89ab
      || (MCD->up[3] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0xcdef
      || (HTPXDDC4_DPRAM_MH_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=(~0xef01 & HTPXDDC4_DPRAM_DATA_BITMASK)
      || (HTPXDDC4_DPRAM_HM_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=0
      || HTPXDDC4_DPRAM_IRQ_DATA!=0
      || _xirq0_data()!=0
      || _xirq1_data()!=0
      || _xirq2_data()!=0
     )
	goto  error;

   n=47;
   HTPXDDC4_SET_DPRAM_MH_RQ(0);
   if(   HTPXDDC4_DPRAM_ERR_DATA!=0
      || (HTPXDDC4_DPRAM_MH_RQ_DATA & HTPXDDC4_DPRAM_DATA_BITMASK)!=0
      ) goto error;




	//------- testing communication via DPSEM ...
  for(i=0;i<8;i++)	// repeat test for all semaphores
  {
   MCD->cmnd=MCMND_DPSEM_COMM;
   MCD->error=0xffff;
   MCD->up[0]=0xffff;
   MCD->up[1]=0xffff;
   MCD->up[2]=0xffff;
   MCD->up[3]=(long)i;
   MCD->status=MSTAT_ATTN;

	// wait for DSP will init
   n=50;
   while ((MCD->up[0] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x0740)
     {
      	if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;
     }


   	//--test semaphore fm host
   n=51;
   HTPXDDC4_SET_DPSEM(i,1);
   if(HTPXDDC4_DPSEM_DATA(i)!=1) goto error;

   n=52;
   HTPXDDC4_SET_DPSEM(i,0);
   if(HTPXDDC4_DPSEM_DATA(i)!=0) goto error;

        // now allow DSP to set semaphore and ensure failure doing this
   MCD->up[0]=0x0741;
   while ((MCD->up[0] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x0742)
     {
      	if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;
     }


   n=53;
   HTPXDDC4_SET_DPSEM(i,1);
   if(HTPXDDC4_DPSEM_DATA(i)!=1) goto error;

        // now allow DSP to set semaphore and ensure OK
   MCD->up[0]=0x0743;
   while ((MCD->status & HTPXDDC4_DPRAM_DATA_BITMASK)!=MSTAT_RDY)
     {
      	if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;
     }

   n=54;
   if(	 (MCD->error & HTPXDDC4_DPRAM_DATA_BITMASK)!=ERR_OK
      || (MCD->up[0] & HTPXDDC4_DPRAM_DATA_BITMASK)!=0x0744
      || HTPXDDC4_DPSEM_DATA(i)!=1
     )
       goto error;

   n=55;
   HTPXDDC4_SET_DPSEM(i,0);
   if(HTPXDDC4_DPSEM_DATA(i)!=0) goto error;

   n=56;
   HTPXDDC4_SET_DPSEM(i,1);
   if(HTPXDDC4_DPSEM_DATA(i)!=1) goto error;

   n=57;
   if(HTPXDDC4_DPRAM_ERR_DATA!=0) goto error;
  }


	//---------------- execution trap code ------------

  exit_ok:	// get here on successful execution
    while(1);

  error:	// get here on any error occured (analyze N var data)
    while(1) ;

}


