/********************************************************************
   TORNADO-PX/DDC4 DCM Demo Utility. rev.1A
   Host-to-DSP communication test for T-PX/DDC4 and TORNADO-3x/E3x/P3x.
   (C) MicroLAB Systems, 2000

   file:
   -----
   	DSP resident driver for TORNADO-PX/DDC4 DCM.

   Description:
   ------------
	This is the T-PX/DDC4 resident DSP component of complex
	communication test between DSP of host TORNADO-3x/E3x/P3x
	mainboard and DSP of T-PX/DDC4 DCM. Refer for details to the
	host driver.

   Host TORNADO platform:
   ----------------------
	This demo utilize host TORNADO resources and runs in HOST mode.
	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board.

	This demo utilize T-PX/DDC4 on-board DSP resources. MIRAGE
	emulator is required to upload code to the DSP of T-PX/DDC4.

	TI CCS tools are required to recompile this demo.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - HOST-mode
	  - DCM is installed onto TORNADO-3x/E3x/P3x mainboard
***********************************************************************/


#include	"ddc4_dsp.h"	// include general DDC4 C54x defs
#include	"ddc4_dx.h"	// include general DDC4 extension functions



#define	MCD_ADDR		0xff00

#define	M_ID			0x110
#define	PRG_ID			0x111

#define	MCMND_NOP		0
#define	MCMND_SRAM_COMM		1
#define	MCMND_IRQ_COMM		2
#define	MCMND_DPSEM_COMM	3
#define	MCMND_HPI_HINT_COMM	5

#define	MSTAT_BUSY		2
#define	MSTAT_ATTN		1
#define	MSTAT_RDY		0

#define	ERR_OK			0
#define	ERR_INV_CMND		-2
#define	ERR_INV_PARAM		-1
#define	ERR_INV_INIT		-100
#define	ERR_CMND_EXEC_ERR	-3


struct MCD_DD
 { int  	m_id,
		prg_id,
		cmnd,
		status,
		error;

   unsigned     up[4];
 };

struct MCD_DD *MCD;

unsigned	mlock_flag_rq=0;


interrupt void c_hm_rq_int()
{ unsigned d;

  if(MCD->cmnd==MCMND_IRQ_COMM)
	{  MCD->up[0]=0x0123;
	  MCD->up[1]=0x4567;
	  MCD->up[2]=0x89ab;
	  MCD->up[3]=0xcdef;
	  d= TPXDDC4_dpram_hm_rq();
	  TPXDDC4_set_dpram_hm_rq(0);
          TPXDDC4_set_dpram_mh_rq(~d);
	  MCD->error=ERR_OK;
	}
}



init_cpu()
{
	TPXDDC4_init_cpu();
}



void _set_dpsem(n,d)
unsigned n,d;
{
  switch(n & 0x7)
   {
     case 0:
         TPXDDC4_set_dpsem0(d);
	 break;

     case 1:
         TPXDDC4_set_dpsem1(d);
	 break;

     case 2:
         TPXDDC4_set_dpsem2(d);
	 break;

     case 3:
         TPXDDC4_set_dpsem3(d);
	 break;

     case 4:
         TPXDDC4_set_dpsem4(d);
	 break;

     case 5:
         TPXDDC4_set_dpsem5(d);
	 break;

     case 6:
         TPXDDC4_set_dpsem6(d);
	 break;

     case 7:
         TPXDDC4_set_dpsem7(d);
	 break;

     default:
	 break;
   }
}


unsigned _dpsem(n)
unsigned n;
{
  switch(n & 0x7)
   {
     case 0:
         return(TPXDDC4_dpsem0());

     case 1:
         return(TPXDDC4_dpsem1());

     case 2:
         return(TPXDDC4_dpsem2());

     case 3:
         return(TPXDDC4_dpsem3());

     case 4:
         return(TPXDDC4_dpsem4());

     case 5:
         return(TPXDDC4_dpsem5());

     case 6:
         return(TPXDDC4_dpsem6());

     case 7:
         return(TPXDDC4_dpsem7());

     default:
	 break;
   }
}

main()
{int d,i;

   init_cpu();

	//-- clear DPSEMs in order host can take over them first
	//   in the DPSEM test
   for(i=0;i<8;i++) _set_dpsem(i,1);

	   // read & clear false Host->DSP IRQ request since it can appear
	   // during loading software to PX/DDC4
	   // (since DPRAM_HM is allocated to the DSP INT TABLE area)
   d=TPXDDC4_dpram_hm_rq();

	MCD=(struct MCD_DD *) MCD_ADDR;

		// initialize MCD data
        MCD->m_id=M_ID;
        MCD->prg_id=PRG_ID;
        MCD->cmnd=MCMND_NOP;
	MCD->error=ERR_OK;
	MCD->up[0]=0x5555;
	MCD->up[1]=0xaaaa;
	MCD->status=MSTAT_RDY;

            // enable IRQ-3 (HM_RQ)
   C54X_IFR_CLEAR;        // clear IFR/IFR
   *C54X_IMR=0;
   TPXDDC4_set_mirq3_sel(TPXDDC4_MIRQ_SEL_DPIRQ);
   C54X_XINT3_ENABLE;     
   C54X_INT_ENABLE;


   while(1)
    {
       while(MCD->status !=MSTAT_ATTN);
       MCD->status=MSTAT_BUSY;

       switch(MCD->cmnd)
	{
          case MCMND_NOP:
	       MCD->error=ERR_OK;
	       break;

          case MCMND_SRAM_COMM:
	        MCD->up[0]=0xfedc;
		MCD->up[1]=0xba98;
		MCD->up[2]=0x7654;
		MCD->up[3]=0x3210;
	        MCD->error=ERR_OK;
	       break;

          case MCMND_HPI_HINT_COMM:
	  	*C54X_HPIC |= 0x8;
	       MCD->error=ERR_OK;
	       break;

          case MCMND_DPSEM_COMM:
                MCD->up[0]=0x0740; 		// set synchro to Host
                while(MCD->up[0]!=0x0741); 	// wait for permission flag fm Host
                _set_dpsem(MCD->up[3],0);	//MCD->up[3] is the sem#
                if(_dpsem(MCD->up[3])==0)
		   { dpsem_error:
                        MCD->error=ERR_CMND_EXEC_ERR;
		        break;
		    }
                MCD->up[0]=0x0742;         	// proceed permission to host
                while(MCD->up[0]!=0x0743); 	// wait for permission flag fm Host
                if(_dpsem(MCD->up[3])!=0) goto dpsem_error;
                _set_dpsem(MCD->up[3],1);
                if(_dpsem(MCD->up[3])==0) goto dpsem_error;
                MCD->up[0]=0x0744; 		// test OK
	        MCD->error=ERR_OK;
	        break;


          default:
	       MCD->error=ERR_INV_CMND;
	       break;

	}

       MCD->status=MSTAT_RDY;
    }


   error:
      while(1);

}



