/********************************************************************
   TORNADO-PX/DDC4 DCM Demo Utility. rev.1A
   Host HPI test for TORNADO-3x/E3x/P3x.
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo performs HPI test via host TORNADO-3x/E3x/P3x mainboard.
	T-PX/DDC4 on-board DSP is placed into the 'LOOP' state by means of 
	uploading LOOP code in order to allow HPI operation for C5416 DSP
	(C5410 DSP, however, allows HPI access while in the RESET state).

   Host TORNADO platform:
   ----------------------
	This demo utilize host TORNADO resources and runs in HOST mode.
	MIRAGE emulator or COFF-loader shall be used to upload
	code to host TORNADO board.

	TI CCS tools are required to recompile this demo and to
	upload code to host TORNADO on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - HOST-mode
	  - DCM is installed onto TORNADO-3x/E3x/P3x mainboard
***********************************************************************/


#include <c3x_def.h>
#include <t3x_dsp.h>
#include <ddc4_h3x.h>


init_cpu()
{
   *C3X_PBUS_GC=T3X_PBUS_GC_DATA;
}



unsigned initial_data_main = 0xe97f,
	 initial_data_x    = 0x7531,
	 data_inc_main     = 0xb5f1,
	 data_inc_x        = 0xa359,
	 data_start_main, data_start_x;

unsigned pass_counter;


// this procedure fills in and verifies the DSP on-chip memory via HPI	

int fill_hpi_ram(saddr,eaddr,sdata)
{ unsigned sa,data;

       	   //-- fill HPI data using direct address mode
           for (sa=saddr, data=sdata ;sa<=eaddr;sa++)
	      {
	
	        HTPXDDC4_SET_HPIA(sa);
		if(HTPXDDC4_HPIA_DATA!=sa) return(-1);

		HTPXDDC4_SET_HPID(data++);
                if(HTPXDDC4_HPI_ERR_DATA!=0) return(-1);
	      }
	   return(0);
}


int verify_hpi_ram(saddr,eaddr,sdata)
{ unsigned sa,data, data_rd;

             	//-- verify HPI data using address auto increment mode
           data = sdata;
           HTPXDDC4_SET_HPIA(saddr);
	   if(HTPXDDC4_HPIA_DATA!=saddr) return(-1);

           for (sa=saddr, data=sdata ;sa<=eaddr;sa++)
	      {
		data_rd=HTPXDDC4_HPID_AINC_DATA;
                if(HTPXDDC4_HPI_ERR_DATA!=0)  return(-1);
		if(data_rd!=((data++)& 0xffff))  return(-1);
	      }

	   return(0);
}

// define start address of C54x DSP program memory, where LOOP code must be loaded
#define	C54X_DSP_CODE_START_PTR	((unsigned *) (HTPXDDC4_DPRAM_SADDR+0x7f80))


main()
{ 	int i;
  	unsigned d;
	unsigned *ptr ;


   init_cpu();

	// set DSP RESET state
   HTPXDDC4_SET_MRESET;
   if(HTPXDDC4_CNTR1_RG_DATA!=HTPXDDC4_MRESET_RESET) goto error;

	// upload LOOP.ASM code (refer to the attached program) in
	// order to allow C54x DSP HPI port access
	ptr = C54X_DSP_CODE_START_PTR;
	*ptr++	= 0x772c;
	*ptr++	= 0x0002;
	*ptr++	= 0xf495;
	*ptr++	= 0xf495;
	*ptr++	= 0xf495;
	*ptr++	= 0xf495;
	*ptr++	= 0xf495;
	*ptr++	= 0xf495;
	*ptr++	= 0xf073;
	*ptr++	= 0xff82;

	// rlease DSP RESET
   HTPXDDC4_SET_MGO;

	// initilize C54x HPI port 
   HTPXDDC4_HPI_INIT;
   HTPXDDC4_CLR_HPI_ERR;

   data_start_main = initial_data_main;
   data_start_x = initial_data_x;
   pass_counter =0;

	// provide HPI memory scan
 while(1)
     {
           // first fill-in main on-chip memory within 1st 64KW memory area
	   HTPXDDC4_XHPIA_RESET;
	   if(fill_hpi_ram(HTPXDDC4_HPIA_MIN,
			   HTPXDDC4_HPIA_MAX,
			   data_start_main
			  ) !=0
             ) goto error;


           // now fill-in the SARAM2 extended on-chip memory
	   HTPXDDC4_SET_XHPIA(HTPXDDC4_HPIA_XRAM_MAX>>16);
	   if(fill_hpi_ram((HTPXDDC4_HPIA_XRAM_MIN & 0xffff),
			   (HTPXDDC4_HPIA_XRAM_MAX & 0xffff),
			   data_start_x
			  )!=0
             ) goto error;

           // now verify main on-chip memory within 1st 64KW memory area
	   HTPXDDC4_XHPIA_RESET;
	   if(verify_hpi_ram(HTPXDDC4_HPIA_MIN,
			     HTPXDDC4_HPIA_MAX,
			     data_start_main
			    )!=0
             ) goto error;

           // now verify extended on-chip SRAM
	   HTPXDDC4_SET_XHPIA(HTPXDDC4_HPIA_XRAM_MAX>>16);
	   if(verify_hpi_ram((HTPXDDC4_HPIA_XRAM_MIN & 0xffff),
			     (HTPXDDC4_HPIA_XRAM_MAX & 0xffff),
			      data_start_x
			    )!=0
             ) goto error;

	   // now increment data and repeat
           data_start_main = data_start_main+0x8000+data_inc_main ;
           data_start_x    = data_start_x   +0x8000+data_inc_x ;


           pass_counter++;
      }

  error:
    while(1) ;

}


