/***********************************************************************
	T/SDAS-DDC1 DCM Demo Utility. PDC Demo. v.1A
	(C) MicroLAB Systems, 1999-2000


   Description:
   ------------
	This demo utilizes ADC/PFG/PDC resources and provides
	real-time output of AM demodulated signal and FM demodulated
	signal via on-board XDAC-1/XDAC-2 outputs. The output of XDAC-1 is
	proportional to the magnitude of input RF signal and demonstrates
	AM demodulation feature of HSP50214 PDC. The output of XDAC-2
	is proportional	to the frequency of input RF signal and
	demonstrates FM demodulation feature of HSP50214 PDC.

	This demo allows to configure source file to utilize either of
	the ADC-1 and ADC-2 inputs, PFG-1 and PFG-2 sampling frequency
	generators, and any of the PDC-1/PDC-2/PDC-3/PDC-4 on-board PDCs.
	User acan configure source file using _ADC_, _PFG_ and _PDC_
        definitions.

        This demo assumes that TORNADO-PX/DDC4 DCM is running in
	stand-alone mode with NO BOOT. MIRAGE JTAG emulator and TI CCS tools are
	required to recompile this demo and to upload code to the
	DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	RF_in frequency = 1MHz +-40kHz  
	RF_in amplitude = 0.4 V

        DAC-1 output:  proportional to RF_in magnitude
                       (can be used for AM demodulation)

	DAC-2 output:  proportional to the frequency of
                       (RF_in-1MHz) signal
                       (can be used for FM demodulation)

	PDC output FIR filter:
	      filter type:	LPF, 251-th order
	         bandpass:	<40kHz
	         bandstop:     	>42kHz
	  bandstop ripple: 	-60dB
	  bandpass ripple: 	<1dB

        on-board configuration jumpers:
          - no interrupt from UART
	  - no interrupt from DIO
	  - on-board Fs/CLKIN 60MHz oscillator is used
	  - on-board PDC PROCLK  50MHz oscillator is used

	on-board configuration switch:
	------------------------------
          - SA-mode
	  - no BOOT mode


***********************************************************************/

#define _TI_ENHANCED_MATH_H 1
#include	"math.h"
#include	<stdlib.h>

	// include standard defs for TORNADO-PX/DDC4 DCM
#include	"ddc4_dsp.h"
#include	"ddc4_dx.h"



//======== DEMO Configuration Options ========================
// This section contains demo configuration options, which are used
// during compilation on order o select a particular ADC/PDC/PFG
// resources utilized in the demo

	// available ADCs (do not modify)
#define	__ADC1__	1
#define	__ADC2__	2

	// available PDCs (do not modify)
#define	__PDC1__	1
#define	__PDC2__	2
#define	__PDC3__	3
#define	__PDC4__	4

	// available PFGs (do not modify)
#define	__PFG1__	1
#define	__PFG2__	2


// !!!! ------- for user modification -------!!!!
// These are the ADC/PDC/PFG selectors for this demo.
// User can modify these compiler params in order to
// select particular ADC/PDC/PFG configurations.
#define	_ADC_		__ADC1__
#define	_PDC_		__PDC1__
#define	_PFG_		__PFG1__




//>>>>>>>> substitutions for different ADC/PDC/PFG selections
//	   !!! do not modify

//----- substitutions for different ADC selection 
#if(_ADC_==__ADC1__)
	// PDC/FIFO input selectors
#define	_PDC12_ISEL	TPXDDC4_PDC12_ISEL_ADC1	
#define	_PDC34_ISEL	TPXDDC4_PDC34_ISEL_ADC1	
#define	_FIFO_ISEL	TPXDDC4_FIFO_ISEL_ADC1	
#endif


#if(_ADC_==__ADC2__)
	// PDC/FIFO input selectors
#define	_PDC12_ISEL	TPXDDC4_PDC12_ISEL_ADC2
#define	_PDC34_ISEL	TPXDDC4_PDC34_ISEL_ADC2	
#define	_FIFO_ISEL	TPXDDC4_FIFO_ISEL_ADC2	

#endif

//-------------------------------------------------



//----- substitutions for different PDC selection 

#if(_PDC_==__PDC1__)

	// macros for wait for PDC READY
#define _WAIT_PDC_READY 	while(TPXDDC4_pdc1_rdy()==0)

	// PDC A/B-output data streams
#define	_PDC_AOUT_DATA		TPXDDC4_pdc1_aout_data() 
#define	_PDC_BOUT_DATA		TPXDDC4_pdc1_bout_data() 

	// PDC uP I/F ports
#define	_PDC_HLD0(d)		TPXDDC4_set_pdc1_hld0(d)
#define	_PDC_HLD1(d)		TPXDDC4_set_pdc1_hld1(d)
#define	_PDC_HLD2(d)		TPXDDC4_set_pdc1_hld2(d)
#define	_PDC_HLD3(d)		TPXDDC4_set_pdc1_hld3(d)
#define	_PDC_WRADDR(d)		TPXDDC4_set_pdc1_wraddr(d)

#endif



#if(_PDC_==__PDC2__)

	// macros for wait for PDC READY
#define _WAIT_PDC_READY 	while(TPXDDC4_pdc2_rdy()==0)

	// PDC A/B-output data streams
#define	_PDC_AOUT_DATA		TPXDDC4_pdc2_aout_data() 
#define	_PDC_BOUT_DATA		TPXDDC4_pdc2_bout_data() 

	// PDC uP I/F ports
#define	_PDC_HLD0(d)		TPXDDC4_set_pdc2_hld0(d)
#define	_PDC_HLD1(d)		TPXDDC4_set_pdc2_hld1(d)
#define	_PDC_HLD2(d)		TPXDDC4_set_pdc2_hld2(d)
#define	_PDC_HLD3(d)		TPXDDC4_set_pdc2_hld3(d)
#define	_PDC_WRADDR(d)		TPXDDC4_set_pdc2_wraddr(d)


#endif



#if(_PDC_==__PDC3__)

	// macros for wait for PDC READY
#define _WAIT_PDC_READY 	while(TPXDDC4_pdc3_rdy()==0)

	// PDC A/B-output data streams
#define	_PDC_AOUT_DATA		TPXDDC4_pdc3_aout_data() 
#define	_PDC_BOUT_DATA		TPXDDC4_pdc3_bout_data() 

	// PDC uP I/F ports
#define	_PDC_HLD0(d)		TPXDDC4_set_pdc3_hld0(d)
#define	_PDC_HLD1(d)		TPXDDC4_set_pdc3_hld1(d)
#define	_PDC_HLD2(d)		TPXDDC4_set_pdc3_hld2(d)
#define	_PDC_HLD3(d)		TPXDDC4_set_pdc3_hld3(d)
#define	_PDC_WRADDR(d)		TPXDDC4_set_pdc3_wraddr(d)


#endif




#if(_PDC_==__PDC4__)

	// macros for wait for PDC READY
#define _WAIT_PDC_READY 	while(TPXDDC4_pdc4_rdy()==0)

	// PDC A/B-output data streams
#define	_PDC_AOUT_DATA		TPXDDC4_pdc4_aout_data() 
#define	_PDC_BOUT_DATA		TPXDDC4_pdc4_bout_data() 

	// PDC uP I/F ports
#define	_PDC_HLD0(d)		TPXDDC4_set_pdc4_hld0(d)
#define	_PDC_HLD1(d)		TPXDDC4_set_pdc4_hld1(d)
#define	_PDC_HLD2(d)		TPXDDC4_set_pdc4_hld2(d)
#define	_PDC_HLD3(d)		TPXDDC4_set_pdc4_hld3(d)
#define	_PDC_WRADDR(d)		TPXDDC4_set_pdc4_wraddr(d)

#endif
//-------------------------------------------------



//----- substitutions for different PFG selection 

#if(_PFG_==__PFG1__)
	// PFG selector for PFG config procedure
#define	_PFG_SEL	TPXDDC4_PFG1_SEL

	// ADC clock selectors
#define	_ADC1_CLK_SEL	TPXDDC4_ADC1_CLK_SEL_PFG1
#define	_ADC2_CLK_SEL	TPXDDC4_ADC2_CLK_SEL_PFG1


#endif


#if(_PFG_==__PFG2__)
	// PFG selector for PFG config procedure
#define	_PFG_SEL	TPXDDC4_PFG2_SEL

	// ADC clock selectors
#define	_ADC1_CLK_SEL	TPXDDC4_ADC1_CLK_SEL_PFG2
#define	_ADC2_CLK_SEL	TPXDDC4_ADC2_CLK_SEL_PFG2

#endif
//--------------------------------------------------

//==========================================================





//~~~~~~~~~~~ HSP50214 PDC uploadable configuration data ~~~~~~~~~~

unsigned long hsp50214B_init[256]=
{
0x0000a708,  0x00010000,  0x00000000,  0x04444444,
0x00000000,  0x00000000,  0x00000000,  0x000f8efb,
0x00000000,  0x0b000b00,  0x00000000,  0x00000000,
0xcccccccc,  0x00000000,  0x00000000,  0x00000000,
0x00000a61,  0x00004a80,  0x00000000,  0x0a254800,
0x02a00000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000278,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
/* Discriminator FIR Coefficients */
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x00000000,  0x00000000,  0x00000000,  0x00000000,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
0x001fffff,  0x001fffff,  0x001fffff,  0x001fffff,
/* Programmable FIR Coefficients */
0x0014fdf3,  0x0008fc3c,  0xfffbc5de,  0xffffa8e9,
0x0002594a,  0xfffe7000,  0xffffa9c7,  0x000161de,
0xffff1da5,  0xffffab35,  0x0000fb7c,  0xffff6e79,
0xffffad2d,  0x0000c1b4,  0xffff9dd4,  0xffffafa5,
0x00009b78,  0xffffbd28,  0xffffb294,  0x00007f98,
0xffffd36a,  0xffffb5ee,  0x000069f0,  0xffffe3ef,
0xffffb9a3,  0x00005860,  0xfffff083,  0xffffbda6,
0x000049b6,  0xfffffa35,  0xffffc1e7,  0x00003d3b,
0x000001b1,  0xffffc654,  0x0000327a,  0x0000076e,
0xffffcadf,  0x00002927,  0x00000bbe,  0xffffcf76,
0x0000210e,  0x00000ee2,  0xffffd40a,  0x00001a06,
0x0000110b,  0xffffd88c,  0x000013f3,  0x00001263,
0xffffdcef,  0x00000ebd,  0x0000130c,  0xffffe128,
0x00000a50,  0x00001325,  0xffffe52a,  0x00000699,
0x000012ca,  0xffffe8ef,  0x00000387,  0x00001211,
0xffffec6e,  0x0000010a,  0x00001111,  0xffffefa3,
0xffffff15,  0x00000fdc,  0xfffff28b,  0xfffffd96,
0x00000e84,  0xfffff523,  0xfffffc7f,  0x00000d17,
0xfffff76d,  0xfffffbc3,  0x00000ba3,  0xfffff969,
0xfffffb53,  0x00000a31,  0xfffffb1b,  0xfffffb23,
0x000008cc,  0xfffffc87,  0xfffffb27,  0x0000077a,
0xfffffdb1,  0xfffffb53,  0x00000640,  0xfffffea0,
0xfffffb9e,  0x00000523,  0xffffff5a,  0xfffffbfe,
0x00000425,  0xffffffe4,  0xfffffc6b,  0x00000346,
0x00000046,  0xfffffcde,  0x00000286,  0x00000086,
0xfffffd53,  0x000001e5,  0x000000ab,  0xfffffdc3,
0x00000161,  0x000000ba,  0xfffffe2d,  0x000000f7,
0x000000b9,  0xfffffe8d,  0x000000a4,  0x000000ab,
0xfffffee2,  0x00000067,  0x00000096,  0xffffff2b,
0x0000003a,  0x0000007c,  0xffffff67,  0x0000001c,
0x00000061,  0xffffff99,  0x00000009,  0x00000047,
0xffffffbf,  0x00000000,  0x00000000,  0x00000000

};

// On-board PFG is used to generate 60MHz sampling freqency.
// set PFG to 60MHz = (10MHz*(N=36)*(V=1)/(M=3)*(R=2)*(X=1))
// (N=36, M=3, V=0 (1), X=0 (1), R=1 (2))
#define PFG_N_DATA       36
#define PFG_M_DATA       3
#define PFG_V_DATA       0
#define PFG_X_DATA       0
#define PFG_R_DATA       1


	// this function gnerates the delay abot 500uS,
	// which is required to lock frequency by PFG
void pfg_setup_delay()
 { unsigned i;

	for(i=0;i<10000;i++);
 }




	// this function performs DSP and system initialization
cpu_init()
{
		// use standard PX/DDC4 DSP init procedure
	TPXDDC4_init_cpu();	

		// use standard McBSP-1 port init procedure for
		// communication with PFG-1 and XDAC-1/XDAC-2
	TPXDDC4_init_mcbsp1();

		// configure XDAC-1/2 to 4v scale
	TPXDDC4_set_xdac_scale_4v();

	  	// configure PFG and wait until
		// output frequency will be locked
	TPXDDC4_write_pfg(_PFG_SEL,
			  PFG_N_DATA,
                          PFG_M_DATA,
                          PFG_V_DATA,
  	  	    	  PFG_X_DATA,
                          PFG_R_DATA);
	pfg_setup_delay();


		// set ADC-1 sampling frequency from PFG-1
        TPXDDC4_set_adc_clk_sel(  _ADC1_CLK_SEL
				| _ADC2_CLK_SEL
				| TPXDDC4_PFG1_AUX_FDIV_OFF
				| TPXDDC4_PFG2_AUX_FDIV_OFF
			       );

		// route ADC stream to the PDC/FIFO input streams
	TPXDDC4_set_pdc_fifo_isel(  _PDC12_ISEL 
	  			  | _PDC34_ISEL
				  | _FIFO_ISEL
				 );
	return(0);
}


	// this procedure performs PDC init operation
	// using param table
void init_DDC(unsigned long *data,int addr,int len)
{
	unsigned long i,j,k,l,dat;

	j=addr;
	k=addr;
	l=len;



    rep0:
	for(i=0;i<l;i++)
        {
		// get 32-bi config data word fm the table
		dat=data[j++];

		// load 32-bit data words as four bytes
		_PDC_HLD0(dat);
                dat>>=8;
		_PDC_HLD1(dat);
		dat>>=8;
		_PDC_HLD2(dat);
		dat>>=8;
		_PDC_HLD3(dat);

		// load address
		_PDC_WRADDR(k);

		// proceed with the next config word		
		k++;
	}
	if(l==28)
        {
		l=25;
		k=0;
		j=0;
		goto rep0;
	}
}


	// load demonulated MAG data to XDAC-1
void out_MAG(int mag_level)
{
	TPXDDC4_write_xdac(TPXDDC4_XDAC1_SEL, mag_level & 0xfff);
}

	// load demodulated FREQ data to XDAC-2
void out_FREQ(int freq_level)
{
	TPXDDC4_write_xdac(TPXDDC4_XDAC2_SEL, freq_level &0xfff);
}



void main()
{
	if( cpu_init()!=0) while(1);	// idle here in incorrect DSP init

		// init PDC, assume 16-bit Parallel Mode with
		// AOUT=FREQ, BOUT=MAG outputs
	init_DDC(hsp50214B_init,0,28);
	init_DDC(hsp50214B_init,0,28);
	init_DDC(hsp50214B_init,64,192);
	init_DDC(hsp50214B_init,64,192);


		// loop infinitely with PDC read and
		// reload demodulated MAG/FREQ data to XDAC-1/2
	while(1)
        {
		// wait for PDC DATAREADY signal
          _WAIT_PDC_READY;

		// read and send MAG data to XDAC-1
	  out_MAG (_PDC_BOUT_DATA >>4);

		//!!! shift FREQ data to the mid of XDAC-2 scale
	  out_FREQ(0x800 ^ _PDC_AOUT_DATA); 
	}
}
