/********************************************************************
   TORNADO-PX/DDC4 DCM Demo Utility.
   XDAC-1/2, DIO, XSL-1/2, PFG-1/2 demo. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo writes RAMPs to DAC-1/2 and (1/2* sampling_frequency)
	to the DIO output.
	XSL-1/2 are loaded with test patterns in 30-bit data format.
	PFG-1/2 are loaded with predefined codes once per 100Hz.
	ADC-1_CLK is configured to PFG-1/8.
	ADC-2_CLK is configured to PFG-2.


   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
        on-board configuration switch:
          - SA-mode
	  - no BOOT mode
***********************************************************************/

#include	"math.h"
#include	"c54x_def.h"
#include	"ddc4_dsp.h"
#include	"ddc4_dx.h"


#define	_CLEAR_TM_EVENT		*C54X_IFR=8
#define	_WAIT_TM_EVENT		while(!(*C54X_IFR & 8))


void set_dio_output(data)
{
		// set GPIO-0 as direct and GPIO-1 as inversed DATA
	TPXDDC4_set_gpio0_data(data);	
	TPXDDC4_set_gpio1_data(~data);
}



void init_cpu()
 {
	TPXDDC4_init_cpu();
 }

void init_sio()
{
		// init McBSP-1 port
	TPXDDC4_init_mcbsp1();

		// configure GPIO-0/1 as outputs
	TPXDDC4_set_gpio_dir(TPXDDC4_GPIO0_DIR_OUTPUT | TPXDDC4_GPIO1_DIR_OUTPUT);

		// configure XDAC-1/2 to 4v output scale
	TPXDDC4_set_xdac_scale_4v();

		// configure XSL-1/2 to 30-bit data, direct/framed CLKX
        TPXDDC4_set_xsl_fmt(  TPXDDC4_XSL1_FMT_WLEN30
			    | TPXDDC4_XSL2_FMT_WLEN30
			    | TPXDDC4_XSL_CLKPOL_POS
			    | TPXDDC4_XSL_CLKFRM_ON
			   );

		// configure ADC-1/2 CLK selectors.
		// ADC-1 CLK is set to PFG-1 clock with AUX_DIV /8
		// ADC-2 CLK is set PFG-2.
        TPXDDC4_set_adc_clk_sel(  TPXDDC4_ADC1_CLK_SEL_PFG1
				| TPXDDC4_ADC2_CLK_SEL_PFG2
				| TPXDDC4_PFG1_AUX_FDIV_ON
				| TPXDDC4_PFG2_AUX_FDIV_OFF
			       );

		//--- set 20 kHz sampling frequency
	*C54X_TM_TCR = (TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410) ? 0x0019:0x001F;
	*C54X_TM_PRD = 500;
	*C54X_TM_TCR = (TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410) ? 0x0009:0x000F;	// start timer

        	//--- clear TIMER interrupt request
	_CLEAR_TM_EVENT;
}

#define	XSL1_DATA	0x10000002L
#define	XSL2_DATA	0x05555555L

	// set PFG-1 = (100/4*8)=3.1MHz
	// (N=30, M=3, V=0 (1), X=2, R=3)
#define PFG1_N_DATA       30
#define PFG1_M_DATA       3
#define PFG1_V_DATA       0
#define PFG1_X_DATA       2
#define PFG1_R_DATA       3

	// set PFG-2 = (60/1*4)=15MHz
	// (N=24, M=4, V=0 (1), X=0, R=2)
#define PFG2_N_DATA       24
#define PFG2_M_DATA       4
#define PFG2_V_DATA       0
#define PFG2_X_DATA       0
#define PFG2_R_DATA       2

void main()
{  int i;

	init_cpu();
	init_sio();

        i=0;
	while (1)
	 {
	  _WAIT_TM_EVENT;
	  _CLEAR_TM_EVENT;

	  	// toggle DIO-0/1 outputs
	  set_dio_output(i%2);		


		// load RAMP codes to XDAC-1/2
	  TPXDDC4_write_xdac(1,4*i);
	  TPXDDC4_write_xdac(2,4*(i%500));

		// load predefined patterns to XSL-1/2
	  TPXDDC4_write_xsl(1,XSL1_DATA);
	  TPXDDC4_write_xsl(2,XSL2_DATA);

		// perform load for PFG-1/2 once per 100 cycles
		// in order to give PFG enough time for freq lock
	  if((i%100)==0)
             {
	  	// write to PFG-1
	      TPXDDC4_write_pfg(1,PFG1_N_DATA,PFG1_M_DATA,PFG1_V_DATA,
  	  	    	PFG1_X_DATA,PFG1_R_DATA);

	  		// write to PFG-2
	      TPXDDC4_write_pfg(2,PFG2_N_DATA,PFG2_M_DATA,PFG2_V_DATA,
  	  	    	PFG2_X_DATA,PFG2_R_DATA);
             }
	  if((++i)>=1000) i=0;
	 }

}

