/********************************************************************
   TORNADO-PX/DDC4 Coprocessor Demo Utility.
   UART-2 demo for SA-mode. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo utilizes UART chip and provides communication
	with any external RS232C terminal at 9600 bauds. This demo
	prints greetings message and aferthat just returns the
	received symbols back to terminal, and detects the CTS/DSR
	status with printing the corresponding message at the terminal.

   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	baud rate: 	9600
	word length:	8 bits
	stop bits: 	2

        on-board configuration switch:
          - SA-mode
	  - no BOOT mode

***********************************************************************/

#include	"c54x_def.h"
#include	"ddc4_dsp.h"
#include	"ddc4_dx.h"


char welcome[]	= "TORNADO-PX/DDC4 welcomes you via UART-2 I/F !\n\n>>";
char cts_low[]	= "<< RTS: CTS=LOW >>";
char cts_high[]	= "<< RTS: CTS=HIGH >>";
char dsr_low[]	= "<< DTR: DSR=LOW >>";
char dsr_high[]	= "<< DTR: DSR=HIGH >>";


uart_transmit_char(c)
int c;
{
  while(!(TPXDDC4_UART2_LSTAT_RG & TPXDDC4_UART_LSTAT_THEMPY));
  TPXDDC4_UART2_TDATA_RG = c;
}


unsigned uart_receive_char()
{   unsigned d;

  do
    { d = TPXDDC4_UART2_LSTAT_RG;
    }
     while(!(d & TPXDDC4_UART_LSTAT_DR));

  return(TPXDDC4_UART2_RDATA_RG & 0xff);
}


int uart_receiver_status()
{
  return(TPXDDC4_UART2_LSTAT_RG & TPXDDC4_UART_LSTAT_DR);
}


void init_uart()
 {
		// configure UART for 9600/8/N/2

	   TPXDDC4_UART2_LCNTR_RG =   TPXDDC4_UART_LCNTR_DLAB
				    | TPXDDC4_UART_LCNTR_WLEN8
		     		    | TPXDDC4_UART_LCNTR_STOP2 ;

	   TPXDDC4_UART2_DIVL_RG  = TPXDDC4_UART_DIVL_9600;
	   TPXDDC4_UART2_DIVH_RG  = TPXDDC4_UART_DIVH_9600;

	   TPXDDC4_UART2_LCNTR_RG =   TPXDDC4_UART_LCNTR_WLEN8
		                    | TPXDDC4_UART_LCNTR_STOP2 ;

	   TPXDDC4_UART2_MCNTR_RG = 0;
	   TPXDDC4_UART2_IE_RG    = TPXDDC4_UART_IE_DR;
 }


void uart_transmit_mess(s)
char *s;
{
	  while((*s &0xff)!=0) uart_transmit_char(*(s++));
}


void init_cpu()
 {
	TPXDDC4_init_cpu();
 }



void main()
{
   int modem_status;

	init_cpu();
	init_uart();

        uart_transmit_mess(welcome);


     while (1)
	{
	 modem_status = TPXDDC4_UART2_MSTAT_RG;

		// copy CTS->RTS
	 if(modem_status & TPXDDC4_UART_MSTAT_CTS)  
	     TPXDDC4_UART2_MCNTR_RG = (TPXDDC4_UART2_MCNTR_RG | TPXDDC4_UART_MCNTR_RTS);
	   else
	     TPXDDC4_UART2_MCNTR_RG = (TPXDDC4_UART2_MCNTR_RG & ~TPXDDC4_UART_MCNTR_RTS);

		// copy DSR->DTR
	 if(modem_status & TPXDDC4_UART_MSTAT_DSR)  
	     TPXDDC4_UART2_MCNTR_RG = (TPXDDC4_UART2_MCNTR_RG | TPXDDC4_UART_MCNTR_DTR);
	   else
	     TPXDDC4_UART2_MCNTR_RG = (TPXDDC4_UART2_MCNTR_RG & ~TPXDDC4_UART_MCNTR_DTR);


		// detect CTS change
   	 if(modem_status & TPXDDC4_UART_MSTAT_DCTS)  
	    {
	      if(modem_status & TPXDDC4_UART_MSTAT_CTS)
		   uart_transmit_mess(cts_high);
		else
		   uart_transmit_mess(cts_low);
	    }

		// detect DSR change
	 if(modem_status & TPXDDC4_UART_MSTAT_DDSR)
	    {
	      if(modem_status & TPXDDC4_UART_MSTAT_DSR)
		   uart_transmit_mess(dsr_high);
		else
		   uart_transmit_mess(dsr_low);
	    }


	 if(uart_receiver_status()==TPXDDC4_UART_LSTAT_DR)
              uart_transmit_char(uart_receive_char());
	 }

}

