/****************************************************
   TORNADO-PX/DDC4 DCM Demo Utility.
   XLINK demo. rev.1A
   (C) MicroLAB Systems, 2000


   Description:
   ------------
	This demo configres XLINK port pins of on-board JP4 External Lnk/Power
	connector as OUTPUT pins via C54xx DSP on-chip McBSP-0 port control
	register and runs '1' thru all XLINK pins. XLINK signal status can be 
	viewed via oscilloscope.

	It is required that XLINK XL-BDR pin (pin #5 of JP4 connector) is
	connected with XL-BDX pin (pin #11 of JP4 connector) for correct running
	this test unless demo code is modified to exclude chech for XL-BDR pin
	operation.



   TORNADO platform:
   -----------------
	This demo does not utilize any host TORNADO resources
	and runs in stand-alone mode. MIRAGE JTAG emulator and
	TI CCS tools are required to recompile this demo and to
	upload code to the on-board DSP environment.

  Copyright:
  ----------
	This demo is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


  Demo and Signal Parameters:
  ---------------------------
	on-board configuration switch:
    	- SA-mode
	  	- no BOOT mode

*****************************************************/

#include 	"c54x_def.h"
#include	"ddc4_dsp.h"
#include	"ddc4_dx.h"

#define DX_OUT_0      0x3f00
#define DX_OUT_1      0x3f20
#define FSX_OUT_0     0x3f00
#define FSX_OUT_1     0x3f08
#define FSR_OUT_0     0x3f00
#define FSR_OUT_1     0x3f04
#define CLKX_OUT_0    0x3f00
#define CLKX_OUT_1    0x3f02
#define CLKR_OUT_0    0x3f00
#define CLKR_OUT_1    0x3f01

#define DR_BIT_MASK   0x10
#define CLKS_BIT_MASK 0x40
#define CLKR_BIT_MASK 0x1
#define CLKX_BIT_MASK 0x2
#define FSX_BIT_MASK  0x8
#define FSR_BIT_MASK  0x4



unsigned pass_counter=0;

main()
{
	TPXDDC4_init_cpu();

   	*C54X_IFR=0;        // clear IFR/IFR
   	*C54X_IMR=0;

	// configure McBSP-0 port routed to on-board XLINK port
	// All McBSP-0 pins are configured as OUTPUT pins except for DR pin, which is still INPUT pin.
   	*C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_SPCR1;
   	*C54X_MCBSP0_SPCR1 = 0;

   	*C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_SPCR2;
   	*C54X_MCBSP0_SPCR2 = 0;


   	while(1)
    {
      	// run '1' thru all McBSP-0 pins except for DR
	
		// set DX=1
      	*C54X_MCBSP0_SPSA = C54X_MCBSP_SBA_PCR;

      	*C54X_MCBSP0_PCR = DX_OUT_1;

		// check whether DX status is returned via DR value
		// comment this line in case it is not required to test DR pin
      	if(((*C54X_MCBSP0_PCR) & DR_BIT_MASK  ) ==0) break;

		// set DX=0
      	*C54X_MCBSP0_PCR = DX_OUT_0;

		// check whether DX status is returned via DR value
		// comment this line in case it is not required to test DR pin
      	if((*C54X_MCBSP0_PCR & DR_BIT_MASK  ) !=0) break;

		// generate '1' and '0' thru FSX output pin
		*C54X_MCBSP0_PCR = FSX_OUT_1;
      	*C54X_MCBSP0_PCR = FSX_OUT_0;

		// generate '1' and '0' thru FSR output pin
      	*C54X_MCBSP0_PCR = FSR_OUT_1;
      	*C54X_MCBSP0_PCR = FSR_OUT_0;

		// generate '1' and '0' thru CLKX output pin
      	*C54X_MCBSP0_PCR = CLKX_OUT_1;
      	*C54X_MCBSP0_PCR = CLKX_OUT_0;

		// generate '1' and '0' thru CLKR output pin
		*C54X_MCBSP0_PCR = CLKR_OUT_1;
      	*C54X_MCBSP0_PCR = CLKR_OUT_0;

     	pass_counter++;
    }

error:
	// loop here on any error condition
	while(1);
}



