/*******************************************************************
  TORNADO-PX/DDC4 Quad DRR PIOX-16 Coprocessor/Controller. Rev.1B
  On-board C54xx DSP Environment. Main definitions.

  (C) MicroLAB Systems Ltd, 1998-2003
  http://www.mlabsys.com


  Notes:
  ------
    1. This file must be used as .H header file for
       PX/DDC4 on-board C54xx DSP applications.

  Revision History:
  -----------------
    v.1A	- September'2000. Initial release.

	v.1A-2	- March'2001. Added structurized support for automatic
			  detection of DSP type and 4-bit external FLASH memory
			  page addressing.
	v.1B	- April-2003. Added BOARD REV_ID and PFG refrence clock frequency ID
			  into SYS_STAT_RG, and full support for 512Kx8 on-board FLASH 
			  memory is provided.
*******************************************************************/

#ifndef __DDC4_DEF__
#define __DDC4_DEF__		1

#define	TPXDDC4_OFF			0
#define	TPXDDC4_ON			1


//========= recommended SWWSR register settings =======
// RAM on-chip -0ws
// RAM/@>8000H - 2ws (required for DP_BUSY processing)
// IO          - 2ws (required for external ready processing)
// CLKOUT      = 1xDSP CLK (100MHz) for C5410 (via BSCR)
//               1/2xDSP CLK (80MHz) for C5416 (via BSCR)
// PLL	       = x1 (100MHz) for C5410 (via CLKMD)
//               x2 (160MHz) for C5416 DSP (via CLKMD)

#define TPXDDC4_C5410_SWWSR_DATA		0x2410
#define TPXDDC4_C5410_SWCR_DATA			0x0000
#define TPXDDC4_C5410_BSCR_DATA			0x8000
#define TPXDDC4_C5410_CLKMD_DATA		0x07f7
#define TPXDDC4_C5416_SWWSR_DATA		0x2410
#define TPXDDC4_C5416_SWCR_DATA			0x0000
#define TPXDDC4_C5416_BSCR_DATA			0xA000
#define TPXDDC4_C5416_CLKMD_DATA		0x17f7

	// Intermediate data for CLKMD rg prior loading final data (corresponds to /2).
	// This is required to configure C54x PLL via software.
#define TPXDDC4_CLKMD_DATA0				0x07f0

//=====================================================

//============== DPRAM/FLASH defs =====================
#define TPXDDC4_DPRAM_SADDR			0x8000
#define TPXDDC4_DPRAM_EADDR			0xffff
#define TPXDDC4_DPRAM_LEN			0x8000
#define	TPXDDC4_DPRAM_HM_RQ_ADDR	0xffff	// DPRAM: HostPIOX-to-DSP irq
#define	TPXDDC4_DPRAM_MH_RQ_ADDR	0xfffe	// DPRAM: DSP-to-HostPIOX irq

	// FLASH parameters are set upon the DDC4 board revision ID and
	// correspond to 128Kx8 FLASH memory for DDC4 rev.1A and to 512Kx8 FLASH memory
	// for DDC4 rev.1B
#define TPXDDC4_FLASH_PAGE_SADDR	0x8000
#define TPXDDC4_FLASH_PAGE_EADDR	0xffff
#define TPXDDC4_FLASH_PAGE_LEN		0x8000

#define TPXDDC4_FLASH_PAGES_REV1A	4
#define TPXDDC4_FLASH_PAGES_REV1B	16
#define TPXDDC4_FLASH_PAGES			((TPXDDC4_REV_ID_IS_REV1B)? TPXDDC4_FLASH_PAGES_REV1B:TPXDDC4_FLASH_PAGES_REV1A)

#define TPXDDC4_FLASH_DATA_BITMASK	0xff

		// these defs are for TPXDDC4_flash_xxx functions from DDC4_DX.H file
#define TPXDDC4_FLASH_SADDR			0x00000L

#define TPXDDC4_FLASH_EADDR_REV1A	0x1ffffL
#define TPXDDC4_FLASH_EADDR_REV1B	0x7ffffL
#define TPXDDC4_FLASH_EADDR			((TPXDDC4_REV_ID_IS_REV1B)? TPXDDC4_FLASH_EADDR_REV1B:TPXDDC4_FLASH_EADDR_REV1A)

#define TPXDDC4_FLASH_LEN           (TPXDDC4_FLASH_EADDR-TPXDDC4_FLASH_SADDR+1L)

#define TPXDDC4_FLASH_SECTOR_LEN_REV1A	0x4000L
#define TPXDDC4_FLASH_SECTOR_LEN_REV1B	0x10000L
#define TPXDDC4_FLASH_SECTOR_LEN		((TPXDDC4_REV_ID_IS_REV1B)? TPXDDC4_FLASH_SECTOR_LEN_REV1B:TPXDDC4_FLASH_SECTOR_LEN_REV1A)

#define TPXDDC4_FLASH_SECTORS		8

	// DPSEM defs
#define	TPXDDC4_DPSEM_BITMASK		0x1	// DPSEM data bitmask and values
#define	TPXDDC4_DPSEM_OFF			0x1
#define	TPXDDC4_DPSEM_ON			0x0

	// FIFO defs
#define	TPXDDC4_FIFO_LEN			0x40000L
#define	TPXDDC4_FIFO_DATA_BITMASK	0x0fff

//=====================================================


//================= I/O ports defs ====================

//~~~~ UART-1/2
#define TPXDDC4_UART1_RDATA_RG		port0008
#define TPXDDC4_UART1_TDATA_RG		port0008
#define TPXDDC4_UART1_DIVL_RG		port0008
#define TPXDDC4_UART1_IE_RG			port0009
#define TPXDDC4_UART1_DIVH_RG		port0009
#define TPXDDC4_UART1_FIFOCNTR_RG	port000a
#define TPXDDC4_UART1_IID_RG		port000a
#define TPXDDC4_UART1_ALTFUNC_RG	port000a
#define TPXDDC4_UART1_LCNTR_RG		port000b
#define TPXDDC4_UART1_MCNTR_RG		port000c
#define TPXDDC4_UART1_LSTAT_RG		port000d
#define TPXDDC4_UART1_MSTAT_RG		port000e
#define TPXDDC4_UART1_SCRATCH_RG	port000f

#define TPXDDC4_UART2_RDATA_RG		port0000
#define TPXDDC4_UART2_TDATA_RG		port0000
#define TPXDDC4_UART2_DIVL_RG		port0000
#define TPXDDC4_UART2_IE_RG			port0001
#define TPXDDC4_UART2_DIVH_RG		port0001
#define TPXDDC4_UART2_FIFOCNTR_RG	port0002
#define TPXDDC4_UART2_IID_RG		port0002
#define TPXDDC4_UART2_ALTFUNC_RG	port0002
#define TPXDDC4_UART2_LCNTR_RG		port0003
#define TPXDDC4_UART2_MCNTR_RG		port0004
#define TPXDDC4_UART2_LSTAT_RG		port0005
#define TPXDDC4_UART2_MSTAT_RG		port0006
#define TPXDDC4_UART2_SCRATCH_RG	port0007


//~~~~ PDC-1/2/3/4

	// PDC-1 uP I/F registers
	// (least significant 8-bit byte is valid only)
#define TPXDDC4_PDC1_HLD0_RG		port0040
#define TPXDDC4_PDC1_HLD1_RG		port0041
#define TPXDDC4_PDC1_HLD2_RG		port0042
#define TPXDDC4_PDC1_HLD3_RG		port0043
#define TPXDDC4_PDC1_WRADDR_RG		port0044
#define TPXDDC4_PDC1_RDADDR_RG		port0045

	//read-only PDC-1 AOUT/BOUT regs for 16-bit Parallel Mode
#define TPXDDC4_PDC1_OEA_RG			port0080	//read-only
#define TPXDDC4_PDC1_OEB_RG			port0088	//read-only

	//read-only PDC-1 AOUT/BOUT regs for 16-BIT FIFO buffer mode
	//  16-bits data FIFO data:
	//	LSB = BOUT[0..7]
	//	MSB = AOUT[0..7]	
#define TPXDDC4_PDC1_OEA0_RG		port0080	
#define TPXDDC4_PDC1_OEA1_RG		port0081	
#define TPXDDC4_PDC1_OEA2_RG		port0082	
#define TPXDDC4_PDC1_OEA3_RG		port0083
#define TPXDDC4_PDC1_OEA4_RG		port0084	
#define TPXDDC4_PDC1_OEA5_RG		port0085	
#define TPXDDC4_PDC1_OEA6_RG		port0086	
#define TPXDDC4_PDC1_OEA7_RG		port0087	

#define TPXDDC4_PDC1_OEB0_RG		port0088	
#define TPXDDC4_PDC1_OEB1_RG		port0089	
#define TPXDDC4_PDC1_OEB2_RG		port008a	
#define TPXDDC4_PDC1_OEB3_RG		port008b	
#define TPXDDC4_PDC1_OEB4_RG		port008c	
#define TPXDDC4_PDC1_OEB5_RG		port008d	
#define TPXDDC4_PDC1_OEB6_RG		port008e	
#define TPXDDC4_PDC1_OEB7_RG		port008f	


	// PDC-2 uP I/F registers
	// (least significant 8-bit byte is valid only)
#define TPXDDC4_PDC2_HLD0_RG		port0048
#define TPXDDC4_PDC2_HLD1_RG		port0049
#define TPXDDC4_PDC2_HLD2_RG		port004a
#define TPXDDC4_PDC2_HLD3_RG		port004b
#define TPXDDC4_PDC2_WRADDR_RG		port004c
#define TPXDDC4_PDC2_RDADDR_RG		port004d

	//read-only PDC-2 AOUT/BOUT regs for 16-bit Parallel Mode
#define TPXDDC4_PDC2_OEA_RG			port0090	
#define TPXDDC4_PDC2_OEB_RG			port0098

	//read-only PDC-2 AOUT/BOUT regs for 16-BIT FIFO buffer mode
	//  16-bits data FIFO data:
	//	LSB = BOUT[0..7]
	//	MSB = AOUT[0..7]	
#define TPXDDC4_PDC2_OEA0_RG		port0090
#define TPXDDC4_PDC2_OEA1_RG		port0091
#define TPXDDC4_PDC2_OEA2_RG		port0092	
#define TPXDDC4_PDC2_OEA3_RG		port0093	
#define TPXDDC4_PDC2_OEA4_RG		port0094	
#define TPXDDC4_PDC2_OEA5_RG		port0095	
#define TPXDDC4_PDC2_OEA6_RG		port0096	
#define TPXDDC4_PDC2_OEA7_RG		port0097	

#define TPXDDC4_PDC2_OEB0_RG		port0098	
#define TPXDDC4_PDC2_OEB1_RG		port0099	
#define TPXDDC4_PDC2_OEB2_RG		port009a	
#define TPXDDC4_PDC2_OEB3_RG		port009b	
#define TPXDDC4_PDC2_OEB4_RG		port009c	
#define TPXDDC4_PDC2_OEB5_RG		port009d
#define TPXDDC4_PDC2_OEB6_RG		port009e	
#define TPXDDC4_PDC2_OEB7_RG		port009f	


	// PDC-3 uP I/F registers
	// (least significant 8-bit byte is valid only)
#define TPXDDC4_PDC3_HLD0_RG		port0050
#define TPXDDC4_PDC3_HLD1_RG		port0051
#define TPXDDC4_PDC3_HLD2_RG		port0052
#define TPXDDC4_PDC3_HLD3_RG		port0053
#define TPXDDC4_PDC3_WRADDR_RG		port0054
#define TPXDDC4_PDC3_RDADDR_RG		port0055

	//read-only PDC-3 AOUT/BOUT regs for 16-bit Parallel Mode
#define TPXDDC4_PDC3_OEA_RG			port00a0
#define TPXDDC4_PDC3_OEB_RG			port00a8

	//read-only PDC-3 AOUT/BOUT regs for 16-BIT FIFO buffer mode
	//  16-bits data FIFO data:
	//	LSB = BOUT[0..7]
	//	MSB = AOUT[0..7]	
#define TPXDDC4_PDC3_OEA0_RG		port00a0	
#define TPXDDC4_PDC3_OEA1_RG		port00a1	
#define TPXDDC4_PDC3_OEA2_RG		port00a2	
#define TPXDDC4_PDC3_OEA3_RG		port00a3
#define TPXDDC4_PDC3_OEA4_RG		port00a4	
#define TPXDDC4_PDC3_OEA5_RG		port00a5	
#define TPXDDC4_PDC3_OEA6_RG		port00a6	
#define TPXDDC4_PDC3_OEA7_RG		port00a7	

#define TPXDDC4_PDC3_OEB0_RG		port00a8
#define TPXDDC4_PDC3_OEB1_RG		port00a9	
#define TPXDDC4_PDC3_OEB2_RG		port00aa
#define TPXDDC4_PDC3_OEB3_RG		port00ab
#define TPXDDC4_PDC3_OEB4_RG		port00ac	
#define TPXDDC4_PDC3_OEB5_RG		port00ad	
#define TPXDDC4_PDC3_OEB6_RG		port00ae	
#define TPXDDC4_PDC3_OEB7_RG		port00af	


	// PDC-4 uP I/F registers
	// (least significant 8-bit byte is valid only)
#define TPXDDC4_PDC4_HLD0_RG		port0058
#define TPXDDC4_PDC4_HLD1_RG		port0059
#define TPXDDC4_PDC4_HLD2_RG		port005a
#define TPXDDC4_PDC4_HLD3_RG		port005b
#define TPXDDC4_PDC4_WRADDR_RG		port005c
#define TPXDDC4_PDC4_RDADDR_RG		port005d

	//read-only PDC-4 AOUT/BOUT regs for 16-bit Parallel Mode
#define TPXDDC4_PDC4_OEA_RG			port00b0
#define TPXDDC4_PDC4_OEB_RG			port00b8

	//read-only PDC-4 AOUT/BOUT regs for 16-BIT FIFO buffer mode
	//  16-bits data FIFO data:
	//	LSB = BOUT[0..7]
	//	MSB = AOUT[0..7]	
#define TPXDDC4_PDC4_OEA0_RG		port00b0	
#define TPXDDC4_PDC4_OEA1_RG		port00b1	
#define TPXDDC4_PDC4_OEA2_RG		port00b2	
#define TPXDDC4_PDC4_OEA3_RG		port00b3	
#define TPXDDC4_PDC4_OEA4_RG		port00b4	
#define TPXDDC4_PDC4_OEA5_RG		port00b5	
#define TPXDDC4_PDC4_OEA6_RG		port00b6	
#define TPXDDC4_PDC4_OEA7_RG		port00b7	

#define TPXDDC4_PDC4_OEB0_RG		port00b8	
#define TPXDDC4_PDC4_OEB1_RG		port00b9	
#define TPXDDC4_PDC4_OEB2_RG		port00ba	
#define TPXDDC4_PDC4_OEB3_RG		port00bb	
#define TPXDDC4_PDC4_OEB4_RG		port00bc	
#define TPXDDC4_PDC4_OEB5_RG		port00bd
#define TPXDDC4_PDC4_OEB6_RG		port00be	
#define TPXDDC4_PDC4_OEB7_RG		port00bf	



//~~~~ DPRAM semaphores
// (DPSEM-0..7 shall be accessed via I/O operation)
#define TPXDDC4_DPSEM0_RG			port0010
#define TPXDDC4_DPSEM1_RG			port0011
#define TPXDDC4_DPSEM2_RG			port0012
#define TPXDDC4_DPSEM3_RG			port0013
#define TPXDDC4_DPSEM4_RG			port0014
#define TPXDDC4_DPSEM5_RG			port0015
#define TPXDDC4_DPSEM6_RG			port0016
#define TPXDDC4_DPSEM7_RG			port0017

//~~~~ FIFO control
#define TPXDDC4_FIFO_DATA_RG		port003c
#define TPXDDC4_FIFO_RESET_RG		port003d	// w-only
#define TPXDDC4_FIFO_CNTR_RG		port00c8

//~~~~ PDC/FIFO input/CLK control, ADC control, status rgs
#define TPXDDC4_PDC_FIFO_ISEL_RG	port00cb	
#define TPXDDC4_ADC_CLK_SEL_RG		port00ca	
#define TPXDDC4_ADC_FIFO_STAT_RG	port00cc	// r-only
#define TPXDDC4_PDC_STAT_RG			port00cd	// r-only
#define TPXDDC4_CLR_ADC_OVF_RG		port003f	// w-only

//~~~~ WDT control control
#define TPXDDC4_WDT_EN_RG			port00e4	
#define TPXDDC4_WDT_RESET_RG		port003e	// w-only	

//~~~~ DSP H/W IRQ control
#define TPXDDC4_MIRQ0_SEL_RG		port00d0	
#define TPXDDC4_MIRQ1_SEL_RG		port00d1	
#define TPXDDC4_MIRQ2_SEL_RG		port00d2	
#define TPXDDC4_MIRQ3_SEL_RG		port00d3	
#define TPXDDC4_MNMI_SEL_RG			port00d4	
#define TPXDDC4_AUX_IRQ_STAT_RG		port00d8	// r-only

//~~~~ GPIO and XSLink control
#define TPXDDC4_GPIO_DATA_RG		port00e2	
#define TPXDDC4_GPIO_DIR_RG			port00e3	
#define TPXDDC4_XSL_FMT_RG			port00e6	


//~~~~ system control/status
#define TPXDDC4_XDMP_RG				port00e0	
#define TPXDDC4_SYS_STAT_RG			port00fe	// r-only	
#define TPXDDC4_BRS_RG				port00ff	// r-only (contain LSB of BRS)


//~~~~~ C54x I/O ports alias defs (required for C compiler) ~~~
ioport	unsigned	port0000;
ioport	unsigned	port0001;
ioport	unsigned	port0002;
ioport	unsigned	port0003;
ioport	unsigned	port0004;
ioport	unsigned	port0005;
ioport	unsigned	port0006;
ioport	unsigned	port0007;
ioport	unsigned	port0008;
ioport	unsigned	port0009;
ioport	unsigned	port000a;
ioport	unsigned	port000b;
ioport	unsigned	port000c;
ioport	unsigned	port000d;
ioport	unsigned	port000e;
ioport	unsigned	port000f;
ioport	unsigned	port0010;
ioport	unsigned	port0011;
ioport	unsigned	port0012;
ioport	unsigned	port0013;
ioport	unsigned	port0014;
ioport	unsigned	port0015;
ioport	unsigned	port0016;
ioport	unsigned	port0017;
ioport	unsigned	port003c;
ioport	unsigned	port003d;
ioport	unsigned	port003e;
ioport	unsigned	port003f;
ioport	unsigned	port0040;
ioport	unsigned	port0041;
ioport	unsigned	port0042;
ioport	unsigned	port0043;
ioport	unsigned	port0044;
ioport	unsigned	port0045;
ioport	unsigned	port0046;
ioport	unsigned	port0047;
ioport	unsigned	port0048;
ioport	unsigned	port0049;
ioport	unsigned	port004a;
ioport	unsigned	port004b;
ioport	unsigned	port004c;
ioport	unsigned	port004d;
ioport	unsigned	port004e;
ioport	unsigned	port004f;
ioport	unsigned	port0050;
ioport	unsigned	port0051;
ioport	unsigned	port0052;
ioport	unsigned	port0053;
ioport	unsigned	port0054;
ioport	unsigned	port0055;
ioport	unsigned	port0056;
ioport	unsigned	port0057;
ioport	unsigned	port0058;
ioport	unsigned	port0059;
ioport	unsigned	port005a;
ioport	unsigned	port005b;
ioport	unsigned	port005c;
ioport	unsigned	port005d;
ioport	unsigned	port005e;
ioport	unsigned	port005f;
ioport	unsigned	port0080;
ioport	unsigned	port0081;
ioport	unsigned	port0082;
ioport	unsigned	port0083;
ioport	unsigned	port0084;
ioport	unsigned	port0085;
ioport	unsigned	port0086;
ioport	unsigned	port0087;
ioport	unsigned	port0088;
ioport	unsigned	port0089;
ioport	unsigned	port008a;
ioport	unsigned	port008b;
ioport	unsigned	port008c;
ioport	unsigned	port008d;
ioport	unsigned	port008e;
ioport	unsigned	port008f;
ioport	unsigned	port0090;
ioport	unsigned	port0091;
ioport	unsigned	port0092;
ioport	unsigned	port0093;
ioport	unsigned	port0094;
ioport	unsigned	port0095;
ioport	unsigned	port0096;
ioport	unsigned	port0097;
ioport	unsigned	port0098;
ioport	unsigned	port0099;
ioport	unsigned	port009a;
ioport	unsigned	port009b;
ioport	unsigned	port009c;
ioport	unsigned	port009d;
ioport	unsigned	port009e;
ioport	unsigned	port009f;
ioport	unsigned	port00a0;
ioport	unsigned	port00a1;
ioport	unsigned	port00a2;
ioport	unsigned	port00a3;
ioport	unsigned	port00a4;
ioport	unsigned	port00a5;
ioport	unsigned	port00a6;
ioport	unsigned	port00a7;
ioport	unsigned	port00a8;
ioport	unsigned	port00a9;
ioport	unsigned	port00aa;
ioport	unsigned	port00ab;
ioport	unsigned	port00ac;
ioport	unsigned	port00ad;
ioport	unsigned	port00ae;
ioport	unsigned	port00af;
ioport	unsigned	port00b0;
ioport	unsigned	port00b1;
ioport	unsigned	port00b2;
ioport	unsigned	port00b3;
ioport	unsigned	port00b4;
ioport	unsigned	port00b5;
ioport	unsigned	port00b6;
ioport	unsigned	port00b7;
ioport	unsigned	port00b8;
ioport	unsigned	port00b9;
ioport	unsigned	port00ba;
ioport	unsigned	port00bb;
ioport	unsigned	port00bc;
ioport	unsigned	port00bd;
ioport	unsigned	port00be;
ioport	unsigned	port00bf;

ioport	unsigned	port00c8;
ioport	unsigned	port00ca;
ioport	unsigned	port00cb;
ioport	unsigned	port00cc;
ioport	unsigned	port00cd;
ioport	unsigned	port00d0;
ioport	unsigned	port00d1;
ioport	unsigned	port00d2;
ioport	unsigned	port00d3;
ioport	unsigned	port00d4;
ioport	unsigned	port00d8;
ioport	unsigned	port00e0;
ioport	unsigned	port00e2;
ioport	unsigned	port00e3;
ioport	unsigned	port00e4;
ioport	unsigned	port00e6;
ioport	unsigned	port00fe;
ioport	unsigned	port00ff;
//================================================================


//=================== constant defs ==============================

#define	TPXDDC4_FPGA_DATA_BITMASK	0xff	// read-back bitmask
						// for all FPGA registers


//--------- MIRQ-0..3/NMI selector codes
#define	TPXDDC4_MIRQ_SEL_BITMASK	0x1f	// read-back bitmask

#define	TPXDDC4_MIRQ_SEL_OFF		0x0	// no IRQ source, power-on default
#define	TPXDDC4_MIRQ_SEL_DPIRQ		0x1
#define	TPXDDC4_MIRQ_SEL_UART1		0x2
#define	TPXDDC4_MIRQ_SEL_UART2		0x3
#define	TPXDDC4_MIRQ_SEL_GPIO0		0x4
#define	TPXDDC4_MIRQ_SEL_GPIO1		0x5
#define	TPXDDC4_MIRQ_SEL_ADC1_OVF	0x6
#define	TPXDDC4_MIRQ_SEL_ADC2_OVF	0x7
#define	TPXDDC4_MIRQ_SEL_ADC12_OVF	0x8
#define	TPXDDC4_MIRQ_SEL_FIFO_WCCL_END	0x9
#define	TPXDDC4_MIRQ_SEL_FIFO_EF	0xa
#define	TPXDDC4_MIRQ_SEL_FIFO_FF	0xb
#define	TPXDDC4_MIRQ_SEL_PDC1_IRQ	0xc	// valid for MIRQ-0..3 only
#define	TPXDDC4_MIRQ_SEL_PDC2_IRQ	0xd     // valid for MIRQ-0..3 only
#define	TPXDDC4_MIRQ_SEL_PDC3_IRQ	0xe     // valid for MIRQ-0..3 only
#define	TPXDDC4_MIRQ_SEL_PDC4_IRQ	0xf     // valid for MIRQ-0..3 only
#define	TPXDDC4_MIRQ_SEL_PDC1_RDY	0x10    // valid for MIRQ-0..3 only
#define	TPXDDC4_MIRQ_SEL_PDC2_RDY	0x11    // valid for MIRQ-0..3 only
#define	TPXDDC4_MIRQ_SEL_PDC3_RDY	0x12    // valid for MIRQ-0..3 only
#define	TPXDDC4_MIRQ_SEL_PDC4_RDY	0x13    // valid for MIRQ-0..3 only


//--------- AUX IRQ status defs
#define	TPXDDC4_DPIRQ_BITMASK		0x1
#define	TPXDDC4_UART1_IRQ_BITMASK	0x2
#define	TPXDDC4_UART2_IRQ_BITMASK	0x4


//---------------- XDMP rg defs ----------------------
#define	TPXDDC4_XDM_PAGE_BITMASK	0x0f
#define	TPXDDC4_XDM_SEL_BITMASK		0x80

#define	TPXDDC4_XDM_SEL_DPRAM		0x00
#define	TPXDDC4_XDM_SEL_FLASH		0x80

//---------------- SYS STATUS rg defs ----------------------
#define	TPXDDC4_BMODE_BITMASK					0x3
#define	TPXDDC4_SA_MODE_BITMASK					0x4
#define	TPXDDC4_DSP_TYPE_BITMASK				0x80
#define	TPXDDC4_REV_ID_BITMASK					0x20	// has been added in rev.1B board
#define	TPXDDC4_PFG_REF_CLK_FREQ_ID_BITMASK		0x10	// has been added in rev.1B board

#define	TPXDDC4_BMODE_NONE						0
#define	TPXDDC4_BMODE_NONE1						3
#define	TPXDDC4_BMODE_FLASH						1
#define	TPXDDC4_BMODE_HPI						2

#define	TPXDDC4_SA_MODE_OFF						0
#define	TPXDDC4_SA_MODE_ON						4

#define	TPXDDC4_DSP_TYPE_C5410					0
#define	TPXDDC4_DSP_TYPE_C5416					0x80

#define	TPXDDC4_REV_ID_REV1A					0		// rev.1A board
#define	TPXDDC4_REV_ID_REV1B					0x20	// rev.1B board

#define	TPXDDC4_PFG_REF_CLK_FREQ_ID_10MHZ		0		// regular 10MHz clock 50PPM
#define	TPXDDC4_PFG_REF_CLK_FREQ_ID_HS19MHZ		0x10	// high-stability 2ppm 19.68 MHZ clock for TELECOM applications


//---------------- WDT defs ----------------------
#define	TPXDDC4_WDT_DISABLE				0
#define	TPXDDC4_WDT_ENABLE				1

//------------- FIFO control rg defs ----------------
#define	TPXDDC4_FIFO_MODE_BITMASK		1
#define	TPXDDC4_FIFO_DAQ_RUN_BITMASK	2
#define	TPXDDC4_FIFO_DAQTF_SEL_BITMASK	4
#define	TPXDDC4_FIFO_DAQ_END_BITMASK	0x20

#define	TPXDDC4_FIFO_MODE_CNF			0
#define	TPXDDC4_FIFO_MODE_DAQ			1

#define	TPXDDC4_FIFO_DAQ_RUN			0x2
#define	TPXDDC4_FIFO_DAQ_ABORT			0x10

#define	TPXDDC4_FIFO_DAQTF_SEL_FF		0
#define	TPXDDC4_FIFO_DAQTF_SEL_PAF		4

//------------- ADC clk selector rg defs ----------------
#define	TPXDDC4_ADC1_CLK_SEL_BITMASK	0x3
#define	TPXDDC4_ADC2_CLK_SEL_BITMASK	0xc
#define	TPXDDC4_PFG1_AUX_FDIV_BITMASK	0x10
#define	TPXDDC4_PFG2_AUX_FDIV_BITMASK	0x20
#define	TPXDDC4_ADC1_CLK_SEL_SHIFT		0
#define	TPXDDC4_ADC2_CLK_SEL_SHIFT		2
#define	TPXDDC4_PFG1_AUX_FDIV_SHIFT		4
#define	TPXDDC4_PFG2_AUX_FDIV_SHIFT		5


#define	TPXDDC4_ADC1_CLK_SEL_PFG1	0x0
#define	TPXDDC4_ADC1_CLK_SEL_PFG2	0x1
#define	TPXDDC4_ADC1_CLK_SEL_XFS1	0x2
#define	TPXDDC4_ADC1_CLK_SEL_XFS2	0x3
#define	TPXDDC4_ADC2_CLK_SEL_PFG1	0x0
#define	TPXDDC4_ADC2_CLK_SEL_PFG2	0x4
#define	TPXDDC4_ADC2_CLK_SEL_XFS1	0x8
#define	TPXDDC4_ADC2_CLK_SEL_XFS2	0xc
#define	TPXDDC4_PFG1_AUX_FDIV_OFF	0
#define	TPXDDC4_PFG1_AUX_FDIV_ON	0x10
#define	TPXDDC4_PFG2_AUX_FDIV_OFF	0
#define	TPXDDC4_PFG2_AUX_FDIV_ON	0x20

#define	TPXDDC4_ADC_CLK_SEL_PFG1	0x0
#define	TPXDDC4_ADC_CLK_SEL_PFG2	0x1
#define	TPXDDC4_ADC_CLK_SEL_XFS1	0x2
#define	TPXDDC4_ADC_CLK_SEL_XFS2	0x3
#define	TPXDDC4_PFG_AUX_FDIV_OFF	0
#define	TPXDDC4_PFG_AUX_FDIV_ON		1


//------------- PDC/FIFO input selector rg defs ----------------
#define	TPXDDC4_PDC12_ISEL_BITMASK	0x1
#define	TPXDDC4_PDC34_ISEL_BITMASK	0x2
#define	TPXDDC4_FIFO_ISEL_BITMASK	0x4
#define	TPXDDC4_PDC12_ISEL_SHIFT	0
#define	TPXDDC4_PDC34_ISEL_SHIFT	1
#define	TPXDDC4_FIFO_ISEL_SHIFT		2

#define	TPXDDC4_PDC12_ISEL_ADC1		0x0
#define	TPXDDC4_PDC12_ISEL_ADC2		0x1
#define	TPXDDC4_PDC34_ISEL_ADC1		0x0
#define	TPXDDC4_PDC34_ISEL_ADC2		0x2
#define	TPXDDC4_FIFO_ISEL_ADC1		0x0
#define	TPXDDC4_FIFO_ISEL_ADC2		0x4

#define	TPXDDC4_PDC_FIFO_ISEL_ADC1	0x0	// used by macros below
#define	TPXDDC4_PDC_FIFO_ISEL_ADC2	0x1


//------------- ADC/PDC/FIFO status rg defs ----------------
#define	TPXDDC4_ADC1_OVF_BITMASK	0x20
#define	TPXDDC4_ADC2_OVF_BITMASK	0x40
#define	TPXDDC4_ADC12_OVF_BITMASK	0x80
#define	TPXDDC4_FIFO_EF_BITMASK		0x1
#define	TPXDDC4_FIFO_PAE_BITMASK	0x2
#define	TPXDDC4_FIFO_FF_BITMASK		0x4
#define	TPXDDC4_FIFO_PAF_BITMASK	0x8

#define	TPXDDC4_PDC1_IRQ_BITMASK	0x10
#define	TPXDDC4_PDC2_IRQ_BITMASK	0x20
#define	TPXDDC4_PDC3_IRQ_BITMASK	0x40
#define	TPXDDC4_PDC4_IRQ_BITMASK	0x80
#define	TPXDDC4_PDC1_RDY_BITMASK	0x1
#define	TPXDDC4_PDC2_RDY_BITMASK	0x2
#define	TPXDDC4_PDC3_RDY_BITMASK	0x4
#define	TPXDDC4_PDC4_RDY_BITMASK	0x8


//------------- GPIO-0/1 defs ----------------
#define	TPXDDC4_GPIO0_DIR_BITMASK	1
#define	TPXDDC4_GPIO1_DIR_BITMASK	2
#define	TPXDDC4_GPIO0_DATA_BITMASK	1
#define	TPXDDC4_GPIO1_DATA_BITMASK	2
#define	TPXDDC4_GPIO0_DIR_SHIFT		0
#define	TPXDDC4_GPIO1_DIR_SHIFT		1
#define	TPXDDC4_GPIO0_DATA_SHIFT	0
#define	TPXDDC4_GPIO1_DATA_SHIFT	1

#define	TPXDDC4_GPIO_DIR_INPUT		0
#define	TPXDDC4_GPIO_DIR_OUTPUT		1

#define	TPXDDC4_GPIO0_DIR_INPUT		0
#define	TPXDDC4_GPIO0_DIR_OUTPUT	1
#define	TPXDDC4_GPIO1_DIR_INPUT		0
#define	TPXDDC4_GPIO1_DIR_OUTPUT	2

#define	TPXDDC4_GPIO_DATA_OFF		0
#define	TPXDDC4_GPIO_DATA_ON		1

#define	TPXDDC4_GPIO0_DATA_OFF		0
#define	TPXDDC4_GPIO0_DATA_ON		1
#define	TPXDDC4_GPIO1_DATA_OFF		0
#define	TPXDDC4_GPIO1_DATA_ON		2


//------------- XSL format rg defs ----------------
#define	TPXDDC4_XSL1_FMT_BITMASK	0x3
#define	TPXDDC4_XSL2_FMT_BITMASK	0xC
#define	TPXDDC4_XSL_CLKPOL_BITMASK	0x10
#define	TPXDDC4_XSL_CLKFRM_BITMASK	0x20
#define	TPXDDC4_XSL1_FMT_SHIFT		0
#define	TPXDDC4_XSL2_FMT_SHIFT		2

#define	TPXDDC4_XSL_FMT_WLEN8		0
#define	TPXDDC4_XSL_FMT_WLEN16		1
#define	TPXDDC4_XSL_FMT_WLEN24		2
#define	TPXDDC4_XSL_FMT_WLEN30		3

#define	TPXDDC4_XSL1_FMT_WLEN8		0
#define	TPXDDC4_XSL1_FMT_WLEN16		1
#define	TPXDDC4_XSL1_FMT_WLEN24		2
#define	TPXDDC4_XSL1_FMT_WLEN30		3
#define	TPXDDC4_XSL2_FMT_WLEN8		0
#define	TPXDDC4_XSL2_FMT_WLEN16		4
#define	TPXDDC4_XSL2_FMT_WLEN24		8
#define	TPXDDC4_XSL2_FMT_WLEN30		0xC

#define	TPXDDC4_XSL_CLKPOL_POS		0
#define	TPXDDC4_XSL_CLKPOL_NEG		0x10
#define	TPXDDC4_XSL_CLKFRM_OFF		0
#define	TPXDDC4_XSL_CLKFRM_ON		0x20

	//------- 16552D Dual-UART defs (16C450 compatible defs only)
	// (input clk: 1.8432MHz)
#define TPXDDC4_UART_DATA_BITMASK	0xff

#define TPXDDC4_UART_DIVL_38400		0x03
#define TPXDDC4_UART_DIVH_38400		0x00
#define TPXDDC4_UART_DIVL_19200		0x06
#define TPXDDC4_UART_DIVH_19200		0x00
#define TPXDDC4_UART_DIVL_9600		0x0c
#define TPXDDC4_UART_DIVH_9600		0x00
#define TPXDDC4_UART_DIVL_4800		0x18
#define TPXDDC4_UART_DIVH_4800		0x00
#define TPXDDC4_UART_DIVL_2400		0x30
#define TPXDDC4_UART_DIVH_2400		0x00
#define TPXDDC4_UART_DIVL_1200		0x60
#define TPXDDC4_UART_DIVH_1200		0x00
#define TPXDDC4_UART_DIVL_600		0xc0
#define TPXDDC4_UART_DIVH_600		0x00

#define TPXDDC4_UART_IE_DR		0x01
#define TPXDDC4_UART_IE_TEMPTY		0x02
#define TPXDDC4_UART_IE_RLINESTAT	0x04
#define TPXDDC4_UART_IE_MSTAT		0x08

#define TPXDDC4_UART_LCNTR_DLAB		0x80
#define TPXDDC4_UART_LCNTR_WLEN5	0x00
#define TPXDDC4_UART_LCNTR_WLEN6	0x01
#define TPXDDC4_UART_LCNTR_WLEN7	0x02
#define TPXDDC4_UART_LCNTR_WLEN8	0x03
#define TPXDDC4_UART_LCNTR_STOP1	0x00
#define TPXDDC4_UART_LCNTR_STOP2	0x04
#define TPXDDC4_UART_LCNTR_NOPAR	0x00
#define TPXDDC4_UART_LCNTR_ODDPAR	0x08
#define TPXDDC4_UART_LCNTR_EVENPAR	0x18

#define TPXDDC4_UART_IID_BITMASK	0x06
#define TPXDDC4_UART_IID_RLINESTAT	0x00
#define TPXDDC4_UART_IID_TEMPTY		0x04
#define TPXDDC4_UART_IID_MSTAT		0x06

#define TPXDDC4_UART_MCNTR_DTR		0x01
#define TPXDDC4_UART_MCNTR_RTS		0x02

#define TPXDDC4_UART_LSTAT_DR		0x01
#define TPXDDC4_UART_LSTAT_OE		0x02
#define TPXDDC4_UART_LSTAT_PE		0x04
#define TPXDDC4_UART_LSTAT_FE		0x08
#define TPXDDC4_UART_LSTAT_BI		0x10
#define TPXDDC4_UART_LSTAT_THEMPY	0x20
#define TPXDDC4_UART_LSTAT_TEMPTY	0x40

#define TPXDDC4_UART_MSTAT_DCTS		0x01
#define TPXDDC4_UART_MSTAT_DDSR		0x02
#define TPXDDC4_UART_MSTAT_TERI		0x04
#define TPXDDC4_UART_MSTAT_DDCD		0x08
#define TPXDDC4_UART_MSTAT_CTS		0x10
#define TPXDDC4_UART_MSTAT_DSR		0x20
#define TPXDDC4_UART_MSTAT_RI		0x40
#define TPXDDC4_UART_MSTAT_DCD		0x80
//****************************************************************


//************* Macros for I/O Control ***************************

#define TPXDDC4_sys_stat()				(TPXDDC4_SYS_STAT_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_bmode()					(TPXDDC4_SYS_STAT_RG & TPXDDC4_BMODE_BITMASK)
#define TPXDDC4_sa_mode()				(TPXDDC4_SYS_STAT_RG & TPXDDC4_SA_MODE_BITMASK)
#define TPXDDC4_dsp_type()				(TPXDDC4_SYS_STAT_RG & TPXDDC4_DSP_TYPE_BITMASK)
#define TPXDDC4_rev_id()				(TPXDDC4_SYS_STAT_RG & TPXDDC4_REV_ID_BITMASK)
#define TPXDDC4_pfg_ref_clk_freq_id()	(TPXDDC4_SYS_STAT_RG & TPXDDC4_PFG_REF_CLK_FREQ_ID_BITMASK)

	// condition check macros for use with IF() operator
#define TPXDDC4_REV_ID_IS_REV1A				(TPXDDC4_rev_id()==TPXDDC4_REV_ID_REV1A)
#define TPXDDC4_REV_ID_IS_REV1B				(TPXDDC4_rev_id()==TPXDDC4_REV_ID_REV1B)

#define TPXDDC4_DSP_TYPE_IS_C5410  			(TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5416)
#define TPXDDC4_DSP_TYPE_IS_C5416  			(TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5416)

#define TPXDDC4_PFG_REF_CLK_FREQ_IS_10MHZ	(TPXDDC4_pfg_ref_clk_freq_id()==TPXDDC4_PFG_REF_CLK_FREQ_ID_10MHZ)
#define TPXDDC4_PFG_REF_CLK_FREQ_IS_HS19MHZ	(TPXDDC4_pfg_ref_clk_freq_id()==TPXDDC4_PFG_REF_CLK_FREQ_ID_HS19MHZ)



#define TPXDDC4_set_xdmp(v)		TPXDDC4_XDMP_RG=v
#define TPXDDC4_xdmp()			(TPXDDC4_XDMP_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_xdm_page(v)		TPXDDC4_XDMP_RG=((v & TPXDDC4_XDM_PAGE_BITMASK) | (TPXDDC4_XDMP_RG & ~TPXDDC4_XDM_PAGE_BITMASK))
#define TPXDDC4_xdm_page()		(TPXDDC4_XDMP_RG & TPXDDC4_XDM_PAGE_BITMASK)
#define TPXDDC4_set_xdm_sel(v)		TPXDDC4_XDMP_RG=((v & TPXDDC4_XDM_SEL_BITMASK) | (TPXDDC4_XDMP_RG & ~TPXDDC4_XDM_SEL_BITMASK))
#define TPXDDC4_xdm_sel()		(TPXDDC4_XDMP_RG & TPXDDC4_XDM_SEL_BITMASK)

#define TPXDDC4_set_mnmi_sel(v)		TPXDDC4_MNMI_SEL_RG=v
#define TPXDDC4_mnmi_sel()		(TPXDDC4_MNMI_SEL_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_mirq0_sel(v)	TPXDDC4_MIRQ0_SEL_RG=v
#define TPXDDC4_mirq0_sel()		(TPXDDC4_MIRQ0_SEL_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_mirq1_sel(v)	TPXDDC4_MIRQ1_SEL_RG=v
#define TPXDDC4_mirq1_sel()		(TPXDDC4_MIRQ1_SEL_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_mirq2_sel(v)	TPXDDC4_MIRQ2_SEL_RG=v
#define TPXDDC4_mirq2_sel()		(TPXDDC4_MIRQ2_SEL_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_mirq3_sel(v)	TPXDDC4_MIRQ3_SEL_RG=v
#define TPXDDC4_mirq3_sel()		(TPXDDC4_MIRQ3_SEL_RG & TPXDDC4_FPGA_DATA_BITMASK)

#define TPXDDC4_aux_irq_stat()		(TPXDDC4_AUX_IRQ_STAT_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_dpirq()			(TPXDDC4_AUX_IRQ_STAT_RG & TPXDDC4_DPIRQ_BITMASK)
#define TPXDDC4_uart1_irq()		(TPXDDC4_AUX_IRQ_STAT_RG & TPXDDC4_UART1_IRQ_BITMASK)
#define TPXDDC4_uart2_irq()		(TPXDDC4_AUX_IRQ_STAT_RG & TPXDDC4_UART2_IRQ_BITMASK)

#define TPXDDC4_fifo()			(TPXDDC4_FIFO_DATA_RG & TPXDDC4_FIFO_DATA_BITMASK)
#define TPXDDC4_set_fifo_reset()	TPXDDC4_FIFO_RESET_RG=0
#define TPXDDC4_SET_FIFO_RESET		TPXDDC4_set_fifo_reset()

#define TPXDDC4_set_fifo_cntr(v)	TPXDDC4_FIFO_CNTR_RG=v
#define TPXDDC4_fifo_cntr()		(TPXDDC4_FIFO_CNTR_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_fifo_mode(v)	TPXDDC4_FIFO_CNTR_RG=((v & TPXDDC4_FIFO_MODE_BITMASK) | (TPXDDC4_FIFO_CNTR_RG & ~TPXDDC4_FIFO_MODE_BITMASK))
#define TPXDDC4_fifo_mode()		(TPXDDC4_FIFO_CNTR_RG & TPXDDC4_FIFO_MODE_BITMASK)
#define TPXDDC4_set_fifo_daq_run()	TPXDDC4_FIFO_CNTR_RG=(TPXDDC4_FIFO_DAQ_RUN | (TPXDDC4_FIFO_CNTR_RG & (TPXDDC4_FIFO_MODE_BITMASK | TPXDDC4_FIFO_DAQTF_SEL_BITMASK)))
#define TPXDDC4_fifo_daq_run()		(TPXDDC4_FIFO_CNTR_RG & TPXDDC4_FIFO_DAQ_RUN_BITMASK)
#define TPXDDC4_set_fifo_daq_abort()	TPXDDC4_FIFO_CNTR_RG=(TPXDDC4_FIFO_DAQ_ABORT | (TPXDDC4_FIFO_CNTR_RG & (TPXDDC4_FIFO_MODE_BITMASK | TPXDDC4_FIFO_DAQTF_SEL_BITMASK)))
#define TPXDDC4_fifo_daq_end()		(TPXDDC4_FIFO_CNTR_RG & TPXDDC4_FIFO_DAQ_END_BITMASK)
#define TPXDDC4_set_fifo_daqtf_sel(v)	TPXDDC4_FIFO_CNTR_RG=((v & TPXDDC4_FIFO_DAQTF_SEL_BITMASK) | (TPXDDC4_FIFO_CNTR_RG & ~TPXDDC4_FIFO_DAQTF_SEL_BITMASK))
#define TPXDDC4_fifo_daqtf_sel()	(TPXDDC4_FIFO_CNTR_RG & TPXDDC4_FIFO_DAQTF_SEL_BITMASK)

#define TPXDDC4_set_fifo_mode_cnf()	TPXDDC4_set_fifo_mode(TPXDDC4_FIFO_MODE_CNF)
#define TPXDDC4_set_fifo_mode_daq()	TPXDDC4_set_fifo_mode(TPXDDC4_FIFO_MODE_DAQ)
#define TPXDDC4_SET_FIFO_MODE_CNF	TPXDDC4_set_fifo_mode_cnf()
#define TPXDDC4_SET_FIFO_MODE_DAQ	TPXDDC4_set_fifo_mode_daq()

#define TPXDDC4_SET_FIFO_DAQ_RUN	TPXDDC4_set_fifo_daq_run()
#define TPXDDC4_SET_FIFO_DAQ_ABORT	TPXDDC4_set_fifo_daq_abort()

#define TPXDDC4_SET_FIFO_DAQTF_SEL_FF	TPXDDC4_set_fifo_daqtf_sel(TPXDDC4_FIFO_DAQTF_SEL_FF)
#define TPXDDC4_SET_FIFO_DAQTF_SEL_PAF	TPXDDC4_set_fifo_daqtf_sel(TPXDDC4_FIFO_DAQTF_SEL_PAF)


#define TPXDDC4_set_adc_clk_sel(v)	TPXDDC4_ADC_CLK_SEL_RG=v
#define TPXDDC4_adc_clk_sel()		(TPXDDC4_ADC_CLK_SEL_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_adc1_clk_sel(v)	TPXDDC4_ADC_CLK_SEL_RG=((v & TPXDDC4_ADC1_CLK_SEL_BITMASK) | (TPXDDC4_ADC_CLK_SEL_RG & ~TPXDDC4_ADC1_CLK_SEL_BITMASK))
#define TPXDDC4_adc1_clk_sel()		(TPXDDC4_ADC_CLK_SEL_RG & TPXDDC4_ADC1_CLK_SEL_BITMASK)
#define TPXDDC4_set_adc2_clk_sel(v)	TPXDDC4_ADC_CLK_SEL_RG=(((v<<TPXDDC4_ADC2_CLK_SEL_SHIFT) & TPXDDC4_ADC2_CLK_SEL_BITMASK) | (TPXDDC4_ADC_CLK_SEL_RG & ~TPXDDC4_ADC2_CLK_SEL_BITMASK))
#define TPXDDC4_adc2_clk_sel()		((TPXDDC4_ADC_CLK_SEL_RG & TPXDDC4_ADC2_CLK_SEL_BITMASK)>>TPXDDC4_ADC2_CLK_SEL_SHIFT)
#define TPXDDC4_set_pfg1_aux_fdiv(v)	TPXDDC4_ADC_CLK_SEL_RG=(((v<<TPXDDC4_PFG1_AUX_FDIV_SHIFT) & TPXDDC4_PFG1_AUX_FDIV_BITMASK) | (TPXDDC4_ADC_CLK_SEL_RG & ~TPXDDC4_PFG1_AUX_FDIV_BITMASK))
#define TPXDDC4_pfg1_aux_fdiv()		((TPXDDC4_ADC_CLK_SEL_RG & TPXDDC4_PFG1_AUX_FDIV_BITMASK)>>TPXDDC4_PFG1_AUX_FDIV_SHIFT)
#define TPXDDC4_set_pfg2_aux_fdiv(v)	TPXDDC4_ADC_CLK_SEL_RG=(((v<<TPXDDC4_PFG2_AUX_FDIV_SHIFT) & TPXDDC4_PFG2_AUX_FDIV_BITMASK) | (TPXDDC4_ADC_CLK_SEL_RG & ~TPXDDC4_PFG2_AUX_FDIV_BITMASK))
#define TPXDDC4_pfg2_aux_fdiv()		((TPXDDC4_ADC_CLK_SEL_RG & TPXDDC4_PFG2_AUX_FDIV_BITMASK)>>TPXDDC4_PFG2_AUX_FDIV_SHIFT)

#define TPXDDC4_set_pdc_fifo_isel(v)	TPXDDC4_PDC_FIFO_ISEL_RG=v
#define TPXDDC4_pdc_fifo_isel()		(TPXDDC4_PDC_FIFO_ISEL_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_pdc12_isel(v)	TPXDDC4_PDC_FIFO_ISEL_RG=((v & TPXDDC4_PDC12_ISEL_BITMASK) | (TPXDDC4_PDC_FIFO_ISEL_RG & ~TPXDDC4_PDC12_ISEL_BITMASK))
#define TPXDDC4_pdc12_isel		(TPXDDC4_PDC_FIFO_ISEL_RG & TPXDDC4_PDC12_ISEL_BITMASK)
#define TPXDDC4_set_pdc34_isel(v)	TPXDDC4_PDC_FIFO_ISEL_RG=(((v<<1) & TPXDDC4_PDC34_ISEL_BITMASK) | (TPXDDC4_PDC_FIFO_ISEL_RG & ~TPXDDC4_PDC34_ISEL_BITMASK))
#define TPXDDC4_pdc34_isel		((TPXDDC4_PDC_FIFO_ISEL_RG & TPXDDC4_PDC34_ISEL_BITMASK)>>1)
#define TPXDDC4_set_fifo_isel(v)	TPXDDC4_PDC_FIFO_ISEL_RG=(((v<<2) & TPXDDC4_FIFO_ISEL_BITMASK) | (TPXDDC4_PDC_FIFO_ISEL_RG & ~TPXDDC4_FIFO_ISEL_BITMASK))
#define TPXDDC4_fifo_isel		((TPXDDC4_PDC_FIFO_ISEL_RG & TPXDDC4_FIFO_ISEL_BITMASK)>>2)

#define TPXDDC4_adc_fifo_stat()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_fifo_ef()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_FIFO_EF_BITMASK)
#define TPXDDC4_fifo_pae()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_FIFO_PAE_BITMASK)
#define TPXDDC4_fifo_ff()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_FIFO_FF_BITMASK)
#define TPXDDC4_fifo_paf()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_FIFO_PAF_BITMASK)
#define TPXDDC4_adc1_ovf()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_ADC1_OVF_BITMASK)
#define TPXDDC4_adc2_ovf()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_ADC2_OVF_BITMASK)
#define TPXDDC4_adc12_ovf()		(TPXDDC4_ADC_FIFO_STAT_RG & TPXDDC4_ADC12_OVF_BITMASK)

#define TPXDDC4_pdc_stat()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_pdc1_irq()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC1_IRQ_BITMASK)
#define TPXDDC4_pdc2_irq()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC2_IRQ_BITMASK)
#define TPXDDC4_pdc3_irq()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC3_IRQ_BITMASK)
#define TPXDDC4_pdc4_irq()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC4_IRQ_BITMASK)
#define TPXDDC4_pdc1_rdy()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC1_RDY_BITMASK)
#define TPXDDC4_pdc2_rdy()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC2_RDY_BITMASK)
#define TPXDDC4_pdc3_rdy()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC3_RDY_BITMASK)
#define TPXDDC4_pdc4_rdy()		(TPXDDC4_PDC_STAT_RG & TPXDDC4_PDC4_RDY_BITMASK)

#define TPXDDC4_clr_adc_ovf()		TPXDDC4_CLR_ADC_OVF_RG=0

#define TPXDDC4_wdt_reset()		TPXDDC4_WDT_RESET_RG=0
#define TPXDDC4_set_wdt_enable(v)	TPXDDC4_WDT_EN_RG=v
#define TPXDDC4_wdt_enable()		(TPXDDC4_WDT_EN_RG & TPXDDC4_FPGA_DATA_BITMASK)

#define TPXDDC4_set_gpio_dir(v)		TPXDDC4_GPIO_DIR_RG=v
#define TPXDDC4_gpio_dir()		(TPXDDC4_GPIO_DIR_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_gpio0_dir(v)	TPXDDC4_GPIO_DIR_RG= ((v & TPXDDC4_GPIO0_DIR_BITMASK) | (TPXDDC4_GPIO_DIR_RG & ~TPXDDC4_GPIO0_DIR_BITMASK))
#define TPXDDC4_gpio0_dir()		(TPXDDC4_GPIO_DIR_RG & TPXDDC4_GPIO0_DIR_BITMASK)
#define TPXDDC4_set_gpio1_dir(v)	TPXDDC4_GPIO_DIR_RG= (((v<<1) & TPXDDC4_GPIO1_DIR_BITMASK) | (TPXDDC4_GPIO_DIR_RG & ~TPXDDC4_GPIO1_DIR_BITMASK))
#define TPXDDC4_gpio1_dir()		((TPXDDC4_GPIO_DIR_RG & TPXDDC4_GPIO1_DIR_BITMASK)>>1)
#define TPXDDC4_set_gpio_data(v)	TPXDDC4_GPIO_DATA_RG=v
#define TPXDDC4_gpio_data()		(TPXDDC4_GPIO_DATA_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_gpio0_data(v)	TPXDDC4_GPIO_DATA_RG= ((v & TPXDDC4_GPIO0_DATA_BITMASK) | (TPXDDC4_GPIO_DATA_RG & ~TPXDDC4_GPIO0_DATA_BITMASK))
#define TPXDDC4_gpio0_data()		(TPXDDC4_GPIO_DATA_RG & TPXDDC4_GPIO0_DATA_BITMASK)
#define TPXDDC4_set_gpio1_data(v)	TPXDDC4_GPIO_DATA_RG= (((v<<1) & TPXDDC4_GPIO1_DATA_BITMASK) | (TPXDDC4_GPIO_DATA_RG & ~TPXDDC4_GPIO1_DATA_BITMASK))
#define TPXDDC4_gpio1_data()		((TPXDDC4_GPIO_DATA_RG & TPXDDC4_GPIO1_DATA_BITMASK)>>1)

#define TPXDDC4_set_xsl_fmt(v)		TPXDDC4_XSL_FMT_RG=v
#define TPXDDC4_xsl_fmt()		(TPXDDC4_XSL_FMT_RG & TPXDDC4_FPGA_DATA_BITMASK)
#define TPXDDC4_set_xsl1_fmt(v)		TPXDDC4_XSL_FMT_RG= ((v & TPXDDC4_XSL1_FMT_BITMASK) | (TPXDDC4_XSL_FMT_RG & ~TPXDDC4_XSL1_FMT_BITMASK))
#define TPXDDC4_xsl1_fmt()		(TPXDDC4_XSL_FMT_RG & TPXDDC4_XSL1_FMT_BITMASK)
#define TPXDDC4_set_xsl2_fmt(v)		TPXDDC4_XSL_FMT_RG= (((v<<TPXDDC4_XSL2_FMT_SHIFT) & TPXDDC4_XSL2_FMT_BITMASK) | (TPXDDC4_XSL_FMT_RG & ~TPXDDC4_XSL2_FMT_BITMASK))
#define TPXDDC4_xsl2_fmt()		((TPXDDC4_XSL_FMT_RG & TPXDDC4_XSL2_FMT_BITMASK)>>TPXDDC4_XSL2_FMT_SHIFT)
#define TPXDDC4_set_xsl_clkpol(v)	TPXDDC4_XSL_FMT_RG= ((v & TPXDDC4_XSL_CLKPOL_BITMASK) | (TPXDDC4_XSL_FMT_RG & ~TPXDDC4_XSL_CLKPOL_BITMASK))
#define TPXDDC4_xsl_clkpol()		(TPXDDC4_XSL_FMT_RG & TPXDDC4_XSL_CLKPOL_BITMASK)
#define TPXDDC4_set_xsl_clkfrm(v)	TPXDDC4_XSL_FMT_RG= ((v & TPXDDC4_XSL_CLKFRM_BITMASK) | (TPXDDC4_XSL_FMT_RG & ~TPXDDC4_XSL_CLKFRM_BITMASK))
#define TPXDDC4_xsl_clkfrm()		(TPXDDC4_XSL_FMT_RG & TPXDDC4_XSL_CLKFRM_BITMASK)
//****************************************************************


//**************** Macros for PDC access *************************

	//--- macros for access to PDC uP I/F
	//    Note that all data transactions are 8-biwide!!!

#define TPXDDC4_set_pdc1_hld0(d)	TPXDDC4_PDC1_HLD0_RG=(d&0xff)
#define TPXDDC4_set_pdc1_hld1(d)	TPXDDC4_PDC1_HLD1_RG=(d&0xff)
#define TPXDDC4_set_pdc1_hld2(d)	TPXDDC4_PDC1_HLD2_RG=(d&0xff)
#define TPXDDC4_set_pdc1_hld3(d)	TPXDDC4_PDC1_HLD3_RG=(d&0xff)
#define TPXDDC4_set_pdc1_wraddr(d)	TPXDDC4_PDC1_WRADDR_RG=(d&0xff)
#define TPXDDC4_set_pdc1_rdaddr(d)	TPXDDC4_PDC1_WRADDR_RG=(d&0xff)
#define TPXDDC4_pdc1_hld0_data()	(TPXDDC4_PDC1_HLD0_RG &0xff)
#define TPXDDC4_pdc1_hld1_data()	(TPXDDC4_PDC1_HLD1_RG &0xff)
#define TPXDDC4_pdc1_hld2_data()	(TPXDDC4_PDC1_HLD2_RG &0xff)
#define TPXDDC4_pdc1_hld3_data()	(TPXDDC4_PDC1_HLD3_RG &0xff)

#define TPXDDC4_set_pdc2_hld0(d)	TPXDDC4_PDC2_HLD0_RG=(d&0xff)
#define TPXDDC4_set_pdc2_hld1(d)	TPXDDC4_PDC2_HLD1_RG=(d&0xff)
#define TPXDDC4_set_pdc2_hld2(d)	TPXDDC4_PDC2_HLD2_RG=(d&0xff)
#define TPXDDC4_set_pdc2_hld3(d)	TPXDDC4_PDC2_HLD3_RG=(d&0xff)
#define TPXDDC4_set_pdc2_wraddr(d)	TPXDDC4_PDC2_WRADDR_RG=(d&0xff)
#define TPXDDC4_set_pdc2_rdaddr(d)	TPXDDC4_PDC2_WRADDR_RG=(d&0xff)
#define TPXDDC4_pdc2_hld0_data()	(TPXDDC4_PDC2_HLD0_RG &0xff)
#define TPXDDC4_pdc2_hld1_data()	(TPXDDC4_PDC2_HLD1_RG &0xff)
#define TPXDDC4_pdc2_hld2_data()	(TPXDDC4_PDC2_HLD2_RG &0xff)
#define TPXDDC4_pdc2_hld3_data()	(TPXDDC4_PDC2_HLD3_RG &0xff)

#define TPXDDC4_set_pdc3_hld0(d)	TPXDDC4_PDC3_HLD0_RG=(d&0xff)
#define TPXDDC4_set_pdc3_hld1(d)	TPXDDC4_PDC3_HLD1_RG=(d&0xff)
#define TPXDDC4_set_pdc3_hld2(d)	TPXDDC4_PDC3_HLD2_RG=(d&0xff)
#define TPXDDC4_set_pdc3_hld3(d)	TPXDDC4_PDC3_HLD3_RG=(d&0xff)
#define TPXDDC4_set_pdc3_wraddr(d)	TPXDDC4_PDC3_WRADDR_RG=(d&0xff)
#define TPXDDC4_set_pdc3_rdaddr(d)	TPXDDC4_PDC3_WRADDR_RG=(d&0xff)
#define TPXDDC4_pdc3_hld0_data()	(TPXDDC4_PDC3_HLD0_RG &0xff)
#define TPXDDC4_pdc3_hld1_data()	(TPXDDC4_PDC3_HLD1_RG &0xff)
#define TPXDDC4_pdc3_hld2_data()	(TPXDDC4_PDC3_HLD2_RG &0xff)
#define TPXDDC4_pdc3_hld3_data()	(TPXDDC4_PDC3_HLD3_RG &0xff)

#define TPXDDC4_set_pdc4_hld0(d)	TPXDDC4_PDC4_HLD0_RG=(d&0xff)
#define TPXDDC4_set_pdc4_hld1(d)	TPXDDC4_PDC4_HLD1_RG=(d&0xff)
#define TPXDDC4_set_pdc4_hld2(d)	TPXDDC4_PDC4_HLD2_RG=(d&0xff)
#define TPXDDC4_set_pdc4_hld3(d)	TPXDDC4_PDC4_HLD3_RG=(d&0xff)
#define TPXDDC4_set_pdc4_wraddr(d)	TPXDDC4_PDC4_WRADDR_RG=(d&0xff)
#define TPXDDC4_set_pdc4_rdaddr(d)	TPXDDC4_PDC4_WRADDR_RG=(d&0xff)
#define TPXDDC4_pdc4_hld0_data()	(TPXDDC4_PDC4_HLD0_RG &0xff)
#define TPXDDC4_pdc4_hld1_data()	(TPXDDC4_PDC4_HLD1_RG &0xff)
#define TPXDDC4_pdc4_hld2_data()	(TPXDDC4_PDC4_HLD2_RG &0xff)
#define TPXDDC4_pdc4_hld3_data()	(TPXDDC4_PDC4_HLD3_RG &0xff)


	//--- macros for read of PDC OEA/OEB data in 16-bit Parallel Direct Mode
	//    16-bit PDC data are availale fm OEA/OEB outputs after
	//    PDC-RDY comes active. Note, that PX/DDC4 extends actual
	//    input PDC_RDY signal and clears internal PDC-RDY flag on
	//    read fm either of the OEA/OEB registers.
#define TPXDDC4_pdc1_aout_data()	TPXDDC4_PDC1_OEA_RG
#define TPXDDC4_pdc1_bout_data()	TPXDDC4_PDC1_OEB_RG

#define TPXDDC4_pdc2_aout_data()	TPXDDC4_PDC2_OEA_RG
#define TPXDDC4_pdc2_bout_data()	TPXDDC4_PDC2_OEB_RG

#define TPXDDC4_pdc3_aout_data()	TPXDDC4_PDC3_OEA_RG
#define TPXDDC4_pdc3_bout_data()	TPXDDC4_PDC3_OEB_RG

#define TPXDDC4_pdc4_aout_data()	TPXDDC4_PDC4_OEA_RG
#define TPXDDC4_pdc4_bout_data()	TPXDDC4_PDC4_OEB_RG


	//--- macros for read of PDC 16-bit data in FIFO Buffer Mode
	//    16-bit PDC FIFO data is composed as LSB=OEB[0..7] and
	//    MSB=OEA[0..7]. Data is available on PDC-INT comes active,
	//    which must be rounted to any of the DSP MIRQ-0..3 interrupt
	//    request inputs. Note, that PX/DDC4 does not extend
	//    PDC_INT input.
	//    Each FIFO data frame contains 7 words (#0..#6),
	//    and the 8-th word (#7) is increment the FIFO frame counter.

#define TPXDDC4_pdc1_fifo_i_data()	(((TPXDDC4_PDC1_OEA0_RG & 0xff)<<8) | (TPXDDC4_PDC1_OEB0_RG & 0xff))
#define TPXDDC4_pdc1_fifo_q_data()	(((TPXDDC4_PDC1_OEA1_RG & 0xff)<<8) | (TPXDDC4_PDC1_OEB1_RG & 0xff))
#define TPXDDC4_pdc1_fifo_mag_data()	(((TPXDDC4_PDC1_OEA2_RG & 0xff)<<8) | (TPXDDC4_PDC1_OEB2_RG & 0xff))
#define TPXDDC4_pdc1_fifo_phase_data()	(((TPXDDC4_PDC1_OEA3_RG & 0xff)<<8) | (TPXDDC4_PDC1_OEB3_RG & 0xff))
#define TPXDDC4_pdc1_fifo_freq_data()	(((TPXDDC4_PDC1_OEA4_RG & 0xff)<<8) | (TPXDDC4_PDC1_OEB4_RG & 0xff))
#define TPXDDC4_pdc1_fifo_mstat_data()	(((TPXDDC4_PDC1_OEA6_RG & 0xfc)<<8) | (TPXDDC4_PDC1_OEB6_RG & 0xff))
#define TPXDDC4_pdc1_fifo_next_frame()	(TPXDDC4_PDC1_OEB7_RG)

#define TPXDDC4_pdc2_fifo_i_data()	(((TPXDDC4_PDC2_OEA0_RG & 0xff)<<8) | (TPXDDC4_PDC2_OEB0_RG & 0xff))
#define TPXDDC4_pdc2_fifo_q_data()	(((TPXDDC4_PDC2_OEA1_RG & 0xff)<<8) | (TPXDDC4_PDC2_OEB1_RG & 0xff))
#define TPXDDC4_pdc2_fifo_mag_data()	(((TPXDDC4_PDC2_OEA2_RG & 0xff)<<8) | (TPXDDC4_PDC2_OEB2_RG & 0xff))
#define TPXDDC4_pdc2_fifo_phase_data()	(((TPXDDC4_PDC2_OEA3_RG & 0xff)<<8) | (TPXDDC4_PDC2_OEB3_RG & 0xff))
#define TPXDDC4_pdc2_fifo_freq_data()	(((TPXDDC4_PDC2_OEA4_RG & 0xff)<<8) | (TPXDDC4_PDC2_OEB4_RG & 0xff))
#define TPXDDC4_pdc2_fifo_mstat_data()	(((TPXDDC4_PDC2_OEA6_RG & 0xfc)<<8) | (TPXDDC4_PDC2_OEB6_RG & 0xff))
#define TPXDDC4_pdc2_fifo_next_frame()	(TPXDDC4_PDC2_OEB7_RG)

#define TPXDDC4_pdc3_fifo_i_data()	(((TPXDDC4_PDC3_OEA0_RG & 0xff)<<8) | (TPXDDC4_PDC3_OEB0_RG & 0xff))
#define TPXDDC4_pdc3_fifo_q_data()	(((TPXDDC4_PDC3_OEA1_RG & 0xff)<<8) | (TPXDDC4_PDC3_OEB1_RG & 0xff))
#define TPXDDC4_pdc3_fifo_mag_data()	(((TPXDDC4_PDC3_OEA2_RG & 0xff)<<8) | (TPXDDC4_PDC3_OEB2_RG & 0xff))
#define TPXDDC4_pdc3_fifo_phase_data()	(((TPXDDC4_PDC3_OEA3_RG & 0xff)<<8) | (TPXDDC4_PDC3_OEB3_RG & 0xff))
#define TPXDDC4_pdc3_fifo_freq_data()	(((TPXDDC4_PDC3_OEA4_RG & 0xff)<<8) | (TPXDDC4_PDC3_OEB4_RG & 0xff))
#define TPXDDC4_pdc3_fifo_mstat_data()	(((TPXDDC4_PDC3_OEA6_RG & 0xfc)<<8) | (TPXDDC4_PDC3_OEB6_RG & 0xff))
#define TPXDDC4_pdc3_fifo_next_frame()	(TPXDDC4_PDC3_OEB7_RG)

#define TPXDDC4_pdc4_fifo_i_data()	(((TPXDDC4_PDC4_OEA0_RG & 0xff)<<8) | (TPXDDC4_PDC4_OEB0_RG & 0xff))
#define TPXDDC4_pdc4_fifo_q_data()	(((TPXDDC4_PDC4_OEA1_RG & 0xff)<<8) | (TPXDDC4_PDC4_OEB1_RG & 0xff))
#define TPXDDC4_pdc4_fifo_mag_data()	(((TPXDDC4_PDC4_OEA2_RG & 0xff)<<8) | (TPXDDC4_PDC4_OEB2_RG & 0xff))
#define TPXDDC4_pdc4_fifo_phase_data()	(((TPXDDC4_PDC4_OEA3_RG & 0xff)<<8) | (TPXDDC4_PDC4_OEB3_RG & 0xff))
#define TPXDDC4_pdc4_fifo_freq_data()	(((TPXDDC4_PDC4_OEA4_RG & 0xff)<<8) | (TPXDDC4_PDC4_OEB4_RG & 0xff))
#define TPXDDC4_pdc4_fifo_mstat_data()	(((TPXDDC4_PDC4_OEA6_RG & 0xfc)<<8) | (TPXDDC4_PDC4_OEB6_RG & 0xff))
#define TPXDDC4_pdc4_fifo_next_frame()	(TPXDDC4_PDC4_OEB7_RG)


//****************************************************************



//**************** Macros for DPRAM/DPSEM access *****************

#define TPXDDC4_set_dpram(a,v)		(((unsigned *)TPXDDC4_DPRAM_SADDR)[TPXDDC4_DPRAM_LEN])=v
#define TPXDDC4_dpram(a)		(((unsigned *)TPXDDC4_DPRAM_SADDR)[TPXDDC4_DPRAM_LEN])

#define TPXDDC4_set_dpram_mh_rq(v)	(*(unsigned *) TPXDDC4_DPRAM_MH_RQ_ADDR)=v
#define TPXDDC4_dpram_mh_rq()		(*(unsigned *) TPXDDC4_DPRAM_MH_RQ_ADDR)

#define TPXDDC4_set_dpram_hm_rq(v)	(*(unsigned *) TPXDDC4_DPRAM_HM_RQ_ADDR)=v
#define TPXDDC4_dpram_hm_rq()		(*(unsigned *) TPXDDC4_DPRAM_HM_RQ_ADDR)

#define TPXDDC4_set_dpsem0(v)		TPXDDC4_DPSEM0_RG=v
#define TPXDDC4_dpsem0()		(TPXDDC4_DPSEM0_RG & TPXDDC4_DPSEM_BITMASK)
#define TPXDDC4_set_dpsem1(v)		TPXDDC4_DPSEM1_RG=v
#define TPXDDC4_dpsem1()		(TPXDDC4_DPSEM1_RG & TPXDDC4_DPSEM_BITMASK)
#define TPXDDC4_set_dpsem2(v)		TPXDDC4_DPSEM2_RG=v
#define TPXDDC4_dpsem2()		(TPXDDC4_DPSEM2_RG & TPXDDC4_DPSEM_BITMASK)
#define TPXDDC4_set_dpsem3(v)		TPXDDC4_DPSEM3_RG=v
#define TPXDDC4_dpsem3()		(TPXDDC4_DPSEM3_RG & TPXDDC4_DPSEM_BITMASK)
#define TPXDDC4_set_dpsem4(v)		TPXDDC4_DPSEM4_RG=v
#define TPXDDC4_dpsem4()		(TPXDDC4_DPSEM4_RG & TPXDDC4_DPSEM_BITMASK)
#define TPXDDC4_set_dpsem5(v)		TPXDDC4_DPSEM5_RG=v
#define TPXDDC4_dpsem5()		(TPXDDC4_DPSEM5_RG & TPXDDC4_DPSEM_BITMASK)
#define TPXDDC4_set_dpsem6(v)		TPXDDC4_DPSEM6_RG=v
#define TPXDDC4_dpsem6()		(TPXDDC4_DPSEM6_RG & TPXDDC4_DPSEM_BITMASK)
#define TPXDDC4_set_dpsem7(v)		TPXDDC4_DPSEM7_RG=v
#define TPXDDC4_dpsem7()		(TPXDDC4_DPSEM7_RG & TPXDDC4_DPSEM_BITMASK)

//*********************************************************************

//************* XDAC/XSL/PFG control via McBSP-1 ***********************
// notes:
//	1. McBSP-1 transmitter is used for programming PFG, XDAC and XSL.
//      2. McBSP-1 receiver is not used.
//	3. McBSP-1 transmitter must use 32-bit data format.

//---- peripherals write commands
#define	TPXDDC4_PFG1_WR_CMND		0x00000000L
#define	TPXDDC4_PFG2_WR_CMND		0x20000000L
#define	TPXDDC4_XDAC1_WR_CMND		0x40008000L
#define	TPXDDC4_XDAC2_WR_CMND		0x40000000L
#define	TPXDDC4_XDAC_SCALE_2V_WR_CMND	0x40009001L
#define	TPXDDC4_XDAC_SCALE_4V_WR_CMND	0x40009002L
#define	TPXDDC4_XSL1_WR_CMND		0x80000000L
#define	TPXDDC4_XSL2_WR_CMND		0xc0000000L

//---- DAC defs
#define TPXDDC4_XDAC_DATA_BITMASK	0x00000fffL

	// used by TPXDDC4_xdac_cmnd macros to select particular XDAC chanel
#define	TPXDDC4_XDAC1_SEL		1	
#define	TPXDDC4_XDAC2_SEL		2

	// COMMAND word generator for XDAC-1/2.
#define	TPXDDC4_xdac_cmnd(ch,d)		((unsigned long)(  ((ch==1) ? TPXDDC4_XDAC1_WR_CMND:TPXDDC4_XDAC2_WR_CMND)\
			         	 | ((unsigned long)d & TPXDDC4_XDAC_DATA_BITMASK)))


//---- XSL defs
#define TPXDDC4_XSL_DATA8_BITMASK	0x000000ffL
#define TPXDDC4_XSL_DATA16_BITMASK	0x0000ffffL
#define TPXDDC4_XSL_DATA24_BITMASK	0x00ffffffL
#define TPXDDC4_XSL_DATA30_BITMASK	0x3fffffffL

	// used by TPXDDC4_xsl_cmnd macros to select particular XSL chanel
#define	TPXDDC4_XSL1_SEL		1	
#define	TPXDDC4_XSL2_SEL		2

	// COMMAND word generator for XSL-1/2.
#define	TPXDDC4_xsl_cmnd(ch,d)		((unsigned long)  (((ch==1) ? TPXDDC4_XSL1_WR_CMND:TPXDDC4_XSL2_WR_CMND)\
			         	 | ((unsigned long)d & TPXDDC4_XSL_DATA30_BITMASK) ))


//---- PFG defs
//note: all PFG divider codes are bit-reversed !!!
#define TPXDDC4_PFG_REF_CLK_FREQ_10MHZ		10.0e+6		// regular 10.0MHz 50ppm PFG reference clock frequency
#define TPXDDC4_PFG_REF_CLK_FREQ_HS19MHZ	19.68e+6	// high stability 19.68MHz 2ppm PFG reference clock frequency

#define TPXDDC4_PFG_REF_CLK_FREQ			((TPXDDC4_PFG_REF_CLK_FREQ_IS_HS19MHZ) ? TPXDDC4_PFG_REF_CLK_FREQ_HS19MHZ:TPXDDC4_PFG_REF_CLK_FREQ_10MHZ)


#define TPXDDC4_PFG_DATA_BITMASK	0x00ffffffL

#define TPXDDC4_PFG_N_BITMASK		0x00fe0000L
#define TPXDDC4_PFG_M_BITMASK		0x0001fc00L
#define TPXDDC4_PFG_V_BITMASK		0x00000200L
#define TPXDDC4_PFG_X_BITMASK		0x00000180L
#define TPXDDC4_PFG_R_BITMASK		0x00000060L
#define TPXDDC4_PFG_CNF_DATA		0x0000000dL	//- to be ORed for proper operation

#define TPXDDC4_PFG_N_SHIFT		17
#define TPXDDC4_PFG_M_SHIFT		10
#define TPXDDC4_PFG_V_SHIFT		9
#define TPXDDC4_PFG_X_SHIFT		7
#define TPXDDC4_PFG_R_SHIFT		5

	// used by TPXDDC4_pfg_cmnd macros to select particular PFG channel
#define	TPXDDC4_PFG1_SEL		1
#define	TPXDDC4_PFG2_SEL		2

	// COMMAND word generator for PFG-1/2.
	// !!!: n/m/v/x/r must be in the bit-reversed code
#define	TPXDDC4_pfg_cmnd(ch,n,m,v,x,r)	((unsigned long)( ((ch==1)? TPXDDC4_PFG1_WR_CMND:TPXDDC4_PFG2_WR_CMND)\
					  | ((((unsigned long)n)<<TPXDDC4_PFG_N_SHIFT) & TPXDDC4_PFG_N_BITMASK) \
				       	  | ((((unsigned long)m)<<TPXDDC4_PFG_M_SHIFT) & TPXDDC4_PFG_M_BITMASK) \
	       				  | ((((unsigned long)v)<<TPXDDC4_PFG_V_SHIFT) & TPXDDC4_PFG_V_BITMASK) \
			  	  	  | ((((unsigned long)x)<<TPXDDC4_PFG_X_SHIFT) & TPXDDC4_PFG_X_BITMASK) \
					  | ((((unsigned long)r)<<TPXDDC4_PFG_R_SHIFT) & TPXDDC4_PFG_R_BITMASK) \
					  | TPXDDC4_PFG_CNF_DATA ))

//---- McBSP-1 configuration for C54xx
// recommended configuration for C5410/C5416 DSP:
//       XLen=32bits
//       XCLK - Int
//       FSX - output on DX load event
//       CLKXP - active low (1)
//       FSXP - active low (1), advance=+1clkx
//	 	 CLKX = 20MHz (C5410: SRGR1=4, C5416: SRGR1=7)
//       receiver is disabled (!)

// SPCR: 0000 0000 0000 0000 0000 0000 0000 0000
// PCR:  ---- ---- ---- ---- 0000 1010 0000 1010
// XCR:  0000 0000 0000 0001 0000 0000 1010 0000
// SRGR: 0010 0000 0000 0000 0000 0000 0000 0100

#define	TPXDDC4_MCBSP1_SPCR1_DATA	0x0000
#define	TPXDDC4_MCBSP1_SPCR2_DATA	0x0000
#define	TPXDDC4_MCBSP1_PCR_DATA		0x0a0a
#define	TPXDDC4_MCBSP1_XCR1_DATA	0x00a0
#define	TPXDDC4_MCBSP1_XCR2_DATA	0x0001
#define	TPXDDC4_MCBSP1_SRGR1_DATA_C5410	0x0004
#define	TPXDDC4_MCBSP1_SRGR1_DATA_C5416	0x0007
#define	TPXDDC4_MCBSP1_SRGR2_DATA	0x2000

#define	TPXDDC4_MCBSP1_SPCR1_GO_MASK	0x0000
#define	TPXDDC4_MCBSP1_SPCR2_GO_MASK	0x0041

//*********************************************************************

//************* PH-DAC control via McBSP-2 ***********************
// notes:
//	1. McBSP-2 transmitter is used for programming 16-bit PH-DAC (BB PCM56).
//      2. McBSP-2 receiver is not used.
//	3. McBSP-2 transmitter must use 16-bit data format.

#define TPXDDC4_PHDAC_DATA_BITMASK		0xffff

//---- McBSP-2 configuration for C54xx
// recommended configuration for C5410/C5416 DSP:
//       XLen=16bits
//       XCLK - Int
//       FSX - output on DX load event
//       CLKXP - active low (1)
//       FSXP - active low (1), advance=+1clkx
//	 	 CLKX = 10MHz (C5410: SRGR1=9   C5416: SRGR1=15)
//       receiver is disabled (!)

// SPCR: 0000 0000 0000 0000 0000 0000 0000 0000
// PCR:  ---- ---- ---- ---- 0000 1010 0000 1010
// XCR:  0000 0000 0000 0001 0000 0000 0100 0000
// SRGR: 0010 0000 0000 0000 0000 0000 0000 1001

#define	TPXDDC4_MCBSP2_SPCR1_DATA	0x0000
#define	TPXDDC4_MCBSP2_SPCR2_DATA	0x0000
#define	TPXDDC4_MCBSP2_PCR_DATA		0x0a0a
#define	TPXDDC4_MCBSP2_XCR1_DATA	0x0040
#define	TPXDDC4_MCBSP2_XCR2_DATA	0x0001
#define	TPXDDC4_MCBSP2_SRGR1_DATA_C5410	0x0009
#define	TPXDDC4_MCBSP2_SRGR1_DATA_C5416	0x000F
#define	TPXDDC4_MCBSP2_SRGR2_DATA	0x2000

#define	TPXDDC4_MCBSP2_SPCR1_GO_MASK	0x0000
#define	TPXDDC4_MCBSP2_SPCR2_GO_MASK	0x0041

//*********************************************************************


//********* Extension functions (DDC4_DX.H file) ******************

	// FLASH control
int TPXDDC4_flash_set_page(unsigned long,unsigned **);
int TPXDDC4_flash_erase_sector(unsigned long,unsigned);
int TPXDDC4_flash_erase_chip(unsigned);
int TPXDDC4_flash_write(unsigned long,unsigned);
int TPXDDC4_flash_read(unsigned long);

	// XDAC/PFG/XSL control
void TPXDDC4_write_pfg(unsigned,unsigned,unsigned,unsigned,unsigned,unsigned);
void TPXDDC4_write_xsl(unsigned,unsigned long);
void TPXDDC4_write_xdac(unsigned,int);
void TPXDDC4_set_xdac_scale_2v();
void TPXDDC4_set_xdac_scale_4v();

	// PH-DAC control
void TPXDDC4_write_phdac(int);

	// init DSP and McBSP-1/2 ports
void TPXDDC4_init_cpu();
void TPXDDC4_init_mcbsp1();
void TPXDDC4_init_mcbsp2();


	// FIFO programming
int TPXDDC4_set_fifo_flag_offset(unsigned long, unsigned long);


//*********************************************************************

#endif	// end of DDC4 defs

