/********************************************************************
  TORNADO-PX/DDC4 Quad DRR PIOX-16 Coprocessor/Controller. Rev.1B
  On-board C54xx DSP Environment. Optional Utility Functions.

  (C) MicroLAB Systems, 2000-2003
  http://www.mlabsys.com


  Description:
  ------------
	This file contains functions for FLASH control,
	PFG bit reversing, serial I/O, FIFO programming, etc.
    for on-board C54xx DSP.

  Copyright:
  ----------
	This utility is distributed free of charge and as it is without
	any obligation from MicroLAB Systems.


***********************************************************************/

#ifndef __DDC4_DX_DEF__
#define __DDC4_DX_DEF__		1

#include "stddef.h"
#include "c54x_def.h"


//*********************** FLASH control *******************************

int TPXDDC4_flash_set_page(unsigned long a, unsigned **fptr)
	// set the XDMP rg in accordance with FLASH address A,
	// calculates and returns PTR to access the addressed FLASH location
	// returns:  0 - no error
	//          -1 - error
{
	unsigned fpage;

	// validate FLASH address
	if(a>TPXDDC4_FLASH_EADDR) return(-1);

	// calculate and set the XDMP rg to address the FLASH page
	fpage = (unsigned) ((unsigned long) ((a%TPXDDC4_FLASH_LEN)/TPXDDC4_FLASH_PAGE_LEN));
	TPXDDC4_set_xdmp(TPXDDC4_XDM_SEL_FLASH | fpage);

	if(fptr!=NULL)
		*fptr = (unsigned *)(TPXDDC4_FLASH_PAGE_SADDR + (unsigned)((unsigned long)(a%TPXDDC4_FLASH_PAGE_LEN)) );

	return(0);
}


int TPXDDC4_flash_erase_sector(unsigned long a,unsigned vf)
	// Erase sector, which corresponds to FLASH address A.
	// Verify erased sector data if VF==1.
	// returns:  0 - no error
	//          -1 - error occured
{ unsigned *fpage_ptr, *fptr, sa,r;
  unsigned long i;

	        //set FLASH page and FPTR
	if(TPXDDC4_flash_set_page(a,&fptr)!=0)	return(-1);

        	// set pointers and sector address
	fpage_ptr = (unsigned *)TPXDDC4_FLASH_PAGE_SADDR;
	sa = (unsigned)((unsigned long)(a/TPXDDC4_FLASH_SECTOR_LEN));

		// execute SECTOR ERASE command
    fpage_ptr[0x555]=0xaa;
    fpage_ptr[0x2aa]=0x55;
    fpage_ptr[0x555]=0x80;
    fpage_ptr[0x555]=0xaa;
    fpage_ptr[0x2aa]=0x55;
	fpage_ptr[sa]   =0x30;

		// wait until erase is finished
	while(((r=fpage_ptr[sa]) & 0x88)!=0x88)
	{
		// check for time-out failure
		if((r&0x20)!=0)
			return(-1);
	}

	// now check locations for read-back data == 0xFF
    if((*fptr & TPXDDC4_FLASH_DATA_BITMASK)!=0xff) return(-1);

	// perform erased sector data verification if requested
    if(vf!=0)
	{
		// set FPTR to the sector boundary
		if(TPXDDC4_flash_set_page(a-(a%TPXDDC4_FLASH_SECTOR_LEN),&fptr)!=0)
			return(-1);

	   	for(i=0L;i<TPXDDC4_FLASH_SECTOR_LEN;i++,fptr++)
		{
			if((*fptr &TPXDDC4_FLASH_DATA_BITMASK)!=0xff)
		    	return(-1);
		}
	}

	return(0);	// exit OK
}



int TPXDDC4_flash_erase_chip(unsigned vf)
	// Erase all sectors of FLASH. Verify erased data if VF==1.
	// returns:  0 - no error
	//          -1 - error occured
{ unsigned *fpage_ptr,*fptr,r,page;
  unsigned long i;

	TPXDDC4_set_xdmp(TPXDDC4_XDM_SEL_FLASH | 0); //select FLASH page #0
	fpage_ptr = (unsigned *)TPXDDC4_FLASH_PAGE_SADDR;

	// execute ERASE CHIP command
    fpage_ptr[0x555]=0xaa;
    fpage_ptr[0x2aa]=0x55;
    fpage_ptr[0x555]=0x80;
    fpage_ptr[0x555]=0xaa;
    fpage_ptr[0x2aa]=0x55;
	fpage_ptr[0x555]=0x10;

	// wait until erase is finished
	while(((r=fpage_ptr[0x555]) & 0x80)!=0x80)
	{
		// check for time-out failure
		if((r&0x20)!=0)
			return(-1);
	}

	// now check for one-cell read-back data == 0xFF
    if((fpage_ptr[0x555] & TPXDDC4_FLASH_DATA_BITMASK)!=0xff)
		return(-1);

	// verify complete erased data if this has been requested
	if(vf!=0)
    {
		for (page=0;page<TPXDDC4_FLASH_PAGES;page++)
	    {
	                //select FLASH page
     		TPXDDC4_set_xdmp(TPXDDC4_XDM_SEL_FLASH | page);
			for(i=0L,fptr=fpage_ptr;i<TPXDDC4_FLASH_PAGE_LEN;i++,fptr++)
			{
				if((*fptr &TPXDDC4_FLASH_DATA_BITMASK)!=0xff)
		      		return(-1);
			}
	    }
	}

	return(0);	// exit OK
}

int TPXDDC4_flash_write(unsigned long a, unsigned d)
	// writes D (bits 0..7 are valid) to FLASH addr #A
	// returns:  0 - no error
	//          -1 - error occured
{ unsigned *fpage_ptr, *fptr, r,w;

	        //set FLASH page and FPTR
	if(TPXDDC4_flash_set_page(a,&fptr)!=0)	return(-1);
	fpage_ptr = (unsigned *)TPXDDC4_FLASH_PAGE_SADDR;

	//create masked write data
    w = d & TPXDDC4_FLASH_DATA_BITMASK;

	// execute PROGRAM BYTE command
    fpage_ptr[0x555]=0xaa;
    fpage_ptr[0x2aa]=0x55;
    fpage_ptr[0x555]=0xa0;
    *fptr=w ;

	// verify for program ends and errors
    while(1)
	{
		r = (*fptr) & TPXDDC4_FLASH_DATA_BITMASK;

		// verify read data
	  	if(r==w) return(0);		// exit OK

		// if not OK, then test bit D5 as timeout condition
	  	if((r&0x20)!=0)
	    {   	// if timeout, then read again
	    	r = (*fptr) & TPXDDC4_FLASH_DATA_BITMASK;

			if(r==w)
				return(0);	// exit OK

			return(-1);		// otherwise set error and exit
	    }
	}
}

int TPXDDC4_flash_read(unsigned long a)
	// reads/returns data from FLASH addr #A
	// returns:  0 - no error
	//          -1 - error occured
	//	    >0 - FLASH read data
{ unsigned *fptr;

	//set FLASH page and FPTR
	if(TPXDDC4_flash_set_page(a,&fptr)!=0)	return(-1);
	return((*fptr) & TPXDDC4_FLASH_DATA_BITMASK);
}

//********************************************************************



//******************** XDAC/PFG/XSL control **************************

#define	_TPXDDC4_WAIT_MCBSP1_TRDY	while(!((*C54X_MCBSP1_SPCR2) & 0x2))
#define	_TPXDDC4_WRITE_MCBSP1_DATA_LSW	(*C54X_MCBSP1_DXR1)
#define	_TPXDDC4_WRITE_MCBSP1_DATA_MSW	(*C54X_MCBSP1_DXR2)
#define	_TPXDDC4_WRITE_MCBSP1_DATA(x)	{(*C54X_MCBSP1_DXR2)=(unsigned)(((unsigned long)(x>>16))&0xffffL);(*C54X_MCBSP1_DXR1)=(unsigned)((unsigned long)x&0xffffL);}



static unsigned _TPXDDC4_reverse_bits(unsigned x, unsigned nb)
	// This procedure is used for PFG data processing and
	// reverses bits inside the 16-bit X-word for correct load to the PFG data word.
	// number of bits to be reverses is set by NB var.
{ unsigned xr,mask,i;

  for(i=nb,mask=0x1,xr=0;i>0;i--)
   { if((x&mask)!=0)  xr |=0x1;
     if(i>1) xr <<=1;
     mask <<=1;
   }
   return(xr);
}


	//--- writes DATA to PFG-1/2
void TPXDDC4_write_pfg(unsigned sel,unsigned n,unsigned m,unsigned v,unsigned x,unsigned r)
{ unsigned long xd;
		//-- wait for McBSP transmitter is ready
	_TPXDDC4_WAIT_MCBSP1_TRDY;
		//-- assemble PFG data word with reversed bits for N/M/X/R
	xd=TPXDDC4_pfg_cmnd(sel,
			   _TPXDDC4_reverse_bits(n,7),
			   _TPXDDC4_reverse_bits(m,7),
                           v,
			   _TPXDDC4_reverse_bits(x,2),
			   _TPXDDC4_reverse_bits(r,2)
                           );
	_TPXDDC4_WRITE_MCBSP1_DATA(xd);
}


	//--- writes DATA to XDAC-1/2
void TPXDDC4_write_xdac(unsigned sel,int data)
{
		//-- wait for McBSP transmitter is ready
	_TPXDDC4_WAIT_MCBSP1_TRDY;
	_TPXDDC4_WRITE_MCBSP1_DATA(TPXDDC4_xdac_cmnd(sel,data));
}


	//--- set DAC scale
void TPXDDC4_set_xdac_scale_2v()
{
		//-- wait for McBSP transmitter is ready
	_TPXDDC4_WAIT_MCBSP1_TRDY;
	_TPXDDC4_WRITE_MCBSP1_DATA(TPXDDC4_XDAC_SCALE_2V_WR_CMND);
}

void TPXDDC4_set_xdac_scale_4v()
{
		//-- wait for McBSP transmitter is ready
	_TPXDDC4_WAIT_MCBSP1_TRDY;
	_TPXDDC4_WRITE_MCBSP1_DATA(TPXDDC4_XDAC_SCALE_4V_WR_CMND);
}

	//--- writes DATA to XSL-1/2
void TPXDDC4_write_xsl(unsigned sel, unsigned long data)
{
		//-- wait for McBSP transmitter is ready
	_TPXDDC4_WAIT_MCBSP1_TRDY;
	_TPXDDC4_WRITE_MCBSP1_DATA(TPXDDC4_xsl_cmnd(sel,data));
}

//******************************************************************

//******************** PHDAC control *******************************

#define	_TPXDDC4_WAIT_MCBSP2_TRDY	while(!((*C54X_MCBSP2_SPCR2) & 0x2))
#define	_TPXDDC4_WRITE_MCBSP2_DATA	(*C54X_MCBSP2_DXR1)

void TPXDDC4_write_phdac(int data)
{
		//-- wait for McBSP transmitter is ready
	_TPXDDC4_WAIT_MCBSP2_TRDY;
	_TPXDDC4_WRITE_MCBSP2_DATA = (data & TPXDDC4_PHDAC_DATA_BITMASK);
}

//******************************************************************


//**************** init bus I/F of C5410 DSP ***********************

void TPXDDC4_init_cpu()
 {
  *C54X_IMR=0;			// clear IMR since it is not cleared on reset
  *C54X_IFR=0xffff;		// clear all interrupt requests

  *C54X_HPIC=0x2;		// setup shared mode for HPI area
	
  if(TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410)
	{
	  *C54X_SWWSR=TPXDDC4_C5410_SWWSR_DATA;		
	  *C54X_SWCR=TPXDDC4_C5410_SWCR_DATA;		
	  *C54X_BSCR=TPXDDC4_C5410_BSCR_DATA;		
	  *C54X_CLKMD=TPXDDC4_CLKMD_DATA0;		
	  *C54X_CLKMD=TPXDDC4_C5410_CLKMD_DATA;		
	}
	else
	{
	  *C54X_SWWSR=TPXDDC4_C5416_SWWSR_DATA;		
	  *C54X_SWCR=TPXDDC4_C5416_SWCR_DATA;		
	  *C54X_BSCR=TPXDDC4_C5416_BSCR_DATA;		
	  *C54X_CLKMD=TPXDDC4_CLKMD_DATA0;		
	  *C54X_CLKMD=TPXDDC4_C5416_CLKMD_DATA;		
	}

 }

	// configure McBSP-1 port of C5410/C5416 DSP for XDAC/PFG/XSL
void TPXDDC4_init_mcbsp1()
{
	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP1_SPCR1 = TPXDDC4_MCBSP1_SPCR1_DATA;
	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP1_SPCR2 = TPXDDC4_MCBSP1_SPCR2_DATA;

	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP1_PCR   = TPXDDC4_MCBSP1_PCR_DATA;

	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP1_XCR1  = TPXDDC4_MCBSP1_XCR1_DATA;
	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP1_XCR2  = TPXDDC4_MCBSP1_XCR2_DATA;

	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_SRGR1;
	   *C54X_MCBSP1_SRGR1 = (TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410) ? TPXDDC4_MCBSP1_SRGR1_DATA_C5410:TPXDDC4_MCBSP1_SRGR1_DATA_C5416;
	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_SRGR2;
	   *C54X_MCBSP1_SRGR2 = TPXDDC4_MCBSP1_SRGR2_DATA;

		//-- now enable McBSP port operation
	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP1_SPCR1 |= TPXDDC4_MCBSP1_SPCR1_GO_MASK;
	   *C54X_MCBSP1_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP1_SPCR2 |= TPXDDC4_MCBSP1_SPCR2_GO_MASK;
		//!!! leave _SPSA pointint to the _SPCR2
		//    for transmitter ready run-time polling
}


	// configure McBSP-1 port of C5410/C5416 DSP for PH-DAC
void TPXDDC4_init_mcbsp2()
{
	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP2_SPCR1 = TPXDDC4_MCBSP2_SPCR1_DATA;
	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP2_SPCR2 = TPXDDC4_MCBSP2_SPCR2_DATA;

	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_PCR;
	   *C54X_MCBSP2_PCR   = TPXDDC4_MCBSP2_PCR_DATA;

	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_XCR1;
	   *C54X_MCBSP2_XCR1  = TPXDDC4_MCBSP2_XCR1_DATA;
	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_XCR2;
	   *C54X_MCBSP2_XCR2  = TPXDDC4_MCBSP2_XCR2_DATA;

	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_SRGR1;
	   *C54X_MCBSP2_SRGR1 = (TPXDDC4_dsp_type()==TPXDDC4_DSP_TYPE_C5410) ? TPXDDC4_MCBSP2_SRGR1_DATA_C5410:TPXDDC4_MCBSP2_SRGR1_DATA_C5416;
	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_SRGR2;
	   *C54X_MCBSP2_SRGR2 = TPXDDC4_MCBSP2_SRGR2_DATA;


		//-- now enable McBSP port operation
	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_SPCR1;
	   *C54X_MCBSP2_SPCR1 |= TPXDDC4_MCBSP2_SPCR1_GO_MASK;
	   *C54X_MCBSP2_SPSA  = C54X_MCBSP_SBA_SPCR2;
	   *C54X_MCBSP2_SPCR2 |= TPXDDC4_MCBSP2_SPCR2_GO_MASK;
		//!!! leave _SPSA pointint to the _SPCR2
		//    for transmitter ready run-time polling
}

//******************************************************************


//******************** FIFO programming ****************************

	//--- set FIFO PAF/PAE offset values
	// Both PAE/PAF params values are assumed to be the values
	// origined to the FIFO address #0.

int TPXDDC4_set_fifo_flag_offset(unsigned long pae, unsigned long paf)
{ int i;
  unsigned long d;
        	//-- validate input params
	if((paf+2)>=TPXDDC4_FIFO_LEN  || pae>=TPXDDC4_FIFO_LEN) return(-1);

		// set FIFO CNF mode and perform FIFO reset
        TPXDDC4_SET_FIFO_MODE_CNF;	
	TPXDDC4_SET_FIFO_RESET;

		// load 18-bit PAE data starting fm LSB
		// Note!!!
		//   Offset value for the PAE flag, which has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO address #0, so input PAE function parameter
		//   is OK to be transmitted directly to FIFO.
		//   Output FIFO PAE flag will be set active during
		//   writing of PAE samples to FIFO and will be set inactive
		//   after writing the (PAE+1)-th FIFO sample (in case
		//   not FIFO reads is perfromed). Output FIFO PAE flag
		//   is updated on FIFO RCLK.

       	d = pae;
	for(i=0;i<18; i++)
         {
           TPXDDC4_FIFO_DATA_RG = (unsigned)d ;
	   d >>= 1;
	 }


		// load 18-bit PAF data starting fm LSB
		// Note!!!
		//   Offset value for the PAF flag, which has to be
		//   loaded to the FIFO chip, must be origined to the
		//   FIFO LEN, and not to the FIFO address #0. So,
		//   input PAF function parameter has to be converted
		//   to (FIFO_LEN-PAF) prior transmision to FIFO.
		//   Due to continuous WCLK, the output FIFO PAF flag
		//   will be set active after 2 WCLK cycles after writing
		//   (FIFO_LEN-PAF) FIFO samples, so the programmed value
		//   must be (PAF-2).

	d = TPXDDC4_FIFO_LEN-paf+2;
	for(i=0;i<18; i++)
         {
           TPXDDC4_FIFO_DATA_RG = (unsigned)d ;
	   d >>= 1;
	 }

		// set FIFO DAQ mode and exit
        TPXDDC4_SET_FIFO_MODE_DAQ;	
	return(0);
}

//******************************************************************

#endif	// end of DDC4 defs
