/*****************************************************************
  TORNADO-PX/DDC4 Quad DRR PIOX-16 Coprocessor/Controller. Rev.1A
  Host DSP defs for host TORNADO-3x/P3x/E3x DSP systems/controllers.

  (C) MicroLAB Systems Ltd, 2000
  http://www.mlabsys.com


  Notes:
  ------
    1. This file must be used as .H header file for C3x DSP
       environment of host TORNADO-3x/P3x/E3x DSP system/controller.
    2. All commands are adjusted for 32-bit data words and
       occupy least significant 16-bit word.

******************************************************************/

#ifndef __DDC4_H3X__
#define __DDC4_H3X__		1


//!!!!! host PIOX-16 base address (identical for all T3x/TP33/E3x boards)
// note:
//   Base address for T-PXDDC4 DCM is base address for PIOX-16 I/F
//   of particular host TORNADO DSP system/controller (refer to the .H
//   file and documentation for your TORNADO DSP system/controller).

#define	_PIOX16_SADDR_			0xff0000



//================ general defs ==============================

#define	HTPXDDC4_OFF			0
#define	HTPXDDC4_ON			1

//============================================================


//============= host DSP I/O ports definitions ===============

#define HTPXDDC4_DPRAM_SADDR		(_PIOX16_SADDR_+0x8000)	// DPRAM area
#define HTPXDDC4_DPRAM_EADDR		(_PIOX16_SADDR_+0xffff)
#define HTPXDDC4_DPRAM_LEN		0x8000
#define	HTPXDDC4_DPRAM_ADDR_BITMASK	0x7fff
#define	HTPXDDC4_DPRAM_HM_RQ_ADDR	(_PIOX16_SADDR_+0xffff)	// DPRAM: HostPIOX-to-DSP irq
#define	HTPXDDC4_DPRAM_MH_RQ_ADDR	(_PIOX16_SADDR_+0xfffe)	// DPRAM: DSP-to-HostPIOX irq

#define HTPXDDC4_CNTR1_RG		(_PIOX16_SADDR_+0x0000)	// r/w: CONTROL #1 register
#define HTPXDDC4_CNTR2_RG		(_PIOX16_SADDR_+0x0001)	// r/w: CONTROL #2 register

#define HTPXDDC4_IE_RG			(_PIOX16_SADDR_+0x0010)	// r/w: Host Int Enable register
#define HTPXDDC4_IS_RG			(_PIOX16_SADDR_+0x0011)	// r:   Host Int Request Status register
#define HTPXDDC4_HIRQ_SEL_RG		(_PIOX16_SADDR_+0x0012)	// r/w: Host Int Line Selector register

#define HTPXDDC4_CLR_HPI_ERR_RG		(_PIOX16_SADDR_+0x0020)	// w:   Clear HPI timeout error
#define HTPXDDC4_CLR_DPRAM_ERR_RG	(_PIOX16_SADDR_+0x0021)	// w:   Clear DPRAM timeout error

#define HTPXDDC4_HPIC_RG_LSB		(_PIOX16_SADDR_+0x0060)	// r/w: C54x HPIC register
#define HTPXDDC4_HPIC_RG_MSB		(_PIOX16_SADDR_+0x0061)	// r/w: C54x HPIC register
#define HTPXDDC4_HPID_AINC_RG_LSB	(_PIOX16_SADDR_+0x0062)	// r/w: C54x HPID/AINC register
#define HTPXDDC4_HPID_AINC_RG_MSB	(_PIOX16_SADDR_+0x0063)	// r/w: C54x HPID/AINC register
#define HTPXDDC4_HPIA_RG_LSB		(_PIOX16_SADDR_+0x0064)	// r/w: C54x HPIA register
#define HTPXDDC4_HPIA_RG_MSB		(_PIOX16_SADDR_+0x0065)	// r/w: C54x HPIA register
#define HTPXDDC4_HPID_RG_LSB		(_PIOX16_SADDR_+0x0066)	// r/w: C54x HPID register
#define HTPXDDC4_HPID_RG_MSB		(_PIOX16_SADDR_+0x0067)	// r/w: C54x HPID register

#define	HTPXDDC4_DPSEM_SADDR		(_PIOX16_SADDR_+0x0070)	// r/w (D0): DPSEM area
#define	HTPXDDC4_DPSEM_EADDR		(_PIOX16_SADDR_+0x0077)
#define	HTPXDDC4_DPSEM_LEN		0x8
#define	HTPXDDC4_DPSEM_ADDR_BITMASK	0x7

//=============================================================


//============= host data definitions =========================

#define HTPXDDC4_RG_DATA_BITMASK	0xf	// D0..D3 only are valid for all registers	

	//-- Control #1 Register bits
#define HTPXDDC4_MRESET_RESET 		0x0	
#define HTPXDDC4_MRESET_GO		0x1
#define HTPXDDC4_MRESET_BITMASK		0x1


	//-- Control #2 Register bits
#define HTPXDDC4_BMODE_NONE 		0x0	
#define HTPXDDC4_BMODE_FLASH		0x1
#define HTPXDDC4_BMODE_HPI		0x2
#define HTPXDDC4_BMODE_NONE1 		0x3
#define HTPXDDC4_BMODE_BITMASK		0x3

#define HTPXDDC4_DPRAM_ERR_EN_ON	0x4
#define HTPXDDC4_DPRAM_ERR_EN_OFF	0x0
#define HTPXDDC4_DPRAM_ERR_EN_BITMASK	0x4

#define HTPXDDC4_HPI_ERR_EN_ON		0x8
#define HTPXDDC4_HPI_ERR_EN_OFF		0x0
#define HTPXDDC4_HPI_ERR_EN_BITMASK	0x8


	//-- IE Register bits
#define HTPXDDC4_DPRAM_IE_ON		0x1
#define HTPXDDC4_DPRAM_IE_OFF		0x0
#define HTPXDDC4_DPRAM_IE_BITMASK	0x1

#define HTPXDDC4_HINT_IE_ON		0x2
#define HTPXDDC4_HINT_IE_OFF		0x0
#define HTPXDDC4_HINT_IE_BITMASK	0x2

#define HTPXDDC4_DPRAM_ERR_IE_ON	0x4
#define HTPXDDC4_DPRAM_ERR_IE_OFF	0x0
#define HTPXDDC4_DPRAM_ERR_IE_BITMASK	0x4

#define HTPXDDC4_HPI_ERR_IE_ON		0x8
#define HTPXDDC4_HPI_ERR_IE_OFF		0x0
#define HTPXDDC4_HPI_ERR_IE_BITMASK	0x8


	//-- IS Register bits
#define HTPXDDC4_DPRAM_IRQ_BITMASK	0x1
#define HTPXDDC4_HINT_BITMASK		0x2
#define HTPXDDC4_DPRAM_ERR_IRQ_BITMASK	0x4
#define HTPXDDC4_HPI_ERR_IRQ_BITMASK	0x8


	//-- PIOX-IRQ Line Selector Register bits
#define HTPXDDC4_HIRQ_EN_ON		0x8	
#define HTPXDDC4_HIRQ_EN_OFF		0x0
#define HTPXDDC4_HIRQ_EN_BITMASK	0x8

#define HTPXDDC4_HIRQ0_SEL		0x0
#define HTPXDDC4_HIRQ1_SEL		0x1
#define HTPXDDC4_HIRQ2_SEL		0x2
#define HTPXDDC4_HIRQ_SEL_BITMASK	0x3


	//-- DPSEM defs
#define	HTPXDDC4_DPSEM_BITMASK		0x1	
#define	HTPXDDC4_DPSEM_OFF		0x1
#define	HTPXDDC4_DPSEM_ON		0x0


	//-- HPI defs
#define HTPXDDC4_HPI_DATA_BITMASK  	0xff	//8-bit bitmask for HPIC/HPIA/HPID

#define	HTPXDDC4_HPIC_BOB_BITMASK	0x1
#define	HTPXDDC4_HPIC_SMODE_BITMASK	0x2
#define	HTPXDDC4_HPIC_HINT_BITMASK	0x8
#define	HTPXDDC4_HPIC_XHPIA_BITMASK	0x10
#define HTPXDDC4_HPIC_READ_BITMASK  	0x1b

#define	HTPXDDC4_HPIC_DSPINT		0x4
#define HTPXDDC4_HPIC_WRITE_BITMASK  	0x1d

#define	HTPXDDC4_HPIA_MIN		0x0060	// valid HPI address for C5410 DSP
#define	HTPXDDC4_HPIA_MAX		0x7fff
#define	HTPXDDC4_HPI_RAM_LEN		0x7fa0
#define	HTPXDDC4_HPIA_XRAM_MIN		0x18000	// X-HPI area for 64K on-chip SRAM with OVLY=1 and DROM=1
#define	HTPXDDC4_HPIA_XRAM_MAX		0x1ffff
#define	HTPXDDC4_HPI_XRAM_LEN		0x8000


	//-- DPRAM defs
#define	HTPXDDC4_DPRAM_DATA_BITMASK	0xffff

//===========================================================




//========== macros for accessing HIF registers =============

	//--- CONTROL-1 register
#define HTPXDDC4_CNTR1_RG_DATA		((*(unsigned *)HTPXDDC4_CNTR1_RG) & HTPXDDC4_RG_DATA_BITMASK)
#define HTPXDDC4_SET_CNTR1_RG(d)	(*(unsigned *)HTPXDDC4_CNTR1_RG) = d

#define HTPXDDC4_MRESET_DATA		HTPXDDC4_CNTR1_RG_DATA
#define HTPXDDC4_SET_MRESET		HTPXDDC4_SET_CNTR1_RG(HTPXDDC4_MRESET_RESET)
#define HTPXDDC4_SET_MGO		HTPXDDC4_SET_CNTR1_RG(HTPXDDC4_MRESET_GO)


	//--- CONTROL-2 register
#define HTPXDDC4_CNTR2_RG_DATA		((*(unsigned *)HTPXDDC4_CNTR2_RG) & HTPXDDC4_RG_DATA_BITMASK)
#define HTPXDDC4_SET_CNTR2_RG(d)	(*(unsigned *)HTPXDDC4_CNTR2_RG) = d

#define HTPXDDC4_BMODE_DATA		(HTPXDDC4_CNTR2_RG_DATA & HTPXDDC4_BMODE_BITMASK)
#define HTPXDDC4_SET_BMODE_NONE		HTPXDDC4_SET_CNTR2_RG((HTPXDDC4_CNTR2_RG_DATA & ~HTPXDDC4_BMODE_BITMASK) | HTPXDDC4_BMODE_NONE)
#define HTPXDDC4_SET_BMODE_FLASH	HTPXDDC4_SET_CNTR2_RG((HTPXDDC4_CNTR2_RG_DATA & ~HTPXDDC4_BMODE_BITMASK) | HTPXDDC4_BMODE_FLASH)
#define HTPXDDC4_SET_BMODE_HPI		HTPXDDC4_SET_CNTR2_RG((HTPXDDC4_CNTR2_RG_DATA & ~HTPXDDC4_BMODE_BITMASK) | HTPXDDC4_BMODE_HPI)

#define HTPXDDC4_DPRAM_ERR_EN_DATA	(HTPXDDC4_CNTR2_RG_DATA & HTPXDDC4_DPRAM_ERR_EN_BITMASK)
#define HTPXDDC4_SET_DPRAM_ERR_EN_OFF	HTPXDDC4_SET_CNTR2_RG((HTPXDDC4_CNTR2_RG_DATA & ~HTPXDDC4_DPRAM_ERR_EN_BITMASK) | HTPXDDC4_DPRAM_ERR_EN_OFF)
#define HTPXDDC4_SET_DPRAM_ERR_EN_ON	HTPXDDC4_SET_CNTR2_RG((HTPXDDC4_CNTR2_RG_DATA & ~HTPXDDC4_DPRAM_ERR_EN_BITMASK) | HTPXDDC4_DPRAM_ERR_EN_ON)

#define HTPXDDC4_HPI_ERR_EN_DATA	(HTPXDDC4_CNTR2_RG_DATA & HTPXDDC4_HPI_ERR_EN_BITMASK)
#define HTPXDDC4_SET_HPI_ERR_EN_OFF	HTPXDDC4_SET_CNTR2_RG((HTPXDDC4_CNTR2_RG_DATA & ~HTPXDDC4_HPI_ERR_EN_BITMASK) | HTPXDDC4_HPI_ERR_EN_OFF)
#define HTPXDDC4_SET_HPI_ERR_EN_ON	HTPXDDC4_SET_CNTR2_RG((HTPXDDC4_CNTR2_RG_DATA & ~HTPXDDC4_HPI_ERR_EN_BITMASK) | HTPXDDC4_HPI_ERR_EN_ON)


	//-- IE Register
#define HTPXDDC4_IE_RG_DATA		((*(unsigned *)HTPXDDC4_IE_RG) & HTPXDDC4_RG_DATA_BITMASK)
#define HTPXDDC4_SET_IE_RG(d)		(*(unsigned *)HTPXDDC4_IE_RG) = d

#define HTPXDDC4_DPRAM_IE_DATA		(HTPXDDC4_IE_RG_DATA & HTPXDDC4_DPRAM_IE_BITMASK)
#define HTPXDDC4_SET_DPRAM_IE_OFF	HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_DPRAM_IE_BITMASK) | HTPXDDC4_DPRAM_IE_OFF)
#define HTPXDDC4_SET_DPRAM_IE_ON	HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_DPRAM_IE_BITMASK) | HTPXDDC4_DPRAM_IE_ON)

#define HTPXDDC4_HINT_IE_DATA		(HTPXDDC4_IE_RG_DATA & HTPXDDC4_HINT_IE_BITMASK)
#define HTPXDDC4_SET_HINT_IE_OFF	HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_HINT_IE_BITMASK) | HTPXDDC4_HINT_IE_OFF)
#define HTPXDDC4_SET_HINT_IE_ON		HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_HINT_IE_BITMASK) | HTPXDDC4_HINT_IE_ON)

#define HTPXDDC4_DPRAM_ERR_IE_DATA	(HTPXDDC4_IE_RG_DATA & HTPXDDC4_DPRAM_ERR_IE_BITMASK)
#define HTPXDDC4_SET_DPRAM_ERR_IE_OFF	HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_DPRAM_ERR_IE_BITMASK) | HTPXDDC4_DPRAM_ERR_IE_OFF)
#define HTPXDDC4_SET_DPRAM_ERR_IE_ON	HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_DPRAM_ERR_IE_BITMASK) | HTPXDDC4_DPRAM_ERR_IE_ON)

#define HTPXDDC4_HPI_ERR_IE_DATA	(HTPXDDC4_IE_RG_DATA & HTPXDDC4_HPI_ERR_IE_BITMASK)
#define HTPXDDC4_SET_HPI_ERR_IE_OFF	HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_DPRAM_ERR_IE_BITMASK) | HTPXDDC4_HPI_ERR_IE_OFF)
#define HTPXDDC4_SET_HPI_ERR_IE_ON	HTPXDDC4_SET_IE_RG((HTPXDDC4_IE_RG_DATA & ~HTPXDDC4_DPRAM_ERR_IE_BITMASK) | HTPXDDC4_HPI_ERR_IE_ON)


	//-- IS Register
#define HTPXDDC4_IS_RG_DATA		((*(unsigned *)HTPXDDC4_IS_RG) & HTPXDDC4_RG_DATA_BITMASK)
#define HTPXDDC4_DPRAM_IRQ_DATA		(HTPXDDC4_IS_RG_DATA & HTPXDDC4_DPRAM_IRQ_BITMASK)
#define HTPXDDC4_HINT_IRQ_DATA		(HTPXDDC4_IS_RG_DATA & HTPXDDC4_HINT_BITMASK)
#define HTPXDDC4_DPRAM_ERR_IRQ_DATA	(HTPXDDC4_IS_RG_DATA & HTPXDDC4_DPRAM_ERR_IRQ_BITMASK)
#define HTPXDDC4_HPI_ERR_IRQ_DATA	(HTPXDDC4_IS_RG_DATA & HTPXDDC4_HPI_ERR_IRQ_BITMASK)

#define HTPXDDC4_HINT_DATA	 	HTPXDDC4_HINT_IRQ_DATA	
#define HTPXDDC4_HPI_ERR_DATA	 	HTPXDDC4_HPI_ERR_IRQ_DATA
#define HTPXDDC4_DPRAM_ERR_DATA		HTPXDDC4_DPRAM_ERR_IRQ_DATA	


	//-- 'clear error flags' w-only registers
#define HTPXDDC4_CLR_HPI_ERR		(*(unsigned *)HTPXDDC4_CLR_HPI_ERR_RG) = 0
#define HTPXDDC4_CLR_DPRAM_ERR		(*(unsigned *)HTPXDDC4_CLR_DPRAM_ERR_RG) = 0


	//-- PIOX-IRQ Line Selector Register
#define HTPXDDC4_HIRQ_SEL_RG_DATA	((*(unsigned *)HTPXDDC4_HIRQ_SEL_RG) & HTPXDDC4_RG_DATA_BITMASK)
#define HTPXDDC4_SET_HIRQ_SEL_RG(d)	(*(unsigned *)HTPXDDC4_HIRQ_SEL_RG) = d

#define HTPXDDC4_HIRQ_EN_DATA		(HTPXDDC4_HIRQ_SEL_RG_DATA & HTPXDDC4_HIRQ_EN_BITMASK)
#define HTPXDDC4_SET_HIRQ_EN_OFF	HTPXDDC4_HIRQ_SEL_RG((HTPXDDC4_HIRQ_SEL_RG_DATA & ~HTPXDDC4_HIRQ_EN_BITMASK) | HTPXDDC4_HIRQ_EN_OFF)
#define HTPXDDC4_SET_HIRQ_EN_ON		HTPXDDC4_SET_HIRQ_SEL_RG((HTPXDDC4_HIRQ_SEL_RG_DATA & ~HTPXDDC4_HIRQ_EN_BITMASK) | HTPXDDC4_HIRQ_EN_ON)

#define HTPXDDC4_HIRQ_SEL_DATA		(HTPXDDC4_HIRQ_SEL_RG_DATA & HTPXDDC4_HIRQ_SEL_BITMASK)
#define HTPXDDC4_SET_HIRQ_SEL(d)	HTPXDDC4_SET_HIRQ_SEL_RG((HTPXDDC4_HIRQ_SEL_RG_DATA & ~HTPXDDC4_HIRQ_SEL_BITMASK) | (d & HTPXDDC4_HIRQ_SEL_BITMASK))

//=====================================================================



//========== macros for accessing DPRAM/DPSEM =========================

	//--- DPSEM
#define HTPXDDC4_DPSEM_DATA(n)		((*(unsigned *)(HTPXDDC4_DPSEM_SADDR + (n&HTPXDDC4_DPSEM_ADDR_BITMASK))) & HTPXDDC4_DPSEM_BITMASK)
#define HTPXDDC4_SET_DPSEM(n,d)		( *(unsigned *)(HTPXDDC4_DPSEM_SADDR + (n&HTPXDDC4_DPSEM_ADDR_BITMASK))) = d
#define HTPXDDC4_SET_DPSEM_ON(n)	( *(unsigned *)(HTPXDDC4_DPSEM_SADDR + (n&HTPXDDC4_DPSEM_ADDR_BITMASK))) = HTPXDDC4_DPSEM_ON
#define HTPXDDC4_SET_DPSEM_OFF(n)	( *(unsigned *)(HTPXDDC4_DPSEM_SADDR + (n&HTPXDDC4_DPSEM_ADDR_BITMASK))) = HTPXDDC4_DPSEM_OFF


	//--- DPRAM
#define HTPXDDC4_DPRAM_DATA(n)		((*(unsigned *)(HTPXDDC4_DPRAM_SADDR + (n&HTPXDDC4_DPRAM_ADDR_BITMASK))) & HTPXDDC4_DPRAM_DATA_BITMASK)
#define HTPXDDC4_SET_DPRAM(n,d)		( *(unsigned *)(HTPXDDC4_DPRAM_SADDR + (n&HTPXDDC4_DPRAM_ADDR_BITMASK))) = d

#define HTPXDDC4_DPRAM_HM_RQ_DATA	((*(unsigned *)HTPXDDC4_DPRAM_HM_RQ_ADDR) & HTPXDDC4_DPRAM_DATA_BITMASK)
#define HTPXDDC4_SET_DPRAM_HM_RQ(d)	( *(unsigned *)HTPXDDC4_DPRAM_HM_RQ_ADDR) = d

#define HTPXDDC4_DPRAM_MH_RQ_DATA	((*(unsigned *)HTPXDDC4_DPRAM_MH_RQ_ADDR) & HTPXDDC4_DPRAM_DATA_BITMASK)
#define HTPXDDC4_SET_DPRAM_MH_RQ(d)	( *(unsigned *)HTPXDDC4_DPRAM_MH_RQ_ADDR) = d

//=====================================================================


//===================== macros for HPI access =========================
// note!!!
//   HPIC read/write data is 8-bit data word (since HPC_LSB=HPIC_MSB)


unsigned HTPXDDC4_hpic_data()
{ unsigned lsb,msb;

  lsb = (*(unsigned *)HTPXDDC4_HPIC_RG_LSB) & HTPXDDC4_HPIC_READ_BITMASK;
  msb = (*(unsigned *)HTPXDDC4_HPIC_RG_MSB) & HTPXDDC4_HPIC_READ_BITMASK;
  return(lsb);
}

void HTPXDDC4_set_hpic(d)
{
  *(unsigned *)HTPXDDC4_HPIC_RG_LSB = (d & HTPXDDC4_HPIC_WRITE_BITMASK);
  *(unsigned *)HTPXDDC4_HPIC_RG_MSB = (d & HTPXDDC4_HPIC_WRITE_BITMASK);
}



unsigned HTPXDDC4_hpia_data()
{ unsigned lsb,msb;

  lsb = (*(unsigned *)HTPXDDC4_HPIA_RG_LSB) & HTPXDDC4_HPI_DATA_BITMASK;
  msb = (*(unsigned *)HTPXDDC4_HPIA_RG_MSB) & HTPXDDC4_HPI_DATA_BITMASK;
  return((msb<<8) | lsb);
}

void HTPXDDC4_set_hpia(d)
{
  *(unsigned *)HTPXDDC4_HPIA_RG_LSB = (d & HTPXDDC4_HPI_DATA_BITMASK);
  *(unsigned *)HTPXDDC4_HPIA_RG_MSB = ((d>>8) & HTPXDDC4_HPI_DATA_BITMASK);
}


unsigned HTPXDDC4_hpid_data()		
{ unsigned lsb,msb;

  lsb = (*(unsigned *)HTPXDDC4_HPID_RG_LSB) & HTPXDDC4_HPI_DATA_BITMASK;
  msb = (*(unsigned *)HTPXDDC4_HPID_RG_MSB) & HTPXDDC4_HPI_DATA_BITMASK;
  return((msb<<8) | lsb);
}

void HTPXDDC4_set_hpid(d)
{
  *(unsigned *)HTPXDDC4_HPID_RG_LSB = (d & HTPXDDC4_HPI_DATA_BITMASK);
  *(unsigned *)HTPXDDC4_HPID_RG_MSB = ((d>>8) & HTPXDDC4_HPI_DATA_BITMASK);
}



unsigned HTPXDDC4_hpid_ainc_data()		
{ unsigned lsb,msb;

  lsb = (*(unsigned *)HTPXDDC4_HPID_AINC_RG_LSB) & HTPXDDC4_HPI_DATA_BITMASK;
  msb = (*(unsigned *)HTPXDDC4_HPID_AINC_RG_MSB) & HTPXDDC4_HPI_DATA_BITMASK;
  return((msb<<8) | lsb);
}


void HTPXDDC4_set_hpid_ainc(d)
{
  *(unsigned *)HTPXDDC4_HPID_AINC_RG_LSB = (d & HTPXDDC4_HPI_DATA_BITMASK);
  *(unsigned *)HTPXDDC4_HPID_AINC_RG_MSB = ((d>>8) & HTPXDDC4_HPI_DATA_BITMASK);
}




#define HTPXDDC4_HPIC_DATA		HTPXDDC4_hpic_data()
#define HTPXDDC4_SET_HPIC(d)		HTPXDDC4_set_hpic(d)

#define HTPXDDC4_HPIA_DATA		HTPXDDC4_hpia_data()
#define HTPXDDC4_SET_HPIA(d)		HTPXDDC4_set_hpia(d)


#define HTPXDDC4_SET_HPIC_BOB		HTPXDDC4_SET_HPIC(HTPXDDC4_HPIC_BOB_BITMASK)
#define HTPXDDC4_HPIC_SMODE_DATA	(HTPXDDC4_HPIC_DATA & HTPXDDC4_HPIC_SMODE_BITMASK)
#define HTPXDDC4_HPIC_HINT_DATA		(HTPXDDC4_HPIC_DATA & HTPXDDC4_HPIC_HINT_BITMASK)
#define HTPXDDC4_SET_HPIC_DSPINT	HTPXDDC4_SET_HPIC((HTPXDDC4_HPIC_DATA & ~HTPXDDC4_HPIC_HINT_BITMASK) | HTPXDDC4_HPIC_DSPINT)
#define HTPXDDC4_CLR_HPIC_HINT		HTPXDDC4_SET_HPIC(HTPXDDC4_HPIC_DATA | HTPXDDC4_HPIC_HINT_BITMASK)
#define HTPXDDC4_HPIC_XHPIA_DATA	(HTPXDDC4_HPIC_DATA & HTPXDDC4_HPIC_XHPIA_BITMASK)
#define HTPXDDC4_SET_HPIC_XHPIA		HTPXDDC4_SET_HPIC((HTPXDDC4_HPIC_DATA & ~HTPXDDC4_HPIC_HINT_BITMASK) | HTPXDDC4_HPIC_XHPIA_BITMASK)
#define HTPXDDC4_CLR_HPIC_XHPIA		HTPXDDC4_SET_HPIC((HTPXDDC4_HPIC_DATA & ~HTPXDDC4_HPIC_HINT_BITMASK) & ~HTPXDDC4_HPIC_XHPIA_BITMASK)

#define HTPXDDC4_HPIA_RESET		HTPXDDC4_SET_HPIA(0)
#define HTPXDDC4_SET_XHPIA(d)		{HTPXDDC4_SET_HPIC_XHPIA;HTPXDDC4_SET_HPIA(((d&0x7f)<<8)|(d&0x7f));HTPXDDC4_CLR_HPIC_XHPIA;}
#define HTPXDDC4_XHPIA_RESET		HTPXDDC4_SET_XHPIA(0)

#define HTPXDDC4_HPID_DATA		HTPXDDC4_hpid_data()
#define HTPXDDC4_SET_HPID(d)		HTPXDDC4_set_hpid(d)
#define HTPXDDC4_HPID_AINC_DATA		HTPXDDC4_hpid_ainc_data()
#define HTPXDDC4_SET_HPID_AINC(d)	HTPXDDC4_set_hpid_ainc(d)

#define HTPXDDC4_HPI_INIT		{HTPXDDC4_SET_HPIC(HTPXDDC4_HPIC_HINT_BITMASK | HTPXDDC4_HPIC_BOB_BITMASK);HTPXDDC4_HPIA_RESET;HTPXDDC4_XHPIA_RESET;}



//=====================================================================

#endif	// end of DDC4 defs

